// This code conforms with the UFC specification version 2.0.5
// and was automatically generated by FFC version 1.0.0.
// 
// This code was generated with the following parameters:
// 
//   cache_dir:                      ''
//   convert_exceptions_to_warnings: False
//   cpp_optimize:                   False
//   cpp_optimize_flags:             '-O2'
//   epsilon:                        1e-14
//   error_control:                  False
//   form_postfix:                   True
//   format:                         'ufc'
//   log_level:                      20
//   log_prefix:                     ''
//   optimize:                       True
//   output_dir:                     '.'
//   precision:                      15
//   quadrature_degree:              'auto'
//   quadrature_rule:                'auto'
//   representation:                 'tensor'
//   split:                          False
//   swig_binary:                    'swig'
//   swig_path:                      ''

#ifndef __LAPLACIAN_F3_P1_Q3_TENSOR_H
#define __LAPLACIAN_F3_P1_Q3_TENSOR_H

#include <cmath>
#include <stdexcept>
#include <fstream>
#include <ufc.h>

/// This class defines the interface for a finite element.

class laplacian_f3_p1_q3_tensor_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  laplacian_f3_p1_q3_tensor_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~laplacian_f3_p1_q3_tensor_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "FiniteElement('Lagrange', Cell('triangle', Space(2)), 1, None)";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the topological dimension of the cell shape
  virtual unsigned int topological_dimension() const
  {
    return 2;
  }

  /// Return the geometric dimension of the cell shape
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 3;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = x[1][0] + x[2][0];
    const double C1 = x[1][1] + x[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (J_01*(C1 - 2.0*coordinates[1]) + J_11*(2.0*coordinates[0] - C0)) / detJ;
    double Y = (J_00*(2.0*coordinates[1] - C1) + J_10*(C0 - 2.0*coordinates[0])) / detJ;
    
    // Reset values.
    *values = 0.0;
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues.
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[3] = \
      {0.471404520791032, -0.288675134594813, -0.166666666666667};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 3; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 1:
      {
        
      // Array of basisvalues.
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[3] = \
      {0.471404520791032, 0.288675134594813, -0.166666666666667};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 3; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 2:
      {
        
      // Array of basisvalues.
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[3] = \
      {0.471404520791032, 0.0, 0.333333333333333};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 3; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    }
    
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    // Helper variable to hold values of a single dof.
    double dof_values = 0.0;
    
    // Loop dofs and call evaluate_basis.
    for (unsigned int r = 0; r < 3; r++)
    {
      evaluate_basis(r, &dof_values, coordinates, c);
      values[r] = dof_values;
    }// end loop over 'r'
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    const double K_00 =  J_11 / detJ;
    const double K_01 = -J_01 / detJ;
    const double K_10 = -J_10 / detJ;
    const double K_11 =  J_00 / detJ;
    
    // Compute constants
    const double C0 = x[1][0] + x[2][0];
    const double C1 = x[1][1] + x[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (J_01*(C1 - 2.0*coordinates[1]) + J_11*(2.0*coordinates[0] - C0)) / detJ;
    double Y = (J_00*(2.0*coordinates[1] - C1) + J_10*(C0 - 2.0*coordinates[0])) / detJ;
    
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    }// end loop over 'r'
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      combinations[row] = new unsigned int [n];
      for (unsigned int col = 0; col < n; col++)
        combinations[row][col] = 0;
    }
    
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] = {{K_00, K_01}, {K_10, K_11}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values. Assuming that values is always an array.
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      values[r] = 0.0;
    }// end loop over 'r'
    
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues.
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[3] = \
      {0.471404520791032, -0.288675134594813, -0.166666666666667};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[3][3] = \
      {{0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0},
      {0.0, 0.0, 0.0}};
      
      static const double dmats1[3][3] = \
      {{0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 3; t++)
        {
          for (unsigned int u = 0; u < 3; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 3; s++)
        {
          for (unsigned int t = 0; t < 3; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 1:
      {
        
      // Array of basisvalues.
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[3] = \
      {0.471404520791032, 0.288675134594813, -0.166666666666667};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[3][3] = \
      {{0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0},
      {0.0, 0.0, 0.0}};
      
      static const double dmats1[3][3] = \
      {{0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 3; t++)
        {
          for (unsigned int u = 0; u < 3; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 3; s++)
        {
          for (unsigned int t = 0; t < 3; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 2:
      {
        
      // Array of basisvalues.
      double basisvalues[3] = {0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[1] *= std::sqrt(3.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[3] = \
      {0.471404520791032, 0.0, 0.333333333333333};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[3][3] = \
      {{0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0},
      {0.0, 0.0, 0.0}};
      
      static const double dmats1[3][3] = \
      {{0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[3][3] = \
      {{1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 3; t++)
        {
          for (unsigned int u = 0; u < 3; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 3; t++)
          {
            for (unsigned int u = 0; u < 3; u++)
            {
              for (unsigned int tu = 0; tu < 3; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 3; s++)
        {
          for (unsigned int t = 0; t < 3; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    }
    
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    }// end loop over 'r'
    
    // Helper variable to hold values of a single dof.
    double *dof_values = new double[num_derivatives];
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      dof_values[r] = 0.0;
    }// end loop over 'r'
    
    // Loop dofs and call evaluate_basis_derivatives.
    for (unsigned int r = 0; r < 3; r++)
    {
      evaluate_basis_derivatives(r, n, dof_values, coordinates, c);
      for (unsigned int s = 0; s < num_derivatives; s++)
      {
        values[r*num_derivatives + s] = dof_values[s];
      }// end loop over 's'
    }// end loop over 'r'
    
    // Delete pointer.
    delete [] dof_values;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // Declare variables for result of evaluation.
    double vals[1];
    
    // Declare variable for physical coordinates.
    double y[2];
    const double * const * x = c.coordinates;
    switch (i)
    {
    case 0:
      {
        y[0] = x[0][0];
      y[1] = x[0][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 1:
      {
        y[0] = x[1][0];
      y[1] = x[1][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 2:
      {
        y[0] = x[2][0];
      y[1] = x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    }
    
    return 0.0;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    // Declare variables for result of evaluation.
    double vals[1];
    
    // Declare variable for physical coordinates.
    double y[2];
    const double * const * x = c.coordinates;
    y[0] = x[0][0];
    y[1] = x[0][1];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
    y[0] = x[1][0];
    y[1] = x[1][1];
    f.evaluate(vals, y, c);
    values[1] = vals[0];
    y[0] = x[2][0];
    y[1] = x[2][1];
    f.evaluate(vals, y, c);
    values[2] = vals[0];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Map coordinate xhat from reference cell to coordinate x in cell
  virtual void map_from_reference_cell(double* x,
                                       const double* xhat,
                                       const ufc::cell& c) const
  {
    throw std::runtime_error("map_from_reference_cell not yet implemented (introduced in UFC 2.0).");
  }

  /// Map from coordinate x in cell to coordinate xhat in reference cell
  virtual void map_to_reference_cell(double* xhat,
                                     const double* x,
                                     const ufc::cell& c) const
  {
    throw std::runtime_error("map_to_reference_cell not yet implemented (introduced in UFC 2.0).");
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 0;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return 0;
  }

  /// Create a new class instance
  virtual ufc::finite_element* create() const
  {
    return new laplacian_f3_p1_q3_tensor_finite_element_0();
  }

};

/// This class defines the interface for a finite element.

class laplacian_f3_p1_q3_tensor_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  laplacian_f3_p1_q3_tensor_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~laplacian_f3_p1_q3_tensor_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "FiniteElement('Lagrange', Cell('triangle', Space(2)), 3, None)";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the topological dimension of the cell shape
  virtual unsigned int topological_dimension() const
  {
    return 2;
  }

  /// Return the geometric dimension of the cell shape
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 10;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = x[1][0] + x[2][0];
    const double C1 = x[1][1] + x[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (J_01*(C1 - 2.0*coordinates[1]) + J_11*(2.0*coordinates[0] - C0)) / detJ;
    double Y = (J_00*(2.0*coordinates[1] - C1) + J_10*(C0 - 2.0*coordinates[0])) / detJ;
    
    // Reset values.
    *values = 0.0;
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.0471404520791033, -0.0288675134594813, -0.0166666666666666, 0.0782460796435952, 0.0606091526731326, 0.0349927106111883, -0.0601337794302955, -0.0508223195384204, -0.0393667994375868, -0.0227284322524248};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 10; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 1:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.0471404520791032, 0.0288675134594813, -0.0166666666666666, 0.0782460796435952, -0.0606091526731327, 0.0349927106111883, 0.0601337794302955, -0.0508223195384204, 0.0393667994375868, -0.0227284322524248};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 10; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 2:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.0471404520791032, 0.0, 0.0333333333333334, 0.0, 0.0, 0.104978131833565, 0.0, 0.0, 0.0, 0.0909137290096989};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 10; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 3:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.106066017177982, 0.259807621135332, -0.15, 0.117369119465393, 0.0606091526731326, -0.0787335988751736, 0.0, 0.101644639076841, -0.131222664791956, 0.090913729009699};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 10; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 4:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.106066017177982, 0.0, 0.3, 0.0, 0.151522881682832, 0.0262445329583912, 0.0, 0.0, 0.131222664791956, -0.136370593514548};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 10; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 5:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.106066017177982, -0.259807621135332, -0.15, 0.117369119465393, -0.0606091526731326, -0.0787335988751736, 0.0, 0.101644639076841, 0.131222664791956, 0.090913729009699};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 10; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 6:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.106066017177982, 0.0, 0.3, 0.0, -0.151522881682832, 0.0262445329583912, 0.0, 0.0, -0.131222664791956, -0.136370593514548};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 10; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 7:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.106066017177982, -0.259807621135332, -0.15, -0.0782460796435952, 0.090913729009699, 0.0962299541807677, 0.180401338290886, 0.0508223195384204, -0.0131222664791956, -0.0227284322524247};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 10; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 8:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.106066017177982, 0.259807621135332, -0.15, -0.0782460796435952, -0.090913729009699, 0.0962299541807678, -0.180401338290886, 0.0508223195384204, 0.0131222664791956, -0.0227284322524248};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 10; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 9:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.636396103067893, 0.0, 0.0, -0.234738238930785, 0.0, -0.262445329583912, 0.0, -0.203289278153682, 0.0, 0.090913729009699};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 10; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    }
    
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    // Helper variable to hold values of a single dof.
    double dof_values = 0.0;
    
    // Loop dofs and call evaluate_basis.
    for (unsigned int r = 0; r < 10; r++)
    {
      evaluate_basis(r, &dof_values, coordinates, c);
      values[r] = dof_values;
    }// end loop over 'r'
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    const double K_00 =  J_11 / detJ;
    const double K_01 = -J_01 / detJ;
    const double K_10 = -J_10 / detJ;
    const double K_11 =  J_00 / detJ;
    
    // Compute constants
    const double C0 = x[1][0] + x[2][0];
    const double C1 = x[1][1] + x[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (J_01*(C1 - 2.0*coordinates[1]) + J_11*(2.0*coordinates[0] - C0)) / detJ;
    double Y = (J_00*(2.0*coordinates[1] - C1) + J_10*(C0 - 2.0*coordinates[0])) / detJ;
    
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    }// end loop over 'r'
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      combinations[row] = new unsigned int [n];
      for (unsigned int col = 0; col < n; col++)
        combinations[row][col] = 0;
    }
    
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] = {{K_00, K_01}, {K_10, K_11}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values. Assuming that values is always an array.
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      values[r] = 0.0;
    }// end loop over 'r'
    
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.0471404520791033, -0.0288675134594813, -0.0166666666666666, 0.0782460796435952, 0.0606091526731326, 0.0349927106111883, -0.0601337794302955, -0.0508223195384204, -0.0393667994375868, -0.0227284322524248};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.48683298050514, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.0, 0.0, 7.07106781186548, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.29150262212918, 0.0, -2.99332590941915, 13.6626010212795, 0.0, 0.611010092660779, 0.0, 0.0, 0.0, 0.0},
      {0.0, 4.38178046004133, 0.0, 0.0, 12.5219806739988, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.46410161513776, 0.0, 7.83836717690617, 0.0, 0.0, 8.4, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.58198889747161, 4.74341649025257, -0.912870929175276, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2, 6.12372435695795, 3.53553390593274, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.3094010767585, 0.0, 8.16496580927726, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.64575131106459, 5.18459255872629, -1.49666295470957, 6.83130051063973, -1.05830052442584, 0.305505046330389, 0.0, 0.0, 0.0, 0.0},
      {2.23606797749979, 2.19089023002067, 2.52982212813471, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0.0, 0.0, 0.0, 0.0},
      {1.73205080756888, -5.09116882454314, 3.91918358845309, 0.0, 9.69948452238571, 4.2, 0.0, 0.0, 0.0, 0.0},
      {5, 0.0, -2.82842712474619, 0.0, 0.0, 12.1243556529821, 0.0, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 10; t++)
        {
          for (unsigned int u = 0; u < 10; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 10; s++)
        {
          for (unsigned int t = 0; t < 10; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 1:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.0471404520791032, 0.0288675134594813, -0.0166666666666666, 0.0782460796435952, -0.0606091526731327, 0.0349927106111883, 0.0601337794302955, -0.0508223195384204, 0.0393667994375868, -0.0227284322524248};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.48683298050514, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.0, 0.0, 7.07106781186548, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.29150262212918, 0.0, -2.99332590941915, 13.6626010212795, 0.0, 0.611010092660779, 0.0, 0.0, 0.0, 0.0},
      {0.0, 4.38178046004133, 0.0, 0.0, 12.5219806739988, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.46410161513776, 0.0, 7.83836717690617, 0.0, 0.0, 8.4, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.58198889747161, 4.74341649025257, -0.912870929175276, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2, 6.12372435695795, 3.53553390593274, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.3094010767585, 0.0, 8.16496580927726, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.64575131106459, 5.18459255872629, -1.49666295470957, 6.83130051063973, -1.05830052442584, 0.305505046330389, 0.0, 0.0, 0.0, 0.0},
      {2.23606797749979, 2.19089023002067, 2.52982212813471, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0.0, 0.0, 0.0, 0.0},
      {1.73205080756888, -5.09116882454314, 3.91918358845309, 0.0, 9.69948452238571, 4.2, 0.0, 0.0, 0.0, 0.0},
      {5, 0.0, -2.82842712474619, 0.0, 0.0, 12.1243556529821, 0.0, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 10; t++)
        {
          for (unsigned int u = 0; u < 10; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 10; s++)
        {
          for (unsigned int t = 0; t < 10; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 2:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.0471404520791032, 0.0, 0.0333333333333334, 0.0, 0.0, 0.104978131833565, 0.0, 0.0, 0.0, 0.0909137290096989};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.48683298050514, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.0, 0.0, 7.07106781186548, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.29150262212918, 0.0, -2.99332590941915, 13.6626010212795, 0.0, 0.611010092660779, 0.0, 0.0, 0.0, 0.0},
      {0.0, 4.38178046004133, 0.0, 0.0, 12.5219806739988, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.46410161513776, 0.0, 7.83836717690617, 0.0, 0.0, 8.4, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.58198889747161, 4.74341649025257, -0.912870929175276, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2, 6.12372435695795, 3.53553390593274, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.3094010767585, 0.0, 8.16496580927726, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.64575131106459, 5.18459255872629, -1.49666295470957, 6.83130051063973, -1.05830052442584, 0.305505046330389, 0.0, 0.0, 0.0, 0.0},
      {2.23606797749979, 2.19089023002067, 2.52982212813471, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0.0, 0.0, 0.0, 0.0},
      {1.73205080756888, -5.09116882454314, 3.91918358845309, 0.0, 9.69948452238571, 4.2, 0.0, 0.0, 0.0, 0.0},
      {5, 0.0, -2.82842712474619, 0.0, 0.0, 12.1243556529821, 0.0, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 10; t++)
        {
          for (unsigned int u = 0; u < 10; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 10; s++)
        {
          for (unsigned int t = 0; t < 10; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 3:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.106066017177982, 0.259807621135332, -0.15, 0.117369119465393, 0.0606091526731326, -0.0787335988751736, 0.0, 0.101644639076841, -0.131222664791956, 0.090913729009699};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.48683298050514, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.0, 0.0, 7.07106781186548, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.29150262212918, 0.0, -2.99332590941915, 13.6626010212795, 0.0, 0.611010092660779, 0.0, 0.0, 0.0, 0.0},
      {0.0, 4.38178046004133, 0.0, 0.0, 12.5219806739988, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.46410161513776, 0.0, 7.83836717690617, 0.0, 0.0, 8.4, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.58198889747161, 4.74341649025257, -0.912870929175276, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2, 6.12372435695795, 3.53553390593274, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.3094010767585, 0.0, 8.16496580927726, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.64575131106459, 5.18459255872629, -1.49666295470957, 6.83130051063973, -1.05830052442584, 0.305505046330389, 0.0, 0.0, 0.0, 0.0},
      {2.23606797749979, 2.19089023002067, 2.52982212813471, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0.0, 0.0, 0.0, 0.0},
      {1.73205080756888, -5.09116882454314, 3.91918358845309, 0.0, 9.69948452238571, 4.2, 0.0, 0.0, 0.0, 0.0},
      {5, 0.0, -2.82842712474619, 0.0, 0.0, 12.1243556529821, 0.0, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 10; t++)
        {
          for (unsigned int u = 0; u < 10; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 10; s++)
        {
          for (unsigned int t = 0; t < 10; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 4:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.106066017177982, 0.0, 0.3, 0.0, 0.151522881682832, 0.0262445329583912, 0.0, 0.0, 0.131222664791956, -0.136370593514548};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.48683298050514, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.0, 0.0, 7.07106781186548, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.29150262212918, 0.0, -2.99332590941915, 13.6626010212795, 0.0, 0.611010092660779, 0.0, 0.0, 0.0, 0.0},
      {0.0, 4.38178046004133, 0.0, 0.0, 12.5219806739988, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.46410161513776, 0.0, 7.83836717690617, 0.0, 0.0, 8.4, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.58198889747161, 4.74341649025257, -0.912870929175276, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2, 6.12372435695795, 3.53553390593274, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.3094010767585, 0.0, 8.16496580927726, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.64575131106459, 5.18459255872629, -1.49666295470957, 6.83130051063973, -1.05830052442584, 0.305505046330389, 0.0, 0.0, 0.0, 0.0},
      {2.23606797749979, 2.19089023002067, 2.52982212813471, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0.0, 0.0, 0.0, 0.0},
      {1.73205080756888, -5.09116882454314, 3.91918358845309, 0.0, 9.69948452238571, 4.2, 0.0, 0.0, 0.0, 0.0},
      {5, 0.0, -2.82842712474619, 0.0, 0.0, 12.1243556529821, 0.0, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 10; t++)
        {
          for (unsigned int u = 0; u < 10; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 10; s++)
        {
          for (unsigned int t = 0; t < 10; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 5:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.106066017177982, -0.259807621135332, -0.15, 0.117369119465393, -0.0606091526731326, -0.0787335988751736, 0.0, 0.101644639076841, 0.131222664791956, 0.090913729009699};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.48683298050514, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.0, 0.0, 7.07106781186548, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.29150262212918, 0.0, -2.99332590941915, 13.6626010212795, 0.0, 0.611010092660779, 0.0, 0.0, 0.0, 0.0},
      {0.0, 4.38178046004133, 0.0, 0.0, 12.5219806739988, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.46410161513776, 0.0, 7.83836717690617, 0.0, 0.0, 8.4, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.58198889747161, 4.74341649025257, -0.912870929175276, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2, 6.12372435695795, 3.53553390593274, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.3094010767585, 0.0, 8.16496580927726, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.64575131106459, 5.18459255872629, -1.49666295470957, 6.83130051063973, -1.05830052442584, 0.305505046330389, 0.0, 0.0, 0.0, 0.0},
      {2.23606797749979, 2.19089023002067, 2.52982212813471, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0.0, 0.0, 0.0, 0.0},
      {1.73205080756888, -5.09116882454314, 3.91918358845309, 0.0, 9.69948452238571, 4.2, 0.0, 0.0, 0.0, 0.0},
      {5, 0.0, -2.82842712474619, 0.0, 0.0, 12.1243556529821, 0.0, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 10; t++)
        {
          for (unsigned int u = 0; u < 10; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 10; s++)
        {
          for (unsigned int t = 0; t < 10; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 6:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.106066017177982, 0.0, 0.3, 0.0, -0.151522881682832, 0.0262445329583912, 0.0, 0.0, -0.131222664791956, -0.136370593514548};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.48683298050514, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.0, 0.0, 7.07106781186548, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.29150262212918, 0.0, -2.99332590941915, 13.6626010212795, 0.0, 0.611010092660779, 0.0, 0.0, 0.0, 0.0},
      {0.0, 4.38178046004133, 0.0, 0.0, 12.5219806739988, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.46410161513776, 0.0, 7.83836717690617, 0.0, 0.0, 8.4, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.58198889747161, 4.74341649025257, -0.912870929175276, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2, 6.12372435695795, 3.53553390593274, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.3094010767585, 0.0, 8.16496580927726, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.64575131106459, 5.18459255872629, -1.49666295470957, 6.83130051063973, -1.05830052442584, 0.305505046330389, 0.0, 0.0, 0.0, 0.0},
      {2.23606797749979, 2.19089023002067, 2.52982212813471, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0.0, 0.0, 0.0, 0.0},
      {1.73205080756888, -5.09116882454314, 3.91918358845309, 0.0, 9.69948452238571, 4.2, 0.0, 0.0, 0.0, 0.0},
      {5, 0.0, -2.82842712474619, 0.0, 0.0, 12.1243556529821, 0.0, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 10; t++)
        {
          for (unsigned int u = 0; u < 10; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 10; s++)
        {
          for (unsigned int t = 0; t < 10; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 7:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.106066017177982, -0.259807621135332, -0.15, -0.0782460796435952, 0.090913729009699, 0.0962299541807677, 0.180401338290886, 0.0508223195384204, -0.0131222664791956, -0.0227284322524247};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.48683298050514, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.0, 0.0, 7.07106781186548, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.29150262212918, 0.0, -2.99332590941915, 13.6626010212795, 0.0, 0.611010092660779, 0.0, 0.0, 0.0, 0.0},
      {0.0, 4.38178046004133, 0.0, 0.0, 12.5219806739988, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.46410161513776, 0.0, 7.83836717690617, 0.0, 0.0, 8.4, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.58198889747161, 4.74341649025257, -0.912870929175276, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2, 6.12372435695795, 3.53553390593274, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.3094010767585, 0.0, 8.16496580927726, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.64575131106459, 5.18459255872629, -1.49666295470957, 6.83130051063973, -1.05830052442584, 0.305505046330389, 0.0, 0.0, 0.0, 0.0},
      {2.23606797749979, 2.19089023002067, 2.52982212813471, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0.0, 0.0, 0.0, 0.0},
      {1.73205080756888, -5.09116882454314, 3.91918358845309, 0.0, 9.69948452238571, 4.2, 0.0, 0.0, 0.0, 0.0},
      {5, 0.0, -2.82842712474619, 0.0, 0.0, 12.1243556529821, 0.0, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 10; t++)
        {
          for (unsigned int u = 0; u < 10; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 10; s++)
        {
          for (unsigned int t = 0; t < 10; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 8:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.106066017177982, 0.259807621135332, -0.15, -0.0782460796435952, -0.090913729009699, 0.0962299541807678, -0.180401338290886, 0.0508223195384204, 0.0131222664791956, -0.0227284322524248};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.48683298050514, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.0, 0.0, 7.07106781186548, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.29150262212918, 0.0, -2.99332590941915, 13.6626010212795, 0.0, 0.611010092660779, 0.0, 0.0, 0.0, 0.0},
      {0.0, 4.38178046004133, 0.0, 0.0, 12.5219806739988, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.46410161513776, 0.0, 7.83836717690617, 0.0, 0.0, 8.4, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.58198889747161, 4.74341649025257, -0.912870929175276, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2, 6.12372435695795, 3.53553390593274, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.3094010767585, 0.0, 8.16496580927726, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.64575131106459, 5.18459255872629, -1.49666295470957, 6.83130051063973, -1.05830052442584, 0.305505046330389, 0.0, 0.0, 0.0, 0.0},
      {2.23606797749979, 2.19089023002067, 2.52982212813471, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0.0, 0.0, 0.0, 0.0},
      {1.73205080756888, -5.09116882454314, 3.91918358845309, 0.0, 9.69948452238571, 4.2, 0.0, 0.0, 0.0, 0.0},
      {5, 0.0, -2.82842712474619, 0.0, 0.0, 12.1243556529821, 0.0, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 10; t++)
        {
          for (unsigned int u = 0; u < 10; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 10; s++)
        {
          for (unsigned int t = 0; t < 10; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 9:
      {
        
      // Array of basisvalues.
      double basisvalues[10] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      double tmp0 = (1.0 + Y + 2.0*X)/2.0;
      double tmp1 = (1.0 - Y)/2.0;
      double tmp2 = tmp1*tmp1;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      basisvalues[3] = basisvalues[1]*1.5*tmp0 - basisvalues[0]*0.5*tmp2;
      basisvalues[6] = basisvalues[3]*1.66666666666667*tmp0 - basisvalues[1]*0.666666666666667*tmp2;
      basisvalues[2] = basisvalues[0]*(0.5 + 1.5*Y);
      basisvalues[4] = basisvalues[1]*(1.5 + 2.5*Y);
      basisvalues[7] = basisvalues[3]*(2.5 + 3.5*Y);
      basisvalues[5] = basisvalues[2]*(0.111111111111111 + Y*1.66666666666667) - basisvalues[0]*0.555555555555556;
      basisvalues[9] = basisvalues[5]*(0.05 + Y*1.75) - basisvalues[2]*0.7;
      basisvalues[8] = basisvalues[4]*(0.54 + Y*2.1) - basisvalues[1]*0.56;
      basisvalues[0] *= std::sqrt(0.5);
      basisvalues[2] *= std::sqrt(1.0);
      basisvalues[5] *= std::sqrt(1.5);
      basisvalues[9] *= std::sqrt(2.0);
      basisvalues[1] *= std::sqrt(3.0);
      basisvalues[4] *= std::sqrt(4.5);
      basisvalues[8] *= std::sqrt(6.0);
      basisvalues[3] *= std::sqrt(7.5);
      basisvalues[7] *= std::sqrt(10.0);
      basisvalues[6] *= std::sqrt(14.0);
      
      // Table(s) of coefficients.
      static const double coefficients0[10] = \
      {0.636396103067893, 0.0, 0.0, -0.234738238930785, 0.0, -0.262445329583912, 0.0, -0.203289278153682, 0.0, 0.090913729009699};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.89897948556636, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 9.48683298050514, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.0, 0.0, 7.07106781186548, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.29150262212918, 0.0, -2.99332590941915, 13.6626010212795, 0.0, 0.611010092660779, 0.0, 0.0, 0.0, 0.0},
      {0.0, 4.38178046004133, 0.0, 0.0, 12.5219806739988, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.46410161513776, 0.0, 7.83836717690617, 0.0, 0.0, 8.4, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[10][10] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.44948974278318, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.24264068711928, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.58198889747161, 4.74341649025257, -0.912870929175276, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2, 6.12372435695795, 3.53553390593274, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.3094010767585, 0.0, 8.16496580927726, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.64575131106459, 5.18459255872629, -1.49666295470957, 6.83130051063973, -1.05830052442584, 0.305505046330389, 0.0, 0.0, 0.0, 0.0},
      {2.23606797749979, 2.19089023002067, 2.52982212813471, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0.0, 0.0, 0.0, 0.0},
      {1.73205080756888, -5.09116882454314, 3.91918358845309, 0.0, 9.69948452238571, 4.2, 0.0, 0.0, 0.0, 0.0},
      {5, 0.0, -2.82842712474619, 0.0, 0.0, 12.1243556529821, 0.0, 0.0, 0.0, 0.0}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[10][10] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 10; t++)
        {
          for (unsigned int u = 0; u < 10; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 10; t++)
          {
            for (unsigned int u = 0; u < 10; u++)
            {
              for (unsigned int tu = 0; tu < 10; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 10; s++)
        {
          for (unsigned int t = 0; t < 10; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    }
    
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    }// end loop over 'r'
    
    // Helper variable to hold values of a single dof.
    double *dof_values = new double[num_derivatives];
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      dof_values[r] = 0.0;
    }// end loop over 'r'
    
    // Loop dofs and call evaluate_basis_derivatives.
    for (unsigned int r = 0; r < 10; r++)
    {
      evaluate_basis_derivatives(r, n, dof_values, coordinates, c);
      for (unsigned int s = 0; s < num_derivatives; s++)
      {
        values[r*num_derivatives + s] = dof_values[s];
      }// end loop over 's'
    }// end loop over 'r'
    
    // Delete pointer.
    delete [] dof_values;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // Declare variables for result of evaluation.
    double vals[1];
    
    // Declare variable for physical coordinates.
    double y[2];
    const double * const * x = c.coordinates;
    switch (i)
    {
    case 0:
      {
        y[0] = x[0][0];
      y[1] = x[0][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 1:
      {
        y[0] = x[1][0];
      y[1] = x[1][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 2:
      {
        y[0] = x[2][0];
      y[1] = x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 3:
      {
        y[0] = 0.666666666666667*x[1][0] + 0.333333333333333*x[2][0];
      y[1] = 0.666666666666667*x[1][1] + 0.333333333333333*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 4:
      {
        y[0] = 0.333333333333333*x[1][0] + 0.666666666666667*x[2][0];
      y[1] = 0.333333333333333*x[1][1] + 0.666666666666667*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 5:
      {
        y[0] = 0.666666666666667*x[0][0] + 0.333333333333333*x[2][0];
      y[1] = 0.666666666666667*x[0][1] + 0.333333333333333*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 6:
      {
        y[0] = 0.333333333333333*x[0][0] + 0.666666666666667*x[2][0];
      y[1] = 0.333333333333333*x[0][1] + 0.666666666666667*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 7:
      {
        y[0] = 0.666666666666667*x[0][0] + 0.333333333333333*x[1][0];
      y[1] = 0.666666666666667*x[0][1] + 0.333333333333333*x[1][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 8:
      {
        y[0] = 0.333333333333333*x[0][0] + 0.666666666666667*x[1][0];
      y[1] = 0.333333333333333*x[0][1] + 0.666666666666667*x[1][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 9:
      {
        y[0] = 0.333333333333333*x[0][0] + 0.333333333333333*x[1][0] + 0.333333333333333*x[2][0];
      y[1] = 0.333333333333333*x[0][1] + 0.333333333333333*x[1][1] + 0.333333333333333*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    }
    
    return 0.0;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    // Declare variables for result of evaluation.
    double vals[1];
    
    // Declare variable for physical coordinates.
    double y[2];
    const double * const * x = c.coordinates;
    y[0] = x[0][0];
    y[1] = x[0][1];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
    y[0] = x[1][0];
    y[1] = x[1][1];
    f.evaluate(vals, y, c);
    values[1] = vals[0];
    y[0] = x[2][0];
    y[1] = x[2][1];
    f.evaluate(vals, y, c);
    values[2] = vals[0];
    y[0] = 0.666666666666667*x[1][0] + 0.333333333333333*x[2][0];
    y[1] = 0.666666666666667*x[1][1] + 0.333333333333333*x[2][1];
    f.evaluate(vals, y, c);
    values[3] = vals[0];
    y[0] = 0.333333333333333*x[1][0] + 0.666666666666667*x[2][0];
    y[1] = 0.333333333333333*x[1][1] + 0.666666666666667*x[2][1];
    f.evaluate(vals, y, c);
    values[4] = vals[0];
    y[0] = 0.666666666666667*x[0][0] + 0.333333333333333*x[2][0];
    y[1] = 0.666666666666667*x[0][1] + 0.333333333333333*x[2][1];
    f.evaluate(vals, y, c);
    values[5] = vals[0];
    y[0] = 0.333333333333333*x[0][0] + 0.666666666666667*x[2][0];
    y[1] = 0.333333333333333*x[0][1] + 0.666666666666667*x[2][1];
    f.evaluate(vals, y, c);
    values[6] = vals[0];
    y[0] = 0.666666666666667*x[0][0] + 0.333333333333333*x[1][0];
    y[1] = 0.666666666666667*x[0][1] + 0.333333333333333*x[1][1];
    f.evaluate(vals, y, c);
    values[7] = vals[0];
    y[0] = 0.333333333333333*x[0][0] + 0.666666666666667*x[1][0];
    y[1] = 0.333333333333333*x[0][1] + 0.666666666666667*x[1][1];
    f.evaluate(vals, y, c);
    values[8] = vals[0];
    y[0] = 0.333333333333333*x[0][0] + 0.333333333333333*x[1][0] + 0.333333333333333*x[2][0];
    y[1] = 0.333333333333333*x[0][1] + 0.333333333333333*x[1][1] + 0.333333333333333*x[2][1];
    f.evaluate(vals, y, c);
    values[9] = vals[0];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Map coordinate xhat from reference cell to coordinate x in cell
  virtual void map_from_reference_cell(double* x,
                                       const double* xhat,
                                       const ufc::cell& c) const
  {
    throw std::runtime_error("map_from_reference_cell not yet implemented (introduced in UFC 2.0).");
  }

  /// Map from coordinate x in cell to coordinate xhat in reference cell
  virtual void map_to_reference_cell(double* xhat,
                                     const double* x,
                                     const ufc::cell& c) const
  {
    throw std::runtime_error("map_to_reference_cell not yet implemented (introduced in UFC 2.0).");
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 0;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return 0;
  }

  /// Create a new class instance
  virtual ufc::finite_element* create() const
  {
    return new laplacian_f3_p1_q3_tensor_finite_element_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class laplacian_f3_p1_q3_tensor_dofmap_0: public ufc::dofmap
{
private:

  unsigned int _global_dimension;
public:

  /// Constructor
  laplacian_f3_p1_q3_tensor_dofmap_0() : ufc::dofmap()
  {
    _global_dimension = 0;
  }

  /// Destructor
  virtual ~laplacian_f3_p1_q3_tensor_dofmap_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dofmap
  virtual const char* signature() const
  {
    return "FFC dofmap for FiniteElement('Lagrange', Cell('triangle', Space(2)), 1, None)";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch (d)
    {
    case 0:
      {
        return true;
        break;
      }
    case 1:
      {
        return false;
        break;
      }
    case 2:
      {
        return false;
        break;
      }
    }
    
    return false;
  }

  /// Initialize dofmap for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    _global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dofmap for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dofmap for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the topological dimension of the associated cell shape
  virtual unsigned int topological_dimension() const
  {
    return 2;
  }

  /// Return the geometric dimension of the associated cell shape
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return _global_dimension;
  }

  /// Return the dimension of the local finite element function space for a cell
  virtual unsigned int local_dimension(const ufc::cell& c) const
  {
    return 3;
  }

  /// Return the maximum dimension of the local finite element function space
  virtual unsigned int max_local_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 2;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    switch (d)
    {
    case 0:
      {
        return 1;
        break;
      }
    case 1:
      {
        return 0;
        break;
      }
    case 2:
      {
        return 0;
        break;
      }
    }
    
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch (facet)
    {
    case 0:
      {
        dofs[0] = 1;
      dofs[1] = 2;
        break;
      }
    case 1:
      {
        dofs[0] = 0;
      dofs[1] = 2;
        break;
      }
    case 2:
      {
        dofs[0] = 0;
      dofs[1] = 1;
        break;
      }
    }
    
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    if (d > 2)
    {
    throw std::runtime_error("d is larger than dimension (2)");
    }
    
    switch (d)
    {
    case 0:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 0;
          break;
        }
      case 1:
        {
          dofs[0] = 1;
          break;
        }
      case 2:
        {
          dofs[0] = 2;
          break;
        }
      }
      
        break;
      }
    case 1:
      {
        
        break;
      }
    case 2:
      {
        
        break;
      }
    }
    
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
  }

  /// Return the number of sub dofmaps (for a mixed element)
  virtual unsigned int num_sub_dofmaps() const
  {
    return 0;
  }

  /// Create a new dofmap for sub dofmap i (for a mixed element)
  virtual ufc::dofmap* create_sub_dofmap(unsigned int i) const
  {
    return 0;
  }

  /// Create a new class instance
  virtual ufc::dofmap* create() const
  {
    return new laplacian_f3_p1_q3_tensor_dofmap_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class laplacian_f3_p1_q3_tensor_dofmap_1: public ufc::dofmap
{
private:

  unsigned int _global_dimension;
public:

  /// Constructor
  laplacian_f3_p1_q3_tensor_dofmap_1() : ufc::dofmap()
  {
    _global_dimension = 0;
  }

  /// Destructor
  virtual ~laplacian_f3_p1_q3_tensor_dofmap_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dofmap
  virtual const char* signature() const
  {
    return "FFC dofmap for FiniteElement('Lagrange', Cell('triangle', Space(2)), 3, None)";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch (d)
    {
    case 0:
      {
        return true;
        break;
      }
    case 1:
      {
        return true;
        break;
      }
    case 2:
      {
        return true;
        break;
      }
    }
    
    return false;
  }

  /// Initialize dofmap for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    _global_dimension = m.num_entities[0] + 2*m.num_entities[1] + m.num_entities[2];
    return false;
  }

  /// Initialize dofmap for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dofmap for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the topological dimension of the associated cell shape
  virtual unsigned int topological_dimension() const
  {
    return 2;
  }

  /// Return the geometric dimension of the associated cell shape
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return _global_dimension;
  }

  /// Return the dimension of the local finite element function space for a cell
  virtual unsigned int local_dimension(const ufc::cell& c) const
  {
    return 10;
  }

  /// Return the maximum dimension of the local finite element function space
  virtual unsigned int max_local_dimension() const
  {
    return 10;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 4;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    switch (d)
    {
    case 0:
      {
        return 1;
        break;
      }
    case 1:
      {
        return 2;
        break;
      }
    case 2:
      {
        return 1;
        break;
      }
    }
    
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    unsigned int offset = 0;
    dofs[0] = offset + c.entity_indices[0][0];
    dofs[1] = offset + c.entity_indices[0][1];
    dofs[2] = offset + c.entity_indices[0][2];
    offset += m.num_entities[0];
    dofs[3] = offset + 2*c.entity_indices[1][0];
    dofs[4] = offset + 2*c.entity_indices[1][0] + 1;
    dofs[5] = offset + 2*c.entity_indices[1][1];
    dofs[6] = offset + 2*c.entity_indices[1][1] + 1;
    dofs[7] = offset + 2*c.entity_indices[1][2];
    dofs[8] = offset + 2*c.entity_indices[1][2] + 1;
    offset += 2*m.num_entities[1];
    dofs[9] = offset + c.entity_indices[2][0];
    offset += m.num_entities[2];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch (facet)
    {
    case 0:
      {
        dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
        break;
      }
    case 1:
      {
        dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 5;
      dofs[3] = 6;
        break;
      }
    case 2:
      {
        dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 7;
      dofs[3] = 8;
        break;
      }
    }
    
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    if (d > 2)
    {
    throw std::runtime_error("d is larger than dimension (2)");
    }
    
    switch (d)
    {
    case 0:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 0;
          break;
        }
      case 1:
        {
          dofs[0] = 1;
          break;
        }
      case 2:
        {
          dofs[0] = 2;
          break;
        }
      }
      
        break;
      }
    case 1:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 3;
        dofs[1] = 4;
          break;
        }
      case 1:
        {
          dofs[0] = 5;
        dofs[1] = 6;
          break;
        }
      case 2:
        {
          dofs[0] = 7;
        dofs[1] = 8;
          break;
        }
      }
      
        break;
      }
    case 2:
      {
        if (i > 0)
      {
      throw std::runtime_error("i is larger than number of entities (0)");
      }
      
      dofs[0] = 9;
        break;
      }
    }
    
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[3][0] = 0.666666666666667*x[1][0] + 0.333333333333333*x[2][0];
    coordinates[3][1] = 0.666666666666667*x[1][1] + 0.333333333333333*x[2][1];
    coordinates[4][0] = 0.333333333333333*x[1][0] + 0.666666666666667*x[2][0];
    coordinates[4][1] = 0.333333333333333*x[1][1] + 0.666666666666667*x[2][1];
    coordinates[5][0] = 0.666666666666667*x[0][0] + 0.333333333333333*x[2][0];
    coordinates[5][1] = 0.666666666666667*x[0][1] + 0.333333333333333*x[2][1];
    coordinates[6][0] = 0.333333333333333*x[0][0] + 0.666666666666667*x[2][0];
    coordinates[6][1] = 0.333333333333333*x[0][1] + 0.666666666666667*x[2][1];
    coordinates[7][0] = 0.666666666666667*x[0][0] + 0.333333333333333*x[1][0];
    coordinates[7][1] = 0.666666666666667*x[0][1] + 0.333333333333333*x[1][1];
    coordinates[8][0] = 0.333333333333333*x[0][0] + 0.666666666666667*x[1][0];
    coordinates[8][1] = 0.333333333333333*x[0][1] + 0.666666666666667*x[1][1];
    coordinates[9][0] = 0.333333333333333*x[0][0] + 0.333333333333333*x[1][0] + 0.333333333333333*x[2][0];
    coordinates[9][1] = 0.333333333333333*x[0][1] + 0.333333333333333*x[1][1] + 0.333333333333333*x[2][1];
  }

  /// Return the number of sub dofmaps (for a mixed element)
  virtual unsigned int num_sub_dofmaps() const
  {
    return 0;
  }

  /// Create a new dofmap for sub dofmap i (for a mixed element)
  virtual ufc::dofmap* create_sub_dofmap(unsigned int i) const
  {
    return 0;
  }

  /// Create a new class instance
  virtual ufc::dofmap* create() const
  {
    return new laplacian_f3_p1_q3_tensor_dofmap_1();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class laplacian_f3_p1_q3_tensor_cell_integral_0_0: public ufc::cell_integral
{
public:

  /// Constructor
  laplacian_f3_p1_q3_tensor_cell_integral_0_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~laplacian_f3_p1_q3_tensor_cell_integral_0_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Number of operations (multiply-add pairs) for Jacobian data:      11
    // Number of operations (multiply-add pairs) for geometry tensor:    378
    // Number of operations (multiply-add pairs) for tensor contraction: 5208
    // Total number of operations (multiply-add pairs):                  5597
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    const double K_00 =  J_11 / detJ;
    const double K_01 = -J_01 / detJ;
    const double K_10 = -J_10 / detJ;
    const double K_11 =  J_00 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute geometry tensor
    const double G0_0_0_0_0_0 = det*(w[2][0]*w[0][0]*w[1][0]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_0_0_1 = det*(w[2][0]*w[0][0]*w[1][1]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_0_0_2 = det*(w[2][0]*w[0][0]*w[1][2]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_0_1_0 = det*(w[2][0]*w[0][1]*w[1][0]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_0_1_1 = det*(w[2][0]*w[0][1]*w[1][1]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_0_1_2 = det*(w[2][0]*w[0][1]*w[1][2]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_0_2_0 = det*(w[2][0]*w[0][2]*w[1][0]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_0_2_1 = det*(w[2][0]*w[0][2]*w[1][1]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_0_2_2 = det*(w[2][0]*w[0][2]*w[1][2]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_1_0_0 = det*(w[2][1]*w[0][0]*w[1][0]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_1_0_1 = det*(w[2][1]*w[0][0]*w[1][1]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_1_0_2 = det*(w[2][1]*w[0][0]*w[1][2]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_1_1_0 = det*(w[2][1]*w[0][1]*w[1][0]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_1_1_1 = det*(w[2][1]*w[0][1]*w[1][1]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_1_1_2 = det*(w[2][1]*w[0][1]*w[1][2]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_1_2_0 = det*(w[2][1]*w[0][2]*w[1][0]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_1_2_1 = det*(w[2][1]*w[0][2]*w[1][1]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_1_2_2 = det*(w[2][1]*w[0][2]*w[1][2]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_2_0_0 = det*(w[2][2]*w[0][0]*w[1][0]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_2_0_1 = det*(w[2][2]*w[0][0]*w[1][1]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_2_0_2 = det*(w[2][2]*w[0][0]*w[1][2]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_2_1_0 = det*(w[2][2]*w[0][1]*w[1][0]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_2_1_1 = det*(w[2][2]*w[0][1]*w[1][1]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_2_1_2 = det*(w[2][2]*w[0][1]*w[1][2]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_2_2_0 = det*(w[2][2]*w[0][2]*w[1][0]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_2_2_1 = det*(w[2][2]*w[0][2]*w[1][1]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_0_2_2_2 = det*(w[2][2]*w[0][2]*w[1][2]*((K_00*K_00 + K_01*K_01)));
    const double G0_0_1_0_0_0 = det*(w[2][0]*w[0][0]*w[1][0]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_0_0_1 = det*(w[2][0]*w[0][0]*w[1][1]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_0_0_2 = det*(w[2][0]*w[0][0]*w[1][2]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_0_1_0 = det*(w[2][0]*w[0][1]*w[1][0]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_0_1_1 = det*(w[2][0]*w[0][1]*w[1][1]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_0_1_2 = det*(w[2][0]*w[0][1]*w[1][2]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_0_2_0 = det*(w[2][0]*w[0][2]*w[1][0]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_0_2_1 = det*(w[2][0]*w[0][2]*w[1][1]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_0_2_2 = det*(w[2][0]*w[0][2]*w[1][2]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_1_0_0 = det*(w[2][1]*w[0][0]*w[1][0]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_1_0_1 = det*(w[2][1]*w[0][0]*w[1][1]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_1_0_2 = det*(w[2][1]*w[0][0]*w[1][2]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_1_1_0 = det*(w[2][1]*w[0][1]*w[1][0]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_1_1_1 = det*(w[2][1]*w[0][1]*w[1][1]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_1_1_2 = det*(w[2][1]*w[0][1]*w[1][2]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_1_2_0 = det*(w[2][1]*w[0][2]*w[1][0]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_1_2_1 = det*(w[2][1]*w[0][2]*w[1][1]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_1_2_2 = det*(w[2][1]*w[0][2]*w[1][2]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_2_0_0 = det*(w[2][2]*w[0][0]*w[1][0]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_2_0_1 = det*(w[2][2]*w[0][0]*w[1][1]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_2_0_2 = det*(w[2][2]*w[0][0]*w[1][2]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_2_1_0 = det*(w[2][2]*w[0][1]*w[1][0]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_2_1_1 = det*(w[2][2]*w[0][1]*w[1][1]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_2_1_2 = det*(w[2][2]*w[0][1]*w[1][2]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_2_2_0 = det*(w[2][2]*w[0][2]*w[1][0]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_2_2_1 = det*(w[2][2]*w[0][2]*w[1][1]*((K_00*K_10 + K_01*K_11)));
    const double G0_0_1_2_2_2 = det*(w[2][2]*w[0][2]*w[1][2]*((K_00*K_10 + K_01*K_11)));
    const double G0_1_0_0_0_0 = det*(w[2][0]*w[0][0]*w[1][0]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_0_0_1 = det*(w[2][0]*w[0][0]*w[1][1]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_0_0_2 = det*(w[2][0]*w[0][0]*w[1][2]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_0_1_0 = det*(w[2][0]*w[0][1]*w[1][0]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_0_1_1 = det*(w[2][0]*w[0][1]*w[1][1]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_0_1_2 = det*(w[2][0]*w[0][1]*w[1][2]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_0_2_0 = det*(w[2][0]*w[0][2]*w[1][0]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_0_2_1 = det*(w[2][0]*w[0][2]*w[1][1]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_0_2_2 = det*(w[2][0]*w[0][2]*w[1][2]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_1_0_0 = det*(w[2][1]*w[0][0]*w[1][0]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_1_0_1 = det*(w[2][1]*w[0][0]*w[1][1]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_1_0_2 = det*(w[2][1]*w[0][0]*w[1][2]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_1_1_0 = det*(w[2][1]*w[0][1]*w[1][0]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_1_1_1 = det*(w[2][1]*w[0][1]*w[1][1]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_1_1_2 = det*(w[2][1]*w[0][1]*w[1][2]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_1_2_0 = det*(w[2][1]*w[0][2]*w[1][0]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_1_2_1 = det*(w[2][1]*w[0][2]*w[1][1]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_1_2_2 = det*(w[2][1]*w[0][2]*w[1][2]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_2_0_0 = det*(w[2][2]*w[0][0]*w[1][0]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_2_0_1 = det*(w[2][2]*w[0][0]*w[1][1]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_2_0_2 = det*(w[2][2]*w[0][0]*w[1][2]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_2_1_0 = det*(w[2][2]*w[0][1]*w[1][0]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_2_1_1 = det*(w[2][2]*w[0][1]*w[1][1]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_2_1_2 = det*(w[2][2]*w[0][1]*w[1][2]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_2_2_0 = det*(w[2][2]*w[0][2]*w[1][0]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_2_2_1 = det*(w[2][2]*w[0][2]*w[1][1]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_0_2_2_2 = det*(w[2][2]*w[0][2]*w[1][2]*((K_10*K_00 + K_11*K_01)));
    const double G0_1_1_0_0_0 = det*(w[2][0]*w[0][0]*w[1][0]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_0_0_1 = det*(w[2][0]*w[0][0]*w[1][1]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_0_0_2 = det*(w[2][0]*w[0][0]*w[1][2]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_0_1_0 = det*(w[2][0]*w[0][1]*w[1][0]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_0_1_1 = det*(w[2][0]*w[0][1]*w[1][1]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_0_1_2 = det*(w[2][0]*w[0][1]*w[1][2]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_0_2_0 = det*(w[2][0]*w[0][2]*w[1][0]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_0_2_1 = det*(w[2][0]*w[0][2]*w[1][1]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_0_2_2 = det*(w[2][0]*w[0][2]*w[1][2]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_1_0_0 = det*(w[2][1]*w[0][0]*w[1][0]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_1_0_1 = det*(w[2][1]*w[0][0]*w[1][1]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_1_0_2 = det*(w[2][1]*w[0][0]*w[1][2]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_1_1_0 = det*(w[2][1]*w[0][1]*w[1][0]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_1_1_1 = det*(w[2][1]*w[0][1]*w[1][1]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_1_1_2 = det*(w[2][1]*w[0][1]*w[1][2]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_1_2_0 = det*(w[2][1]*w[0][2]*w[1][0]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_1_2_1 = det*(w[2][1]*w[0][2]*w[1][1]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_1_2_2 = det*(w[2][1]*w[0][2]*w[1][2]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_2_0_0 = det*(w[2][2]*w[0][0]*w[1][0]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_2_0_1 = det*(w[2][2]*w[0][0]*w[1][1]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_2_0_2 = det*(w[2][2]*w[0][0]*w[1][2]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_2_1_0 = det*(w[2][2]*w[0][1]*w[1][0]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_2_1_1 = det*(w[2][2]*w[0][1]*w[1][1]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_2_1_2 = det*(w[2][2]*w[0][1]*w[1][2]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_2_2_0 = det*(w[2][2]*w[0][2]*w[1][0]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_2_2_1 = det*(w[2][2]*w[0][2]*w[1][1]*((K_10*K_10 + K_11*K_11)));
    const double G0_1_1_2_2_2 = det*(w[2][2]*w[0][2]*w[1][2]*((K_10*K_10 + K_11*K_11)));
    
    // Compute element tensor
    A[73] = -0.0482142857142858*G0_0_0_0_0_0 - 0.0160714285714286*G0_0_0_0_0_1 - 0.0160714285714286*G0_0_0_0_0_2 - 0.0160714285714286*G0_0_0_0_1_0 - 0.0140625*G0_0_0_0_1_1 - 0.0060267857142857*G0_0_0_0_1_2 - 0.0160714285714286*G0_0_0_0_2_0 - 0.0060267857142857*G0_0_0_0_2_1 - 0.00602678571428571*G0_0_0_0_2_2 - 0.0160714285714286*G0_0_0_1_0_0 - 0.0140625*G0_0_0_1_0_1 - 0.0060267857142857*G0_0_0_1_0_2 - 0.0140625*G0_0_0_1_1_0 + 0.0783482142857141*G0_0_0_1_1_1 + 0.0281249999999999*G0_0_0_1_1_2 - 0.0060267857142857*G0_0_0_1_2_0 + 0.0281249999999999*G0_0_0_1_2_1 + 0.0180803571428571*G0_0_0_1_2_2 - 0.0160714285714286*G0_0_0_2_0_0 - 0.0060267857142857*G0_0_0_2_0_1 - 0.00602678571428571*G0_0_0_2_0_2 - 0.0060267857142857*G0_0_0_2_1_0 + 0.0281249999999999*G0_0_0_2_1_1 + 0.0180803571428571*G0_0_0_2_1_2 - 0.00602678571428571*G0_0_0_2_2_0 + 0.0180803571428571*G0_0_0_2_2_1 + 0.0241071428571427*G0_0_0_2_2_2 - 0.0301339285714285*G0_0_1_0_0_0 - 0.0180803571428571*G0_0_1_0_0_1 - 0.00401785714285714*G0_0_1_0_0_2 - 0.0180803571428571*G0_0_1_0_1_0 - 0.0180803571428572*G0_0_1_0_1_1 - 0.00200892857142858*G0_0_1_0_1_2 - 0.00401785714285714*G0_0_1_0_2_0 - 0.00200892857142858*G0_0_1_0_2_1 - 0.0180803571428571*G0_0_1_1_0_0 - 0.0180803571428572*G0_0_1_1_0_1 - 0.00200892857142858*G0_0_1_1_0_2 - 0.0180803571428572*G0_0_1_1_1_0 + 0.204910714285714*G0_0_1_1_1_1 + 0.0301339285714285*G0_0_1_1_1_2 - 0.00200892857142858*G0_0_1_1_2_0 + 0.0301339285714285*G0_0_1_1_2_1 + 0.0100446428571428*G0_0_1_1_2_2 - 0.00401785714285714*G0_0_1_2_0_0 - 0.00200892857142858*G0_0_1_2_0_1 - 0.00200892857142858*G0_0_1_2_1_0 + 0.0301339285714285*G0_0_1_2_1_1 + 0.0100446428571428*G0_0_1_2_1_2 + 0.0100446428571428*G0_0_1_2_2_1 + 0.00602678571428567*G0_0_1_2_2_2 - 0.0120535714285714*G0_1_0_0_0_0 - 0.0160714285714285*G0_1_0_0_0_1 - 0.00401785714285713*G0_1_0_0_0_2 - 0.0160714285714285*G0_1_0_0_1_0 - 0.0120535714285714*G0_1_0_0_1_1 - 0.00401785714285713*G0_1_0_0_1_2 - 0.00401785714285713*G0_1_0_0_2_0 - 0.00401785714285713*G0_1_0_0_2_1 - 0.0160714285714285*G0_1_0_1_0_0 - 0.0120535714285714*G0_1_0_1_0_1 - 0.00401785714285713*G0_1_0_1_0_2 - 0.0120535714285714*G0_1_0_1_1_0 + 0.0964285714285712*G0_1_0_1_1_1 + 0.0361607142857142*G0_1_0_1_1_2 - 0.00401785714285713*G0_1_0_1_2_0 + 0.0361607142857142*G0_1_0_1_2_1 + 0.0241071428571428*G0_1_0_1_2_2 - 0.00401785714285713*G0_1_0_2_0_0 - 0.00401785714285713*G0_1_0_2_0_1 - 0.00401785714285713*G0_1_0_2_1_0 + 0.0361607142857142*G0_1_0_2_1_1 + 0.0241071428571428*G0_1_0_2_1_2 + 0.0241071428571428*G0_1_0_2_2_1 + 0.0241071428571426*G0_1_0_2_2_2 + 0.00602678571428558*G0_1_1_0_0_0 - 0.0120535714285715*G0_1_1_0_0_1 + 0.00200892857142855*G0_1_1_0_0_2 - 0.0120535714285715*G0_1_1_0_1_0 - 0.0120535714285715*G0_1_1_0_1_1 + 0.00200892857142855*G0_1_1_0_2_0 + 0.00200892857142856*G0_1_1_0_2_2 - 0.0120535714285715*G0_1_1_1_0_0 - 0.0120535714285715*G0_1_1_1_0_1 - 0.0120535714285715*G0_1_1_1_1_0 + 0.241071428571428*G0_1_1_1_1_1 + 0.0361607142857142*G0_1_1_1_1_2 + 0.0361607142857142*G0_1_1_1_2_1 + 0.0120535714285714*G0_1_1_1_2_2 + 0.00200892857142855*G0_1_1_2_0_0 + 0.00200892857142856*G0_1_1_2_0_2 + 0.0361607142857142*G0_1_1_2_1_1 + 0.0120535714285714*G0_1_1_2_1_2 + 0.00200892857142856*G0_1_1_2_2_0 + 0.0120535714285714*G0_1_1_2_2_1 + 0.00602678571428568*G0_1_1_2_2_2;
    A[69] = -0.0241071428571428*G0_0_0_0_0_2 - 0.0241071428571428*G0_0_0_0_2_0 - 0.0482142857142856*G0_0_0_0_2_2 + 0.0241071428571429*G0_0_0_1_1_2 + 0.0241071428571429*G0_0_0_1_2_1 + 0.0482142857142857*G0_0_0_1_2_2 - 0.0241071428571428*G0_0_0_2_0_0 - 0.0482142857142856*G0_0_0_2_0_2 + 0.0241071428571429*G0_0_0_2_1_1 + 0.0482142857142857*G0_0_0_2_1_2 - 0.0482142857142856*G0_0_0_2_2_0 + 0.0482142857142857*G0_0_0_2_2_1 + 0.024107142857143*G0_0_1_0_0_0 + 0.0120535714285715*G0_0_1_0_0_1 + 0.0120535714285714*G0_0_1_0_0_2 + 0.0120535714285715*G0_0_1_0_1_0 + 0.0120535714285715*G0_0_1_0_1_1 + 0.0120535714285714*G0_0_1_0_1_2 + 0.0120535714285714*G0_0_1_0_2_0 + 0.0120535714285714*G0_0_1_0_2_1 + 0.0361607142857143*G0_0_1_0_2_2 + 0.0120535714285715*G0_0_1_1_0_0 + 0.0120535714285715*G0_0_1_1_0_1 + 0.0120535714285714*G0_0_1_1_0_2 + 0.0120535714285715*G0_0_1_1_1_0 + 0.024107142857143*G0_0_1_1_1_1 + 0.0361607142857143*G0_0_1_1_1_2 + 0.0120535714285714*G0_0_1_1_2_0 + 0.0361607142857143*G0_0_1_1_2_1 + 0.084375*G0_0_1_1_2_2 + 0.0120535714285714*G0_0_1_2_0_0 + 0.0120535714285714*G0_0_1_2_0_1 + 0.0361607142857143*G0_0_1_2_0_2 + 0.0120535714285714*G0_0_1_2_1_0 + 0.0361607142857143*G0_0_1_2_1_1 + 0.084375*G0_0_1_2_1_2 + 0.0361607142857143*G0_0_1_2_2_0 + 0.084375*G0_0_1_2_2_1 + 0.313392857142857*G0_0_1_2_2_2 + 0.0964285714285713*G0_1_0_0_0_0 + 0.0120535714285714*G0_1_0_0_0_1 + 0.0843749999999999*G0_1_0_0_0_2 + 0.0120535714285714*G0_1_0_0_1_0 + 0.0120535714285714*G0_1_0_0_1_2 + 0.0843749999999999*G0_1_0_0_2_0 + 0.0120535714285714*G0_1_0_0_2_1 + 0.0723214285714284*G0_1_0_0_2_2 + 0.0120535714285714*G0_1_0_1_0_0 + 0.0120535714285714*G0_1_0_1_0_2 - 0.0120535714285714*G0_1_0_1_1_1 + 0.0120535714285714*G0_1_0_1_1_2 + 0.0120535714285714*G0_1_0_1_2_0 + 0.0120535714285714*G0_1_0_1_2_1 + 0.0482142857142856*G0_1_0_1_2_2 + 0.0843749999999999*G0_1_0_2_0_0 + 0.0120535714285714*G0_1_0_2_0_1 + 0.0723214285714284*G0_1_0_2_0_2 + 0.0120535714285714*G0_1_0_2_1_0 + 0.0120535714285714*G0_1_0_2_1_1 + 0.0482142857142856*G0_1_0_2_1_2 + 0.0723214285714284*G0_1_0_2_2_0 + 0.0482142857142856*G0_1_0_2_2_1 + 0.16875*G0_1_0_2_2_2 - 0.120535714285714*G0_1_1_0_0_0 - 0.0482142857142857*G0_1_1_0_0_1 - 0.0482142857142857*G0_1_1_0_1_0 - 0.0361607142857143*G0_1_1_0_1_1 + 0.0120535714285714*G0_1_1_0_2_2 - 0.0482142857142857*G0_1_1_1_0_0 - 0.0361607142857143*G0_1_1_1_0_1 - 0.0361607142857143*G0_1_1_1_1_0 - 0.0482142857142856*G0_1_1_1_1_1 + 0.0120535714285714*G0_1_1_1_1_2 + 0.0120535714285714*G0_1_1_1_2_1 + 0.0482142857142857*G0_1_1_1_2_2 + 0.0120535714285714*G0_1_1_2_0_2 + 0.0120535714285714*G0_1_1_2_1_1 + 0.0482142857142857*G0_1_1_2_1_2 + 0.0120535714285714*G0_1_1_2_2_0 + 0.0482142857142857*G0_1_1_2_2_1 + 0.204910714285715*G0_1_1_2_2_2;
    A[16] = -0.000669642857142854*G0_0_0_0_0_0 - 0.000223214285714285*G0_0_0_0_0_1 - 0.003125*G0_0_0_0_0_2 - 0.000223214285714285*G0_0_0_0_1_0 + 0.00178571428571429*G0_0_0_0_1_1 + 0.00044642857142857*G0_0_0_0_1_2 - 0.003125*G0_0_0_0_2_0 + 0.00044642857142857*G0_0_0_0_2_1 - 0.00870535714285715*G0_0_0_0_2_2 - 0.000223214285714285*G0_0_0_1_0_0 + 0.00178571428571429*G0_0_0_1_0_1 + 0.00044642857142857*G0_0_0_1_0_2 + 0.00178571428571429*G0_0_0_1_1_0 + 0.0174107142857143*G0_0_0_1_1_1 + 0.00491071428571428*G0_0_0_1_1_2 + 0.00044642857142857*G0_0_0_1_2_0 + 0.00491071428571428*G0_0_0_1_2_1 + 0.00334821428571426*G0_0_0_1_2_2 - 0.003125*G0_0_0_2_0_0 + 0.00044642857142857*G0_0_0_2_0_1 - 0.00870535714285715*G0_0_0_2_0_2 + 0.00044642857142857*G0_0_0_2_1_0 + 0.00491071428571428*G0_0_0_2_1_1 + 0.00334821428571426*G0_0_0_2_1_2 - 0.00870535714285715*G0_0_0_2_2_0 + 0.00334821428571426*G0_0_0_2_2_1 - 0.0508928571428573*G0_0_0_2_2_2 + 0.00200892857142859*G0_0_1_0_0_0 + 0.000446428571428575*G0_0_1_0_0_1 + 0.00558035714285714*G0_0_1_0_0_2 + 0.000446428571428574*G0_0_1_0_1_0 - 0.00178571428571428*G0_0_1_0_1_1 - 0.000669642857142856*G0_0_1_0_1_2 + 0.00558035714285714*G0_0_1_0_2_0 - 0.000669642857142856*G0_0_1_0_2_1 + 0.00245535714285712*G0_0_1_0_2_2 + 0.000446428571428575*G0_0_1_1_0_0 - 0.00178571428571428*G0_0_1_1_0_1 - 0.000669642857142855*G0_0_1_1_0_2 - 0.00178571428571428*G0_0_1_1_1_0 + 0.00178571428571428*G0_0_1_1_1_2 - 0.000669642857142856*G0_0_1_1_2_0 + 0.00178571428571428*G0_0_1_1_2_1 + 0.000892857142857123*G0_0_1_1_2_2 + 0.00558035714285714*G0_0_1_2_0_0 - 0.000669642857142856*G0_0_1_2_0_1 + 0.00245535714285712*G0_0_1_2_0_2 - 0.000669642857142856*G0_0_1_2_1_0 + 0.00178571428571428*G0_0_1_2_1_1 + 0.000892857142857123*G0_0_1_2_1_2 + 0.00245535714285712*G0_0_1_2_2_0 + 0.000892857142857123*G0_0_1_2_2_1 - 0.0261160714285716*G0_0_1_2_2_2;
    A[37] = A[73] + 0.0180803571428571*G0_0_1_0_0_0 + 0.00200892857142861*G0_0_1_0_0_1 + 0.00200892857142861*G0_0_1_0_1_0 + 0.00602678571428575*G0_0_1_0_1_1 - 0.00200892857142855*G0_0_1_0_1_2 - 0.00200892857142855*G0_0_1_0_2_1 + 0.00200892857142861*G0_0_1_1_0_0 + 0.00602678571428575*G0_0_1_1_0_1 - 0.00200892857142855*G0_0_1_1_0_2 + 0.00602678571428575*G0_0_1_1_1_0 - 0.108482142857143*G0_0_1_1_1_1 + 0.00602678571428574*G0_0_1_1_1_2 - 0.00200892857142855*G0_0_1_1_2_0 + 0.00602678571428574*G0_0_1_1_2_1 + 0.0140625*G0_0_1_1_2_2 - 0.00200892857142855*G0_0_1_2_0_1 - 0.00200892857142855*G0_0_1_2_1_0 + 0.00602678571428574*G0_0_1_2_1_1 + 0.0140625*G0_0_1_2_1_2 + 0.0140625*G0_0_1_2_2_1 + 0.018080357142857*G0_0_1_2_2_2 - 0.0180803571428571*G0_1_0_0_0_0 - 0.00200892857142861*G0_1_0_0_0_1 - 0.00200892857142861*G0_1_0_0_1_0 - 0.00602678571428577*G0_1_0_0_1_1 + 0.00200892857142855*G0_1_0_0_1_2 + 0.00200892857142855*G0_1_0_0_2_1 - 0.00200892857142861*G0_1_0_1_0_0 - 0.00602678571428577*G0_1_0_1_0_1 + 0.00200892857142855*G0_1_0_1_0_2 - 0.00602678571428575*G0_1_0_1_1_0 + 0.108482142857143*G0_1_0_1_1_1 - 0.00602678571428576*G0_1_0_1_1_2 + 0.00200892857142855*G0_1_0_1_2_0 - 0.00602678571428575*G0_1_0_1_2_1 - 0.0140625*G0_1_0_1_2_2 + 0.00200892857142855*G0_1_0_2_0_1 + 0.00200892857142855*G0_1_0_2_1_0 - 0.00602678571428576*G0_1_0_2_1_1 - 0.0140625*G0_1_0_2_1_2 - 0.0140625*G0_1_0_2_2_1 - 0.018080357142857*G0_1_0_2_2_2;
    A[25] = 0.0113839285714286*G0_1_0_0_0_0 + 0.00245535714285714*G0_1_0_0_0_1 + 0.00892857142857142*G0_1_0_0_0_2 + 0.00245535714285714*G0_1_0_0_1_0 + 0.00111607142857142*G0_1_0_0_1_1 + 0.003125*G0_1_0_0_1_2 + 0.00892857142857142*G0_1_0_0_2_0 + 0.003125*G0_1_0_0_2_1 + 0.00937500000000002*G0_1_0_0_2_2 + 0.00245535714285714*G0_1_0_1_0_0 + 0.00111607142857142*G0_1_0_1_0_1 + 0.003125*G0_1_0_1_0_2 + 0.00111607142857142*G0_1_0_1_1_0 - 0.000669642857142879*G0_1_0_1_1_1 + 0.00357142857142857*G0_1_0_1_1_2 + 0.003125*G0_1_0_1_2_0 + 0.00357142857142857*G0_1_0_1_2_1 + 0.0174107142857143*G0_1_0_1_2_2 + 0.00892857142857143*G0_1_0_2_0_0 + 0.003125*G0_1_0_2_0_1 + 0.00937500000000002*G0_1_0_2_0_2 + 0.003125*G0_1_0_2_1_0 + 0.00357142857142857*G0_1_0_2_1_1 + 0.0174107142857143*G0_1_0_2_1_2 + 0.00937500000000002*G0_1_0_2_2_0 + 0.0174107142857143*G0_1_0_2_2_1 + 0.141964285714286*G0_1_0_2_2_2 + 0.0622767857142856*G0_1_1_0_0_0 + 0.0111607142857143*G0_1_1_0_0_1 + 0.00558035714285713*G0_1_1_0_0_2 + 0.0111607142857143*G0_1_1_0_1_0 + 0.00424107142857142*G0_1_1_0_1_1 + 0.00267857142857143*G0_1_1_0_1_2 + 0.00558035714285713*G0_1_1_0_2_0 + 0.00267857142857143*G0_1_1_0_2_1 + 0.00446428571428574*G0_1_1_0_2_2 + 0.0111607142857143*G0_1_1_1_0_0 + 0.00424107142857142*G0_1_1_1_0_1 + 0.00267857142857143*G0_1_1_1_0_2 + 0.00424107142857142*G0_1_1_1_1_0 + 0.00379464285714286*G0_1_1_1_1_2 + 0.00267857142857143*G0_1_1_1_2_0 + 0.00379464285714286*G0_1_1_1_2_1 + 0.015625*G0_1_1_1_2_2 + 0.00558035714285713*G0_1_1_2_0_0 + 0.00267857142857143*G0_1_1_2_0_1 + 0.00446428571428574*G0_1_1_2_0_2 + 0.00267857142857143*G0_1_1_2_1_0 + 0.00379464285714286*G0_1_1_2_1_1 + 0.015625*G0_1_1_2_1_2 + 0.00446428571428574*G0_1_1_2_2_0 + 0.015625*G0_1_1_2_2_1 + 0.124553571428572*G0_1_1_2_2_2;
    A[85] = A[37] + 0.1265625*G0_0_0_0_0_0 + 0.00200892857142857*G0_0_0_0_0_1 + 0.0441964285714286*G0_0_0_0_0_2 + 0.00200892857142857*G0_0_0_0_1_0 - 0.00200892857142856*G0_0_0_0_1_1 + 0.0441964285714286*G0_0_0_0_2_0 + 0.0241071428571428*G0_0_0_0_2_2 + 0.00200892857142857*G0_0_0_1_0_0 - 0.00200892857142856*G0_0_0_1_0_1 - 0.00200892857142857*G0_0_0_1_1_0 - 0.1265625*G0_0_0_1_1_1 - 0.0441964285714284*G0_0_0_1_1_2 - 0.0441964285714284*G0_0_0_1_2_1 - 0.0241071428571428*G0_0_0_1_2_2 + 0.0441964285714286*G0_0_0_2_0_0 + 0.0241071428571428*G0_0_0_2_0_2 - 0.0441964285714285*G0_0_0_2_1_1 - 0.0241071428571428*G0_0_0_2_1_2 + 0.0241071428571428*G0_0_0_2_2_0 - 0.0241071428571428*G0_0_0_2_2_1 - 0.114508928571428*G0_0_1_0_0_0 + 0.0200892857142857*G0_0_1_0_0_1 + 0.0020089285714286*G0_0_1_0_0_2 + 0.0200892857142857*G0_0_1_0_1_0 + 0.0140625*G0_0_1_0_1_1 + 0.0020089285714286*G0_0_1_0_2_0 + 0.0080357142857143*G0_0_1_0_2_2 + 0.0200892857142857*G0_0_1_1_0_0 + 0.0140625*G0_0_1_1_0_1 + 0.0140625*G0_0_1_1_1_0 - 0.114508928571428*G0_0_1_1_1_1 - 0.0482142857142856*G0_0_1_1_1_2 - 0.0482142857142856*G0_0_1_1_2_1 - 0.0301339285714285*G0_0_1_1_2_2 + 0.0020089285714286*G0_0_1_2_0_0 + 0.0080357142857143*G0_0_1_2_0_2 - 0.0482142857142856*G0_0_1_2_1_1 - 0.0301339285714285*G0_0_1_2_1_2 + 0.00803571428571431*G0_0_1_2_2_0 - 0.0301339285714285*G0_0_1_2_2_1 - 0.0060267857142856*G0_0_1_2_2_2 + 0.0120535714285713*G0_1_0_0_0_0 + 0.0160714285714286*G0_1_0_0_0_1 - 0.00401785714285715*G0_1_0_0_0_2 + 0.0160714285714286*G0_1_0_0_1_0 + 0.0180803571428572*G0_1_0_0_1_1 - 0.00401785714285715*G0_1_0_0_2_0 - 0.0060267857142857*G0_1_0_0_2_2 + 0.0160714285714286*G0_1_0_1_0_0 + 0.0180803571428572*G0_1_0_1_0_1 + 0.0180803571428572*G0_1_0_1_1_0 - 0.241071428571428*G0_1_0_1_1_1 - 0.0421874999999999*G0_1_0_1_1_2 - 0.0421874999999999*G0_1_0_1_2_1 - 0.0160714285714285*G0_1_0_1_2_2 - 0.00401785714285715*G0_1_0_2_0_0 - 0.0060267857142857*G0_1_0_2_0_2 - 0.0421874999999999*G0_1_0_2_1_1 - 0.0160714285714285*G0_1_0_2_1_2 - 0.0060267857142857*G0_1_0_2_2_0 - 0.0160714285714285*G0_1_0_2_2_1 - 0.00602678571428569*G0_1_0_2_2_2 + 0.0120535714285714*G0_1_1_0_0_0 + 0.0160714285714286*G0_1_1_0_0_1 - 0.00401785714285713*G0_1_1_0_0_2 + 0.0160714285714286*G0_1_1_0_1_0 + 0.0180803571428572*G0_1_1_0_1_1 - 0.00401785714285713*G0_1_1_0_2_0 - 0.00602678571428569*G0_1_1_0_2_2 + 0.0160714285714286*G0_1_1_1_0_0 + 0.0180803571428572*G0_1_1_1_0_1 + 0.0180803571428572*G0_1_1_1_1_0 - 0.241071428571428*G0_1_1_1_1_1 - 0.0421874999999999*G0_1_1_1_1_2 - 0.0421874999999999*G0_1_1_1_2_1 - 0.0160714285714285*G0_1_1_1_2_2 - 0.00401785714285713*G0_1_1_2_0_0 - 0.00602678571428569*G0_1_1_2_0_2 - 0.0421874999999999*G0_1_1_2_1_1 - 0.0160714285714285*G0_1_1_2_1_2 - 0.00602678571428569*G0_1_1_2_2_0 - 0.0160714285714285*G0_1_1_2_2_1 - 0.00602678571428568*G0_1_1_2_2_2;
    A[58] = A[85] + 0.108482142857142*G0_0_1_0_0_0 - 0.00602678571428574*G0_0_1_0_0_1 - 0.00602678571428575*G0_0_1_0_0_2 - 0.00602678571428574*G0_0_1_0_1_0 - 0.00200892857142856*G0_0_1_0_1_1 + 0.00200892857142856*G0_0_1_0_1_2 - 0.00602678571428575*G0_0_1_0_2_0 + 0.00200892857142856*G0_0_1_0_2_1 - 0.0140625*G0_0_1_0_2_2 - 0.00602678571428574*G0_0_1_1_0_0 - 0.00200892857142856*G0_0_1_1_0_1 + 0.00200892857142856*G0_0_1_1_0_2 - 0.00200892857142856*G0_0_1_1_1_0 - 0.0180803571428569*G0_0_1_1_1_1 + 0.00200892857142856*G0_0_1_1_2_0 - 0.00602678571428575*G0_0_1_2_0_0 + 0.00200892857142856*G0_0_1_2_0_1 - 0.0140625*G0_0_1_2_0_2 + 0.00200892857142856*G0_0_1_2_1_0 - 0.0140625*G0_0_1_2_2_0 - 0.0180803571428571*G0_0_1_2_2_2 - 0.108482142857142*G0_1_0_0_0_0 + 0.00602678571428574*G0_1_0_0_0_1 + 0.00602678571428575*G0_1_0_0_0_2 + 0.00602678571428574*G0_1_0_0_1_0 + 0.00200892857142856*G0_1_0_0_1_1 - 0.00200892857142856*G0_1_0_0_1_2 + 0.00602678571428576*G0_1_0_0_2_0 - 0.00200892857142856*G0_1_0_0_2_1 + 0.0140625*G0_1_0_0_2_2 + 0.00602678571428574*G0_1_0_1_0_0 + 0.00200892857142856*G0_1_0_1_0_1 - 0.00200892857142856*G0_1_0_1_0_2 + 0.00200892857142857*G0_1_0_1_1_0 + 0.0180803571428569*G0_1_0_1_1_1 - 0.00200892857142856*G0_1_0_1_2_0 + 0.00602678571428575*G0_1_0_2_0_0 - 0.00200892857142856*G0_1_0_2_0_1 + 0.0140625*G0_1_0_2_0_2 - 0.00200892857142856*G0_1_0_2_1_0 + 0.0140625*G0_1_0_2_2_0 + 0.0180803571428571*G0_1_0_2_2_2;
    A[11] = 0.0125*G0_0_0_0_0_0 + 0.0050595238095238*G0_0_0_0_0_1 + 0.00416666666666666*G0_0_0_0_0_2 + 0.0050595238095238*G0_0_0_0_1_0 + 0.0193452380952381*G0_0_0_0_1_1 + 0.0025297619047619*G0_0_0_0_1_2 + 0.00416666666666666*G0_0_0_0_2_0 + 0.0025297619047619*G0_0_0_0_2_1 + 0.00416666666666666*G0_0_0_0_2_2 + 0.0050595238095238*G0_0_0_1_0_0 + 0.0193452380952381*G0_0_0_1_0_1 + 0.0025297619047619*G0_0_0_1_0_2 + 0.0193452380952381*G0_0_0_1_1_0 + 0.213392857142857*G0_0_0_1_1_1 + 0.0193452380952381*G0_0_0_1_1_2 + 0.0025297619047619*G0_0_0_1_2_0 + 0.0193452380952381*G0_0_0_1_2_1 + 0.0050595238095238*G0_0_0_1_2_2 + 0.00416666666666666*G0_0_0_2_0_0 + 0.0025297619047619*G0_0_0_2_0_1 + 0.00416666666666666*G0_0_0_2_0_2 + 0.0025297619047619*G0_0_0_2_1_0 + 0.0193452380952381*G0_0_0_2_1_1 + 0.0050595238095238*G0_0_0_2_1_2 + 0.00416666666666666*G0_0_0_2_2_0 + 0.0050595238095238*G0_0_0_2_2_1 + 0.0125*G0_0_0_2_2_2;
    A[75] = A[73] - 0.108482142857143*G0_0_0_0_0_0 + 0.0200892857142857*G0_0_0_0_0_1 - 0.0160714285714286*G0_0_0_0_0_2 + 0.0200892857142857*G0_0_0_0_1_0 + 0.028125*G0_0_0_0_1_1 + 0.0120535714285714*G0_0_0_0_1_2 - 0.0160714285714286*G0_0_0_0_2_0 + 0.0120535714285714*G0_0_0_0_2_1 + 0.0200892857142857*G0_0_0_1_0_0 + 0.028125*G0_0_0_1_0_1 + 0.0120535714285714*G0_0_0_1_0_2 + 0.028125*G0_0_0_1_1_0 - 0.0361607142857142*G0_0_0_1_1_1 - 0.00803571428571426*G0_0_0_1_1_2 + 0.0120535714285714*G0_0_0_1_2_0 - 0.00803571428571426*G0_0_0_1_2_1 - 0.0160714285714286*G0_0_0_2_0_0 + 0.0120535714285714*G0_0_0_2_0_1 + 0.0120535714285714*G0_0_0_2_1_0 - 0.00803571428571426*G0_0_0_2_1_1 + 0.439955357142856*G0_0_1_0_0_0 + 0.0462053571428571*G0_0_1_0_0_1 + 0.032142857142857*G0_0_1_0_0_2 + 0.0462053571428571*G0_0_1_0_1_0 + 0.0261160714285714*G0_0_1_0_1_1 + 0.0100446428571428*G0_0_1_0_1_2 + 0.032142857142857*G0_0_1_0_2_0 + 0.0100446428571428*G0_0_1_0_2_1 + 0.00803571428571427*G0_0_1_0_2_2 + 0.0462053571428571*G0_0_1_1_0_0 + 0.0261160714285714*G0_0_1_1_0_1 + 0.0100446428571428*G0_0_1_1_0_2 + 0.0261160714285714*G0_0_1_1_1_0 - 0.174776785714285*G0_0_1_1_1_1 - 0.0120535714285714*G0_0_1_1_1_2 + 0.0100446428571428*G0_0_1_1_2_0 - 0.0120535714285714*G0_0_1_1_2_1 + 0.00803571428571429*G0_0_1_1_2_2 + 0.032142857142857*G0_0_1_2_0_0 + 0.0100446428571428*G0_0_1_2_0_1 + 0.00803571428571427*G0_0_1_2_0_2 + 0.0100446428571428*G0_0_1_2_1_0 - 0.0120535714285714*G0_0_1_2_1_1 + 0.00803571428571428*G0_0_1_2_1_2 + 0.00803571428571427*G0_0_1_2_2_0 + 0.00803571428571429*G0_0_1_2_2_1 + 0.0241071428571429*G0_0_1_2_2_2 + 0.132589285714286*G0_1_0_0_0_0 + 0.0562499999999999*G0_1_0_0_0_1 + 0.0441964285714285*G0_1_0_0_0_2 + 0.0562499999999999*G0_1_0_0_1_0 + 0.0361607142857142*G0_1_0_0_1_1 + 0.0200892857142857*G0_1_0_0_1_2 + 0.0441964285714285*G0_1_0_0_2_0 + 0.0200892857142857*G0_1_0_0_2_1 + 0.0241071428571428*G0_1_0_0_2_2 + 0.0562499999999999*G0_1_0_1_0_0 + 0.0361607142857142*G0_1_0_1_0_1 + 0.0200892857142857*G0_1_0_1_0_2 + 0.0361607142857142*G0_1_0_1_1_0 - 0.0482142857142856*G0_1_0_1_1_1 - 0.0120535714285714*G0_1_0_1_1_2 + 0.0200892857142857*G0_1_0_1_2_0 - 0.0120535714285714*G0_1_0_1_2_1 + 0.0441964285714285*G0_1_0_2_0_0 + 0.0200892857142857*G0_1_0_2_0_1 + 0.0241071428571428*G0_1_0_2_0_2 + 0.0200892857142857*G0_1_0_2_1_0 - 0.0120535714285714*G0_1_0_2_1_1 + 0.0241071428571428*G0_1_0_2_2_0 + 0.024107142857143*G0_1_0_2_2_2 - 0.162723214285714*G0_1_1_0_0_0 - 0.0200892857142857*G0_1_1_0_0_1 + 0.00200892857142857*G0_1_1_0_0_2 - 0.0200892857142857*G0_1_1_0_1_0 + 0.00602678571428574*G0_1_1_0_1_1 + 0.00602678571428571*G0_1_1_0_1_2 + 0.00200892857142857*G0_1_1_0_2_0 + 0.00602678571428571*G0_1_1_0_2_1 + 0.0120535714285714*G0_1_1_0_2_2 - 0.0200892857142857*G0_1_1_1_0_0 + 0.00602678571428574*G0_1_1_1_0_1 + 0.00602678571428571*G0_1_1_1_0_2 + 0.00602678571428575*G0_1_1_1_1_0 - 0.216964285714285*G0_1_1_1_1_1 - 0.0180803571428571*G0_1_1_1_1_2 + 0.00602678571428571*G0_1_1_1_2_0 - 0.0180803571428571*G0_1_1_1_2_1 + 0.00803571428571427*G0_1_1_1_2_2 + 0.00200892857142858*G0_1_1_2_0_0 + 0.00602678571428571*G0_1_1_2_0_1 + 0.0120535714285714*G0_1_1_2_0_2 + 0.00602678571428571*G0_1_1_2_1_0 - 0.0180803571428571*G0_1_1_2_1_1 + 0.00803571428571427*G0_1_1_2_1_2 + 0.0120535714285714*G0_1_1_2_2_0 + 0.00803571428571427*G0_1_1_2_2_1 + 0.0361607142857142*G0_1_1_2_2_2;
    A[78] = -A[73] - 0.331473214285714*G0_0_0_0_0_0 - 0.0703124999999999*G0_0_0_0_0_1 - 0.0602678571428571*G0_0_0_0_0_2 - 0.0703124999999999*G0_0_0_0_1_0 - 0.0683035714285714*G0_0_0_0_1_1 - 0.0200892857142857*G0_0_0_0_1_2 - 0.0602678571428571*G0_0_0_0_2_0 - 0.0200892857142857*G0_0_0_0_2_1 - 0.0220982142857142*G0_0_0_0_2_2 - 0.0703124999999999*G0_0_0_1_0_0 - 0.0683035714285714*G0_0_0_1_0_1 - 0.0200892857142857*G0_0_0_1_0_2 - 0.0683035714285714*G0_0_0_1_1_0 - 0.204910714285714*G0_0_0_1_1_1 - 0.0160714285714285*G0_0_0_1_1_2 - 0.0200892857142857*G0_0_0_1_2_0 - 0.0160714285714285*G0_0_0_1_2_1 + 0.00200892857142856*G0_0_0_1_2_2 - 0.0602678571428571*G0_0_0_2_0_0 - 0.0200892857142857*G0_0_0_2_0_1 - 0.0220982142857142*G0_0_0_2_0_2 - 0.0200892857142857*G0_0_0_2_1_0 - 0.0160714285714285*G0_0_0_2_1_1 + 0.00200892857142857*G0_0_0_2_1_2 - 0.0220982142857142*G0_0_0_2_2_0 + 0.00200892857142856*G0_0_0_2_2_1 + 0.0120535714285714*G0_0_0_2_2_2 - 0.0904017857142858*G0_1_0_0_0_0 - 0.0883928571428571*G0_1_0_0_0_1 - 0.0180803571428571*G0_1_0_0_0_2 - 0.0883928571428571*G0_1_0_0_1_0 - 0.0843749999999999*G0_1_0_0_1_1 - 0.0200892857142857*G0_1_0_0_1_2 - 0.0180803571428571*G0_1_0_0_2_0 - 0.0200892857142857*G0_1_0_0_2_1 - 0.00602678571428572*G0_1_0_0_2_2 - 0.0883928571428571*G0_1_0_1_0_0 - 0.084375*G0_1_0_1_0_1 - 0.0200892857142857*G0_1_0_1_0_2 - 0.084375*G0_1_0_1_1_0 - 0.216964285714285*G0_1_0_1_1_1 - 0.0120535714285714*G0_1_0_1_1_2 - 0.0200892857142857*G0_1_0_1_2_0 - 0.0120535714285714*G0_1_0_1_2_1 + 0.00803571428571426*G0_1_0_1_2_2 - 0.0180803571428571*G0_1_0_2_0_0 - 0.0200892857142857*G0_1_0_2_0_1 - 0.00602678571428572*G0_1_0_2_0_2 - 0.0200892857142857*G0_1_0_2_1_0 - 0.0120535714285714*G0_1_0_2_1_1 + 0.00803571428571426*G0_1_0_2_1_2 - 0.00602678571428572*G0_1_0_2_2_0 + 0.00803571428571426*G0_1_0_2_2_1 + 0.0180803571428569*G0_1_0_2_2_2;
    A[21] = 0.000446428571428567*G0_1_0_0_0_0 + 0.000595238095238094*G0_1_0_0_0_1 + 0.000595238095238094*G0_1_0_0_0_2 + 0.000595238095238094*G0_1_0_0_1_0 + 0.00372023809523809*G0_1_0_0_1_1 + 0.000520833333333332*G0_1_0_0_1_2 + 0.000595238095238095*G0_1_0_0_2_0 + 0.000520833333333332*G0_1_0_0_2_1 + 0.0037202380952381*G0_1_0_0_2_2 + 0.000595238095238094*G0_1_0_1_0_0 + 0.00372023809523809*G0_1_0_1_0_1 + 0.000520833333333332*G0_1_0_1_0_2 + 0.00372023809523809*G0_1_0_1_1_0 + 0.0285714285714285*G0_1_0_1_1_1 + 0.000148809523809518*G0_1_0_1_1_2 + 0.000520833333333332*G0_1_0_1_2_0 + 0.000148809523809518*G0_1_0_1_2_1 + 0.000148809523809532*G0_1_0_1_2_2 + 0.000595238095238094*G0_1_0_2_0_0 + 0.000520833333333332*G0_1_0_2_0_1 + 0.0037202380952381*G0_1_0_2_0_2 + 0.000520833333333332*G0_1_0_2_1_0 + 0.000148809523809518*G0_1_0_2_1_1 + 0.000148809523809532*G0_1_0_2_1_2 + 0.0037202380952381*G0_1_0_2_2_0 + 0.000148809523809532*G0_1_0_2_2_1 + 0.0285714285714287*G0_1_0_2_2_2;
    A[1] = -A[21] - 0.0285714285714285*G0_0_0_0_0_0 - 0.000148809523809522*G0_0_0_0_0_1 - 0.00372023809523809*G0_0_0_0_0_2 - 0.000148809523809522*G0_0_0_0_1_0 - 0.000148809523809529*G0_0_0_0_1_1 - 0.000520833333333332*G0_0_0_0_1_2 - 0.00372023809523809*G0_0_0_0_2_0 - 0.000520833333333332*G0_0_0_0_2_1 - 0.000595238095238096*G0_0_0_0_2_2 - 0.000148809523809522*G0_0_0_1_0_0 - 0.000148809523809529*G0_0_0_1_0_1 - 0.000520833333333332*G0_0_0_1_0_2 - 0.000148809523809529*G0_0_0_1_1_0 - 0.0285714285714286*G0_0_0_1_1_1 - 0.00372023809523809*G0_0_0_1_1_2 - 0.000520833333333332*G0_0_0_1_2_0 - 0.00372023809523809*G0_0_0_1_2_1 - 0.000595238095238095*G0_0_0_1_2_2 - 0.00372023809523809*G0_0_0_2_0_0 - 0.000520833333333332*G0_0_0_2_0_1 - 0.000595238095238096*G0_0_0_2_0_2 - 0.000520833333333332*G0_0_0_2_1_0 - 0.00372023809523809*G0_0_0_2_1_1 - 0.000595238095238095*G0_0_0_2_1_2 - 0.000595238095238096*G0_0_0_2_2_0 - 0.000595238095238095*G0_0_0_2_2_1 - 0.000446428571428593*G0_0_0_2_2_2 - 0.028125*G0_1_0_0_0_0 + 0.000446428571428572*G0_1_0_0_0_1 - 0.00312499999999999*G0_1_0_0_0_2 + 0.000446428571428572*G0_1_0_0_1_0 + 0.00357142857142856*G0_1_0_0_1_1 - 0.00312499999999999*G0_1_0_0_2_0 + 0.003125*G0_1_0_0_2_2 + 0.000446428571428573*G0_1_0_1_0_0 + 0.00357142857142856*G0_1_0_1_0_1 + 0.00357142857142856*G0_1_0_1_1_0 - 0.00357142857142857*G0_1_0_1_1_2 - 0.00357142857142857*G0_1_0_1_2_1 - 0.000446428571428562*G0_1_0_1_2_2 - 0.00312499999999999*G0_1_0_2_0_0 + 0.003125*G0_1_0_2_0_2 - 0.00357142857142857*G0_1_0_2_1_1 - 0.000446428571428562*G0_1_0_2_1_2 + 0.003125*G0_1_0_2_2_0 - 0.000446428571428562*G0_1_0_2_2_1 + 0.0281250000000001*G0_1_0_2_2_2;
    A[15] = A[16] - 0.0241071428571428*G0_0_0_0_0_0 + 0.00267857142857142*G0_0_0_0_0_1 - 0.00803571428571428*G0_0_0_0_0_2 + 0.00267857142857142*G0_0_0_0_1_0 + 0.00133928571428569*G0_0_0_0_1_1 + 0.000669642857142855*G0_0_0_0_1_2 - 0.00803571428571428*G0_0_0_0_2_0 + 0.000669642857142854*G0_0_0_0_2_1 + 0.00267857142857142*G0_0_0_1_0_0 + 0.00133928571428569*G0_0_0_1_0_1 + 0.000669642857142855*G0_0_0_1_0_2 + 0.00133928571428569*G0_0_0_1_1_0 - 0.00133928571428572*G0_0_0_1_1_2 + 0.000669642857142854*G0_0_0_1_2_0 - 0.00133928571428572*G0_0_0_1_2_1 - 0.00401785714285711*G0_0_0_1_2_2 - 0.00803571428571428*G0_0_0_2_0_0 + 0.000669642857142854*G0_0_0_2_0_1 + 0.000669642857142854*G0_0_0_2_1_0 - 0.00133928571428572*G0_0_0_2_1_1 - 0.00401785714285711*G0_0_0_2_1_2 - 0.00401785714285711*G0_0_0_2_2_1 + 0.048214285714286*G0_0_0_2_2_2 + 0.0241071428571428*G0_0_1_0_0_0 - 0.00133928571428572*G0_0_1_0_0_1 - 0.00803571428571428*G0_0_1_0_0_2 - 0.00133928571428572*G0_0_1_0_1_0 + 0.00133928571428571*G0_0_1_0_1_2 - 0.00803571428571428*G0_0_1_0_2_0 + 0.00133928571428571*G0_0_1_0_2_1 - 0.00803571428571424*G0_0_1_0_2_2 - 0.00133928571428572*G0_0_1_1_0_0 + 0.00133928571428571*G0_0_1_1_0_2 + 0.00133928571428571*G0_0_1_1_2_0 - 0.00133928571428569*G0_0_1_1_2_2 - 0.00803571428571428*G0_0_1_2_0_0 + 0.00133928571428571*G0_0_1_2_0_1 - 0.00803571428571424*G0_0_1_2_0_2 + 0.00133928571428571*G0_0_1_2_1_0 - 0.00133928571428569*G0_0_1_2_1_2 - 0.00803571428571424*G0_0_1_2_2_0 - 0.00133928571428569*G0_0_1_2_2_1 + 0.0241071428571432*G0_0_1_2_2_2;
    A[51] = A[15] - 0.0261160714285714*G0_0_1_0_0_0 + 0.000892857142857145*G0_0_1_0_0_1 + 0.00245535714285713*G0_0_1_0_0_2 + 0.000892857142857144*G0_0_1_0_1_0 + 0.00178571428571429*G0_0_1_0_1_1 - 0.000669642857142856*G0_0_1_0_1_2 + 0.00245535714285713*G0_0_1_0_2_0 - 0.000669642857142856*G0_0_1_0_2_1 + 0.00558035714285712*G0_0_1_0_2_2 + 0.000892857142857143*G0_0_1_1_0_0 + 0.00178571428571429*G0_0_1_1_0_1 - 0.000669642857142856*G0_0_1_1_0_2 + 0.00178571428571429*G0_0_1_1_1_0 - 0.00178571428571428*G0_0_1_1_1_2 - 0.000669642857142857*G0_0_1_1_2_0 - 0.00178571428571428*G0_0_1_1_2_1 + 0.000446428571428563*G0_0_1_1_2_2 + 0.00245535714285713*G0_0_1_2_0_0 - 0.000669642857142856*G0_0_1_2_0_1 + 0.00558035714285712*G0_0_1_2_0_2 - 0.000669642857142856*G0_0_1_2_1_0 - 0.00178571428571428*G0_0_1_2_1_1 + 0.000446428571428563*G0_0_1_2_1_2 + 0.00558035714285712*G0_0_1_2_2_0 + 0.000446428571428563*G0_0_1_2_2_1 + 0.00200892857142843*G0_0_1_2_2_2 + 0.0261160714285714*G0_1_0_0_0_0 - 0.000892857142857143*G0_1_0_0_0_1 - 0.00245535714285713*G0_1_0_0_0_2 - 0.000892857142857144*G0_1_0_0_1_0 - 0.00178571428571429*G0_1_0_0_1_1 + 0.000669642857142856*G0_1_0_0_1_2 - 0.00245535714285713*G0_1_0_0_2_0 + 0.000669642857142856*G0_1_0_0_2_1 - 0.00558035714285712*G0_1_0_0_2_2 - 0.000892857142857143*G0_1_0_1_0_0 - 0.00178571428571429*G0_1_0_1_0_1 + 0.000669642857142856*G0_1_0_1_0_2 - 0.00178571428571429*G0_1_0_1_1_0 + 0.00178571428571428*G0_1_0_1_1_2 + 0.000669642857142857*G0_1_0_1_2_0 + 0.00178571428571428*G0_1_0_1_2_1 - 0.000446428571428563*G0_1_0_1_2_2 - 0.00245535714285713*G0_1_0_2_0_0 + 0.000669642857142856*G0_1_0_2_0_1 - 0.00558035714285712*G0_1_0_2_0_2 + 0.000669642857142857*G0_1_0_2_1_0 + 0.00178571428571428*G0_1_0_2_1_1 - 0.000446428571428563*G0_1_0_2_1_2 - 0.00558035714285712*G0_1_0_2_2_0 - 0.000446428571428563*G0_1_0_2_2_1 - 0.00200892857142843*G0_1_0_2_2_2;
    A[31] = -A[51] - 0.0361607142857143*G0_0_0_0_0_0 + 0.00133928571428572*G0_0_0_0_0_1 - 0.01875*G0_0_0_0_0_2 + 0.00133928571428571*G0_0_0_0_1_0 + 0.0107142857142857*G0_0_0_0_1_1 - 0.01875*G0_0_0_0_2_0 - 0.0200892857142857*G0_0_0_0_2_2 + 0.00133928571428571*G0_0_0_1_0_0 + 0.0107142857142857*G0_0_0_1_0_1 + 0.0107142857142857*G0_0_0_1_1_0 + 0.120535714285714*G0_0_0_1_1_1 + 0.01875*G0_0_0_1_1_2 + 0.01875*G0_0_0_1_2_1 - 0.00401785714285713*G0_0_0_1_2_2 - 0.01875*G0_0_0_2_0_0 - 0.0200892857142857*G0_0_0_2_0_2 + 0.01875*G0_0_0_2_1_1 - 0.00401785714285713*G0_0_0_2_1_2 - 0.0200892857142857*G0_0_0_2_2_0 - 0.00401785714285713*G0_0_0_2_2_1 - 0.0482142857142856*G0_0_0_2_2_2 + 0.0267857142857142*G0_1_0_0_0_0 + 0.00624999999999998*G0_1_0_0_0_1 - 0.00223214285714286*G0_1_0_0_0_2 + 0.00624999999999998*G0_1_0_0_1_0 + 0.0370535714285714*G0_1_0_0_1_1 + 0.00424107142857142*G0_1_0_0_1_2 - 0.00223214285714286*G0_1_0_0_2_0 + 0.00424107142857142*G0_1_0_0_2_1 - 0.00535714285714283*G0_1_0_0_2_2 + 0.00624999999999998*G0_1_0_1_0_0 + 0.0370535714285714*G0_1_0_1_0_1 + 0.00424107142857142*G0_1_0_1_0_2 + 0.0370535714285714*G0_1_0_1_1_0 + 0.412499999999999*G0_1_0_1_1_1 + 0.0406249999999999*G0_1_0_1_1_2 + 0.00424107142857142*G0_1_0_1_2_0 + 0.0406249999999999*G0_1_0_1_2_1 + 0.00669642857142857*G0_1_0_1_2_2 - 0.00223214285714286*G0_1_0_2_0_0 + 0.00424107142857142*G0_1_0_2_0_1 - 0.00535714285714283*G0_1_0_2_0_2 + 0.00424107142857142*G0_1_0_2_1_0 + 0.0406249999999999*G0_1_0_2_1_1 + 0.00669642857142857*G0_1_0_2_1_2 - 0.00535714285714283*G0_1_0_2_2_0 + 0.00669642857142857*G0_1_0_2_2_1 - 0.00133928571428558*G0_1_0_2_2_2;
    A[22] = 0.0125*G0_1_1_0_0_0 + 0.00416666666666666*G0_1_1_0_0_1 + 0.0050595238095238*G0_1_1_0_0_2 + 0.00416666666666666*G0_1_1_0_1_0 + 0.00416666666666666*G0_1_1_0_1_1 + 0.0025297619047619*G0_1_1_0_1_2 + 0.0050595238095238*G0_1_1_0_2_0 + 0.0025297619047619*G0_1_1_0_2_1 + 0.0193452380952381*G0_1_1_0_2_2 + 0.00416666666666666*G0_1_1_1_0_0 + 0.00416666666666666*G0_1_1_1_0_1 + 0.0025297619047619*G0_1_1_1_0_2 + 0.00416666666666666*G0_1_1_1_1_0 + 0.0125*G0_1_1_1_1_1 + 0.00505952380952381*G0_1_1_1_1_2 + 0.0025297619047619*G0_1_1_1_2_0 + 0.00505952380952381*G0_1_1_1_2_1 + 0.0193452380952381*G0_1_1_1_2_2 + 0.0050595238095238*G0_1_1_2_0_0 + 0.0025297619047619*G0_1_1_2_0_1 + 0.0193452380952381*G0_1_1_2_0_2 + 0.0025297619047619*G0_1_1_2_1_0 + 0.00505952380952381*G0_1_1_2_1_1 + 0.0193452380952381*G0_1_1_2_1_2 + 0.0193452380952381*G0_1_1_2_2_0 + 0.0193452380952381*G0_1_1_2_2_1 + 0.213392857142857*G0_1_1_2_2_2;
    A[35] = -A[85] + 0.114508928571429*G0_0_0_0_0_0 - 0.0180803571428571*G0_0_0_0_0_1 + 0.0522321428571429*G0_0_0_0_0_2 - 0.0180803571428571*G0_0_0_0_1_0 - 0.0200892857142857*G0_0_0_0_1_1 - 0.00602678571428569*G0_0_0_0_1_2 + 0.0522321428571429*G0_0_0_0_2_0 - 0.00602678571428569*G0_0_0_0_2_1 + 0.0421874999999999*G0_0_0_0_2_2 - 0.0180803571428571*G0_0_0_1_0_0 - 0.0200892857142857*G0_0_0_1_0_1 - 0.00602678571428569*G0_0_0_1_0_2 - 0.0200892857142857*G0_0_0_1_1_0 - 0.0120535714285712*G0_0_0_1_1_1 + 0.00803571428571429*G0_0_0_1_1_2 - 0.00602678571428569*G0_0_0_1_2_0 + 0.00803571428571429*G0_0_0_1_2_1 + 0.0180803571428571*G0_0_0_1_2_2 + 0.0522321428571429*G0_0_0_2_0_0 - 0.00602678571428569*G0_0_0_2_0_1 + 0.0421874999999999*G0_0_0_2_0_2 - 0.00602678571428569*G0_0_0_2_1_0 + 0.00803571428571429*G0_0_0_2_1_1 + 0.0180803571428571*G0_0_0_2_1_2 + 0.0421874999999999*G0_0_0_2_2_0 + 0.0180803571428571*G0_0_0_2_2_1 + 0.0241071428571424*G0_0_0_2_2_2 - 0.1265625*G0_0_1_0_0_0 + 0.0100446428571429*G0_0_1_0_0_2 - 0.00401785714285712*G0_0_1_0_1_1 - 0.0060267857142857*G0_0_1_0_1_2 + 0.0100446428571429*G0_0_1_0_2_0 - 0.0060267857142857*G0_0_1_0_2_1 + 0.0261160714285714*G0_0_1_0_2_2 - 0.00401785714285712*G0_0_1_1_0_1 - 0.0060267857142857*G0_0_1_1_0_2 - 0.00401785714285712*G0_0_1_1_1_0 + 0.00401785714285714*G0_0_1_1_1_2 - 0.0060267857142857*G0_0_1_1_2_0 + 0.00401785714285714*G0_0_1_1_2_1 + 0.0120535714285714*G0_0_1_1_2_2 + 0.0100446428571429*G0_0_1_2_0_0 - 0.0060267857142857*G0_0_1_2_0_1 + 0.0261160714285714*G0_0_1_2_0_2 - 0.00602678571428569*G0_0_1_2_1_0 + 0.00401785714285714*G0_0_1_2_1_1 + 0.0120535714285714*G0_0_1_2_1_2 + 0.0261160714285714*G0_0_1_2_2_0 + 0.0120535714285714*G0_0_1_2_2_1 + 0.0180803571428567*G0_0_1_2_2_2;
    A[86] = -A[35] + 0.0361607142857144*G0_0_0_0_0_0 + 0.028125*G0_0_0_0_0_2 - 0.00200892857142854*G0_0_0_0_1_1 + 0.028125*G0_0_0_0_2_0 + 0.0301339285714286*G0_0_0_0_2_2 - 0.00200892857142854*G0_0_0_1_0_1 - 0.00200892857142854*G0_0_0_1_1_0 + 0.0180803571428571*G0_0_0_1_1_1 + 0.0200892857142857*G0_0_0_1_1_2 + 0.0200892857142857*G0_0_0_1_2_1 + 0.0180803571428571*G0_0_0_1_2_2 + 0.028125*G0_0_0_2_0_0 + 0.0301339285714285*G0_0_0_2_0_2 + 0.0200892857142857*G0_0_0_2_1_1 + 0.0180803571428571*G0_0_0_2_1_2 + 0.0301339285714285*G0_0_0_2_2_0 + 0.0180803571428571*G0_0_0_2_2_1 - 0.0180803571428572*G0_0_1_0_0_1 - 0.0020089285714286*G0_0_1_0_0_2 - 0.0180803571428572*G0_0_1_0_1_0 - 0.0160714285714286*G0_0_1_0_1_1 - 0.0020089285714286*G0_0_1_0_2_0 + 0.00803571428571427*G0_0_1_0_2_2 - 0.0180803571428572*G0_0_1_1_0_0 - 0.0160714285714286*G0_0_1_1_0_1 - 0.0160714285714286*G0_0_1_1_1_0 + 0.0160714285714285*G0_0_1_1_1_2 + 0.0160714285714285*G0_0_1_1_2_1 + 0.0180803571428571*G0_0_1_1_2_2 - 0.0020089285714286*G0_0_1_2_0_0 + 0.00803571428571426*G0_0_1_2_0_2 + 0.0160714285714285*G0_0_1_2_1_1 + 0.0180803571428571*G0_0_1_2_1_2 + 0.00803571428571426*G0_0_1_2_2_0 + 0.0180803571428571*G0_0_1_2_2_1 - 0.0180803571428572*G0_0_1_2_2_2 + 0.0180803571428571*G0_1_0_0_0_0 + 0.00602678571428569*G0_1_0_0_0_2 - 0.00602678571428576*G0_1_0_0_1_1 + 0.00602678571428569*G0_1_0_0_2_0 - 0.00602678571428576*G0_1_0_1_0_1 - 0.00602678571428576*G0_1_0_1_1_0 + 0.00602678571428568*G0_1_0_1_1_2 + 0.00602678571428569*G0_1_0_1_2_1 + 0.00602678571428569*G0_1_0_2_0_0 + 0.00602678571428569*G0_1_0_2_1_1 - 0.036160714285714*G0_1_0_2_2_2 - 0.0180803571428569*G0_1_1_0_0_0 + 0.00602678571428571*G0_1_1_0_0_2 + 0.00602678571428571*G0_1_1_0_2_0 + 0.0060267857142857*G0_1_1_0_2_2 + 0.00602678571428571*G0_1_1_2_0_0 + 0.00602678571428569*G0_1_1_2_0_2 + 0.0060267857142857*G0_1_1_2_2_0 - 0.018080357142857*G0_1_1_2_2_2;
    A[36] = -A[86] + 0.00602678571428572*G0_0_0_0_0_0 + 0.00602678571428574*G0_0_0_0_0_1 + 0.0160714285714286*G0_0_0_0_0_2 + 0.00602678571428574*G0_0_0_0_1_0 + 0.00401785714285716*G0_0_0_0_1_1 + 0.0160714285714286*G0_0_0_0_2_0 + 0.0421875*G0_0_0_0_2_2 + 0.00602678571428574*G0_0_0_1_0_0 + 0.00401785714285716*G0_0_0_1_0_1 + 0.00401785714285716*G0_0_0_1_1_0 - 0.0120535714285715*G0_0_0_1_1_1 - 0.0160714285714286*G0_0_0_1_1_2 - 0.0160714285714286*G0_0_0_1_2_1 - 0.018080357142857*G0_0_0_1_2_2 + 0.0160714285714286*G0_0_0_2_0_0 + 0.0421875*G0_0_0_2_0_2 - 0.0160714285714286*G0_0_0_2_1_1 - 0.018080357142857*G0_0_0_2_1_2 + 0.0421875*G0_0_0_2_2_0 - 0.018080357142857*G0_0_0_2_2_1 + 0.241071428571429*G0_0_0_2_2_2 - 0.0180803571428572*G0_0_1_0_0_0 - 0.0120535714285714*G0_0_1_0_0_1 - 0.0261160714285714*G0_0_1_0_0_2 - 0.0120535714285714*G0_0_1_0_1_0 - 0.00401785714285714*G0_0_1_0_1_1 + 0.00602678571428571*G0_0_1_0_1_2 - 0.0261160714285714*G0_0_1_0_2_0 + 0.00602678571428571*G0_0_1_0_2_1 - 0.0100446428571428*G0_0_1_0_2_2 - 0.0120535714285714*G0_0_1_1_0_0 - 0.00401785714285714*G0_0_1_1_0_1 + 0.00602678571428571*G0_0_1_1_0_2 - 0.00401785714285714*G0_0_1_1_1_0 + 0.00401785714285715*G0_0_1_1_1_2 + 0.00602678571428571*G0_0_1_1_2_0 + 0.00401785714285716*G0_0_1_1_2_1 - 0.0261160714285714*G0_0_1_2_0_0 + 0.00602678571428571*G0_0_1_2_0_1 - 0.0100446428571428*G0_0_1_2_0_2 + 0.00602678571428571*G0_0_1_2_1_0 + 0.00401785714285716*G0_0_1_2_1_1 - 0.0100446428571428*G0_0_1_2_2_0 + 0.126562500000001*G0_0_1_2_2_2;
    A[64] = A[35] - 0.0482142857142858*G0_0_0_0_0_0 - 0.0482142857142857*G0_0_0_0_0_2 - 0.0120535714285714*G0_0_0_0_1_2 - 0.0482142857142857*G0_0_0_0_2_0 - 0.0120535714285714*G0_0_0_0_2_1 - 0.120535714285714*G0_0_0_0_2_2 - 0.0120535714285714*G0_0_0_1_0_2 - 0.0482142857142857*G0_0_0_1_1_1 - 0.0482142857142857*G0_0_0_1_1_2 - 0.0120535714285714*G0_0_0_1_2_0 - 0.0482142857142857*G0_0_0_1_2_1 - 0.120535714285714*G0_0_0_1_2_2 - 0.0482142857142857*G0_0_0_2_0_0 - 0.0120535714285714*G0_0_0_2_0_1 - 0.120535714285714*G0_0_0_2_0_2 - 0.0120535714285714*G0_0_0_2_1_0 - 0.0482142857142857*G0_0_0_2_1_1 - 0.120535714285714*G0_0_0_2_1_2 - 0.120535714285714*G0_0_0_2_2_0 - 0.120535714285714*G0_0_0_2_2_1 - 0.84375*G0_0_0_2_2_2 - 0.00602678571428561*G0_0_1_0_0_0 - 0.0220982142857143*G0_0_1_0_0_2 - 0.00803571428571429*G0_0_1_0_1_2 - 0.0220982142857143*G0_0_1_0_2_0 - 0.00803571428571429*G0_0_1_0_2_1 - 0.0542410714285714*G0_0_1_0_2_2 - 0.00803571428571429*G0_0_1_1_0_2 - 0.0421875000000001*G0_0_1_1_1_1 - 0.0462053571428571*G0_0_1_1_1_2 - 0.00803571428571429*G0_0_1_1_2_0 - 0.0462053571428571*G0_0_1_1_2_1 - 0.0904017857142857*G0_0_1_1_2_2 - 0.0220982142857143*G0_0_1_2_0_0 - 0.00803571428571429*G0_0_1_2_0_1 - 0.0542410714285714*G0_0_1_2_0_2 - 0.00803571428571429*G0_0_1_2_1_0 - 0.0462053571428571*G0_0_1_2_1_1 - 0.0904017857142857*G0_0_1_2_1_2 - 0.0542410714285714*G0_0_1_2_2_0 - 0.0904017857142857*G0_0_1_2_2_1 - 0.277232142857143*G0_0_1_2_2_2 - 0.00602678571428557*G0_1_0_0_0_0 - 0.00200892857142856*G0_1_0_0_0_2 - 0.00401785714285714*G0_1_0_0_1_2 - 0.00200892857142857*G0_1_0_0_2_0 - 0.00401785714285714*G0_1_0_0_2_1 - 0.0301339285714287*G0_1_0_0_2_2 - 0.00401785714285714*G0_1_0_1_0_2 - 0.0421874999999997*G0_1_0_1_1_1 - 0.0261160714285714*G0_1_0_1_1_2 - 0.00401785714285714*G0_1_0_1_2_0 - 0.0261160714285714*G0_1_0_1_2_1 - 0.0662946428571429*G0_1_0_1_2_2 - 0.00200892857142856*G0_1_0_2_0_0 - 0.00401785714285714*G0_1_0_2_0_1 - 0.0301339285714287*G0_1_0_2_0_2 - 0.00401785714285714*G0_1_0_2_1_0 - 0.0261160714285714*G0_1_0_2_1_1 - 0.066294642857143*G0_1_0_2_1_2 - 0.0301339285714287*G0_1_0_2_2_0 - 0.0662946428571429*G0_1_0_2_2_1 - 0.566517857142858*G0_1_0_2_2_2 + 0.0602678571428571*G0_1_1_0_0_0 + 0.0241071428571428*G0_1_1_0_0_1 + 0.0120535714285714*G0_1_1_0_0_2 + 0.0241071428571428*G0_1_1_0_1_0 + 0.0241071428571428*G0_1_1_0_1_1 + 0.00602678571428571*G0_1_1_0_1_2 + 0.0120535714285714*G0_1_1_0_2_0 + 0.00602678571428571*G0_1_1_0_2_1 + 0.0241071428571428*G0_1_1_1_0_0 + 0.0241071428571428*G0_1_1_1_0_1 + 0.00602678571428571*G0_1_1_1_0_2 + 0.0241071428571428*G0_1_1_1_1_0 + 0.0241071428571429*G0_1_1_1_1_1 - 0.0120535714285714*G0_1_1_1_1_2 + 0.00602678571428571*G0_1_1_1_2_0 - 0.0120535714285714*G0_1_1_1_2_1 - 0.0361607142857143*G0_1_1_1_2_2 + 0.0120535714285714*G0_1_1_2_0_0 + 0.00602678571428571*G0_1_1_2_0_1 + 0.00602678571428571*G0_1_1_2_1_0 - 0.0120535714285714*G0_1_1_2_1_1 - 0.0361607142857143*G0_1_1_2_1_2 - 0.0361607142857143*G0_1_1_2_2_1 - 0.156696428571429*G0_1_1_2_2_2;
    A[46] = A[64] + 0.0180803571428573*G0_0_1_0_0_0 + 0.00602678571428575*G0_0_1_0_0_1 + 0.0160714285714285*G0_0_1_0_0_2 + 0.00602678571428575*G0_0_1_0_1_0 + 0.00602678571428575*G0_0_1_0_1_1 + 0.00803571428571428*G0_0_1_0_1_2 + 0.0160714285714285*G0_0_1_0_2_0 + 0.00803571428571428*G0_0_1_0_2_1 + 0.0120535714285713*G0_0_1_0_2_2 + 0.00602678571428575*G0_0_1_1_0_0 + 0.00602678571428575*G0_0_1_1_0_1 + 0.00803571428571428*G0_0_1_1_0_2 + 0.00602678571428575*G0_0_1_1_1_0 + 0.0180803571428573*G0_0_1_1_1_1 + 0.0160714285714286*G0_0_1_1_1_2 + 0.00803571428571428*G0_0_1_1_2_0 + 0.0160714285714286*G0_0_1_1_2_1 + 0.0120535714285713*G0_0_1_1_2_2 + 0.0160714285714285*G0_0_1_2_0_0 + 0.00803571428571428*G0_0_1_2_0_1 + 0.0120535714285713*G0_0_1_2_0_2 + 0.00803571428571428*G0_0_1_2_1_0 + 0.0160714285714286*G0_0_1_2_1_1 + 0.0120535714285713*G0_0_1_2_1_2 + 0.0120535714285713*G0_0_1_2_2_0 + 0.0120535714285713*G0_0_1_2_2_1 - 0.289285714285715*G0_0_1_2_2_2 - 0.0180803571428573*G0_1_0_0_0_0 - 0.00602678571428575*G0_1_0_0_0_1 - 0.0160714285714285*G0_1_0_0_0_2 - 0.00602678571428575*G0_1_0_0_1_0 - 0.00602678571428575*G0_1_0_0_1_1 - 0.00803571428571428*G0_1_0_0_1_2 - 0.0160714285714285*G0_1_0_0_2_0 - 0.00803571428571428*G0_1_0_0_2_1 - 0.0120535714285713*G0_1_0_0_2_2 - 0.00602678571428575*G0_1_0_1_0_0 - 0.00602678571428575*G0_1_0_1_0_1 - 0.00803571428571428*G0_1_0_1_0_2 - 0.00602678571428575*G0_1_0_1_1_0 - 0.0180803571428573*G0_1_0_1_1_1 - 0.0160714285714286*G0_1_0_1_1_2 - 0.00803571428571427*G0_1_0_1_2_0 - 0.0160714285714286*G0_1_0_1_2_1 - 0.0120535714285713*G0_1_0_1_2_2 - 0.0160714285714285*G0_1_0_2_0_0 - 0.00803571428571428*G0_1_0_2_0_1 - 0.0120535714285713*G0_1_0_2_0_2 - 0.00803571428571427*G0_1_0_2_1_0 - 0.0160714285714286*G0_1_0_2_1_1 - 0.0120535714285713*G0_1_0_2_1_2 - 0.0120535714285713*G0_1_0_2_2_0 - 0.0120535714285713*G0_1_0_2_2_1 + 0.289285714285715*G0_1_0_2_2_2;
    A[92] = -0.0120535714285714*G0_0_1_0_0_0 - 0.00133928571428571*G0_0_1_0_0_1 - 0.00535714285714286*G0_0_1_0_0_2 - 0.00133928571428571*G0_0_1_0_1_0 + 0.00133928571428572*G0_0_1_0_1_1 - 0.00535714285714286*G0_0_1_0_2_0 + 0.00803571428571426*G0_0_1_0_2_2 - 0.00133928571428571*G0_0_1_1_0_0 + 0.00133928571428572*G0_0_1_1_0_1 + 0.00133928571428572*G0_0_1_1_1_0 + 0.0120535714285714*G0_0_1_1_1_1 + 0.00535714285714284*G0_0_1_1_1_2 + 0.00535714285714284*G0_0_1_1_2_1 - 0.0080357142857143*G0_0_1_1_2_2 - 0.00535714285714286*G0_0_1_2_0_0 + 0.00803571428571426*G0_0_1_2_0_2 + 0.00535714285714284*G0_0_1_2_1_1 - 0.0080357142857143*G0_0_1_2_1_2 + 0.00803571428571426*G0_0_1_2_2_0 - 0.0080357142857143*G0_0_1_2_2_1 + 0.0361607142857143*G0_1_1_0_0_0 + 0.01875*G0_1_1_0_0_1 - 0.00133928571428572*G0_1_1_0_0_2 + 0.01875*G0_1_1_0_1_0 + 0.0200892857142857*G0_1_1_0_1_1 - 0.00133928571428572*G0_1_1_0_2_0 - 0.0107142857142857*G0_1_1_0_2_2 + 0.01875*G0_1_1_1_0_0 + 0.0200892857142857*G0_1_1_1_0_1 + 0.0200892857142857*G0_1_1_1_1_0 + 0.0482142857142857*G0_1_1_1_1_1 + 0.00401785714285712*G0_1_1_1_1_2 + 0.00401785714285712*G0_1_1_1_2_1 - 0.01875*G0_1_1_1_2_2 - 0.00133928571428572*G0_1_1_2_0_0 - 0.0107142857142857*G0_1_1_2_0_2 + 0.00401785714285712*G0_1_1_2_1_1 - 0.01875*G0_1_1_2_1_2 - 0.0107142857142857*G0_1_1_2_2_0 - 0.01875*G0_1_1_2_2_1 - 0.120535714285714*G0_1_1_2_2_2;
    A[61] = A[51] + 0.0241071428571428*G0_0_0_0_0_0 - 0.00267857142857142*G0_0_0_0_0_1 + 0.00803571428571428*G0_0_0_0_0_2 - 0.00267857142857142*G0_0_0_0_1_0 - 0.00133928571428569*G0_0_0_0_1_1 - 0.000669642857142855*G0_0_0_0_1_2 + 0.00803571428571428*G0_0_0_0_2_0 - 0.000669642857142854*G0_0_0_0_2_1 - 0.00267857142857142*G0_0_0_1_0_0 - 0.00133928571428569*G0_0_0_1_0_1 - 0.000669642857142855*G0_0_0_1_0_2 - 0.00133928571428569*G0_0_0_1_1_0 + 0.00133928571428572*G0_0_0_1_1_2 - 0.000669642857142855*G0_0_0_1_2_0 + 0.00133928571428572*G0_0_0_1_2_1 + 0.00401785714285711*G0_0_0_1_2_2 + 0.00803571428571428*G0_0_0_2_0_0 - 0.000669642857142854*G0_0_0_2_0_1 - 0.000669642857142854*G0_0_0_2_1_0 + 0.00133928571428572*G0_0_0_2_1_1 + 0.00401785714285711*G0_0_0_2_1_2 + 0.00401785714285711*G0_0_0_2_2_1 - 0.048214285714286*G0_0_0_2_2_2 - 0.0241071428571428*G0_1_0_0_0_0 + 0.00133928571428572*G0_1_0_0_0_1 + 0.00803571428571428*G0_1_0_0_0_2 + 0.00133928571428572*G0_1_0_0_1_0 - 0.00133928571428571*G0_1_0_0_1_2 + 0.00803571428571428*G0_1_0_0_2_0 - 0.00133928571428571*G0_1_0_0_2_1 + 0.00803571428571424*G0_1_0_0_2_2 + 0.00133928571428572*G0_1_0_1_0_0 - 0.00133928571428571*G0_1_0_1_0_2 - 0.00133928571428571*G0_1_0_1_2_0 + 0.00133928571428569*G0_1_0_1_2_2 + 0.00803571428571428*G0_1_0_2_0_0 - 0.00133928571428571*G0_1_0_2_0_1 + 0.00803571428571424*G0_1_0_2_0_2 - 0.00133928571428571*G0_1_0_2_1_0 + 0.00133928571428569*G0_1_0_2_1_2 + 0.00803571428571424*G0_1_0_2_2_0 + 0.00133928571428569*G0_1_0_2_2_1 - 0.0241071428571432*G0_1_0_2_2_2;
    A[41] = -A[61] + 0.00267857142857143*G0_1_0_0_0_0 - 0.00312499999999998*G0_1_0_0_0_1 + 0.00446428571428571*G0_1_0_0_0_2 - 0.00312499999999999*G0_1_0_0_1_0 - 0.0191964285714285*G0_1_0_0_1_1 - 0.00379464285714284*G0_1_0_0_1_2 + 0.00446428571428571*G0_1_0_0_2_0 - 0.00379464285714284*G0_1_0_0_2_1 - 0.00312499999999999*G0_1_0_1_0_0 - 0.0191964285714285*G0_1_0_1_0_1 - 0.00379464285714285*G0_1_0_1_0_2 - 0.0191964285714285*G0_1_0_1_1_0 - 0.141964285714285*G0_1_0_1_1_1 - 0.00758928571428568*G0_1_0_1_1_2 - 0.00379464285714284*G0_1_0_1_2_0 - 0.00758928571428568*G0_1_0_1_2_1 - 0.00803571428571428*G0_1_0_1_2_2 + 0.00446428571428571*G0_1_0_2_0_0 - 0.00379464285714284*G0_1_0_2_0_1 - 0.00379464285714284*G0_1_0_2_1_0 - 0.00758928571428568*G0_1_0_2_1_1 - 0.00803571428571428*G0_1_0_2_1_2 - 0.00803571428571428*G0_1_0_2_2_1 - 0.0375000000000001*G0_1_0_2_2_2;
    A[40] = -A[41] + 0.0180803571428572*G0_0_0_0_0_0 + 0.00200892857142858*G0_0_0_0_0_1 + 0.00803571428571428*G0_0_0_0_0_2 + 0.00200892857142858*G0_0_0_0_1_0 - 0.00200892857142857*G0_0_0_0_1_1 + 0.00803571428571428*G0_0_0_0_2_0 + 0.0120535714285714*G0_0_0_0_2_2 + 0.00200892857142858*G0_0_0_1_0_0 - 0.00200892857142857*G0_0_0_1_0_1 - 0.00200892857142857*G0_0_0_1_1_0 - 0.0180803571428572*G0_0_0_1_1_1 - 0.00803571428571428*G0_0_0_1_1_2 - 0.00803571428571428*G0_0_0_1_2_1 - 0.0120535714285714*G0_0_0_1_2_2 + 0.00803571428571428*G0_0_0_2_0_0 + 0.0120535714285714*G0_0_0_2_0_2 - 0.00803571428571428*G0_0_0_2_1_1 - 0.0120535714285714*G0_0_0_2_1_2 + 0.0120535714285714*G0_0_0_2_2_0 - 0.0120535714285714*G0_0_0_2_2_1 + 0.0174107142857144*G0_0_1_0_0_0 + 0.0017857142857143*G0_0_1_0_0_1 + 0.00491071428571428*G0_0_1_0_0_2 + 0.0017857142857143*G0_0_1_0_1_0 - 0.000223214285714284*G0_0_1_0_1_1 + 0.00044642857142857*G0_0_1_0_1_2 + 0.00491071428571428*G0_0_1_0_2_0 + 0.00044642857142857*G0_0_1_0_2_1 + 0.00334821428571426*G0_0_1_0_2_2 + 0.0017857142857143*G0_0_1_1_0_0 - 0.000223214285714284*G0_0_1_1_0_1 + 0.00044642857142857*G0_0_1_1_0_2 - 0.000223214285714284*G0_0_1_1_1_0 - 0.00066964285714287*G0_0_1_1_1_1 - 0.00312500000000001*G0_0_1_1_1_2 + 0.00044642857142857*G0_0_1_1_2_0 - 0.00312500000000001*G0_0_1_1_2_1 - 0.00870535714285717*G0_0_1_1_2_2 + 0.00491071428571428*G0_0_1_2_0_0 + 0.00044642857142857*G0_0_1_2_0_1 + 0.00334821428571426*G0_0_1_2_0_2 + 0.00044642857142857*G0_0_1_2_1_0 - 0.00312500000000001*G0_0_1_2_1_1 - 0.00870535714285717*G0_0_1_2_1_2 + 0.00334821428571426*G0_0_1_2_2_0 - 0.00870535714285717*G0_0_1_2_2_1 - 0.0508928571428574*G0_0_1_2_2_2 + 0.0180803571428573*G0_1_0_0_0_0 + 0.00200892857142858*G0_1_0_0_0_2 - 0.0180803571428571*G0_1_0_0_1_1 - 0.00200892857142856*G0_1_0_0_1_2 + 0.00200892857142858*G0_1_0_0_2_0 - 0.00200892857142856*G0_1_0_0_2_1 - 0.0180803571428571*G0_1_0_1_0_1 - 0.00200892857142856*G0_1_0_1_0_2 - 0.0180803571428571*G0_1_0_1_1_0 - 0.144642857142857*G0_1_0_1_1_1 - 0.0180803571428571*G0_1_0_1_1_2 - 0.00200892857142856*G0_1_0_1_2_0 - 0.0180803571428571*G0_1_0_1_2_1 - 0.0200892857142857*G0_1_0_1_2_2 + 0.00200892857142858*G0_1_0_2_0_0 - 0.00200892857142856*G0_1_0_2_0_1 - 0.00200892857142856*G0_1_0_2_1_0 - 0.0180803571428571*G0_1_0_2_1_1 - 0.0200892857142857*G0_1_0_2_1_2 - 0.0200892857142857*G0_1_0_2_2_1 - 0.0361607142857143*G0_1_0_2_2_2 + 0.0174107142857145*G0_1_1_0_0_0 + 0.00357142857142859*G0_1_1_0_0_1 + 0.00312500000000001*G0_1_1_0_0_2 + 0.00357142857142859*G0_1_1_0_1_0 - 0.000669642857142848*G0_1_1_0_1_1 + 0.00111607142857143*G0_1_1_0_1_2 + 0.00312500000000001*G0_1_1_0_2_0 + 0.00111607142857143*G0_1_1_0_2_1 + 0.00245535714285713*G0_1_1_0_2_2 + 0.00357142857142859*G0_1_1_1_0_0 - 0.000669642857142847*G0_1_1_1_0_1 + 0.00111607142857143*G0_1_1_1_0_2 - 0.000669642857142847*G0_1_1_1_1_0 - 0.0026785714285714*G0_1_1_1_1_1 - 0.00870535714285712*G0_1_1_1_1_2 + 0.00111607142857143*G0_1_1_1_2_0 - 0.00870535714285712*G0_1_1_1_2_1 - 0.0111607142857143*G0_1_1_1_2_2 + 0.00312500000000001*G0_1_1_2_0_0 + 0.00111607142857143*G0_1_1_2_0_1 + 0.00245535714285713*G0_1_1_2_0_2 + 0.00111607142857143*G0_1_1_2_1_0 - 0.00870535714285712*G0_1_1_2_1_1 - 0.0111607142857143*G0_1_1_2_1_2 + 0.00245535714285713*G0_1_1_2_2_0 - 0.0111607142857143*G0_1_1_2_2_1 - 0.0247767857142858*G0_1_1_2_2_2;
    A[4] = A[40] + 0.00178571428571429*G0_0_1_0_0_1 - 0.00178571428571428*G0_0_1_0_0_2 + 0.00178571428571429*G0_0_1_0_1_0 - 0.000446428571428566*G0_0_1_0_1_1 + 0.000669642857142856*G0_0_1_0_1_2 - 0.00178571428571428*G0_0_1_0_2_0 + 0.000669642857142856*G0_0_1_0_2_1 - 0.000892857142857134*G0_0_1_0_2_2 + 0.00178571428571429*G0_0_1_1_0_0 - 0.000446428571428565*G0_0_1_1_0_1 + 0.000669642857142856*G0_0_1_1_0_2 - 0.000446428571428565*G0_0_1_1_1_0 - 0.00200892857142851*G0_0_1_1_1_1 - 0.00558035714285712*G0_0_1_1_1_2 + 0.000669642857142856*G0_0_1_1_2_0 - 0.00558035714285712*G0_0_1_1_2_1 - 0.00245535714285712*G0_0_1_1_2_2 - 0.00178571428571428*G0_0_1_2_0_0 + 0.000669642857142856*G0_0_1_2_0_1 - 0.000892857142857133*G0_0_1_2_0_2 + 0.000669642857142856*G0_0_1_2_1_0 - 0.00558035714285712*G0_0_1_2_1_1 - 0.00245535714285712*G0_0_1_2_1_2 - 0.000892857142857134*G0_0_1_2_2_0 - 0.00245535714285712*G0_0_1_2_2_1 + 0.0261160714285716*G0_0_1_2_2_2 - 0.00178571428571429*G0_1_0_0_0_1 + 0.00178571428571428*G0_1_0_0_0_2 - 0.00178571428571429*G0_1_0_0_1_0 + 0.000446428571428565*G0_1_0_0_1_1 - 0.000669642857142856*G0_1_0_0_1_2 + 0.00178571428571428*G0_1_0_0_2_0 - 0.000669642857142856*G0_1_0_0_2_1 + 0.000892857142857136*G0_1_0_0_2_2 - 0.00178571428571429*G0_1_0_1_0_0 + 0.000446428571428564*G0_1_0_1_0_1 - 0.000669642857142856*G0_1_0_1_0_2 + 0.000446428571428565*G0_1_0_1_1_0 + 0.00200892857142851*G0_1_0_1_1_1 + 0.00558035714285712*G0_1_0_1_1_2 - 0.000669642857142856*G0_1_0_1_2_0 + 0.00558035714285712*G0_1_0_1_2_1 + 0.00245535714285712*G0_1_0_1_2_2 + 0.00178571428571428*G0_1_0_2_0_0 - 0.000669642857142856*G0_1_0_2_0_1 + 0.000892857142857133*G0_1_0_2_0_2 - 0.000669642857142856*G0_1_0_2_1_0 + 0.00558035714285712*G0_1_0_2_1_1 + 0.00245535714285712*G0_1_0_2_1_2 + 0.000892857142857135*G0_1_0_2_2_0 + 0.00245535714285712*G0_1_0_2_2_1 - 0.0261160714285716*G0_1_0_2_2_2;
    A[7] = -A[4] - 0.291964285714285*G0_0_0_0_0_0 - 0.0218749999999999*G0_0_0_0_0_1 - 0.0263392857142856*G0_0_0_0_0_2 - 0.0218749999999999*G0_0_0_0_1_0 - 0.0107142857142857*G0_0_0_0_1_1 - 0.00424107142857142*G0_0_0_0_1_2 - 0.0263392857142856*G0_0_0_0_2_0 - 0.00424107142857142*G0_0_0_0_2_1 - 0.00491071428571429*G0_0_0_0_2_2 - 0.0218749999999999*G0_0_0_1_0_0 - 0.0107142857142857*G0_0_0_1_0_1 - 0.00424107142857142*G0_0_0_1_0_2 - 0.0107142857142857*G0_0_0_1_1_0 - 0.046875*G0_0_0_1_1_1 - 0.0147321428571428*G0_0_0_1_1_2 - 0.00424107142857142*G0_0_0_1_2_0 - 0.0147321428571428*G0_0_0_1_2_1 - 0.0165178571428572*G0_0_0_1_2_2 - 0.0263392857142856*G0_0_0_2_0_0 - 0.00424107142857142*G0_0_0_2_0_1 - 0.0049107142857143*G0_0_0_2_0_2 - 0.00424107142857142*G0_0_0_2_1_0 - 0.0147321428571428*G0_0_0_2_1_1 - 0.0165178571428572*G0_0_0_2_1_2 - 0.00491071428571429*G0_0_0_2_2_0 - 0.0165178571428572*G0_0_0_2_2_1 - 0.0629464285714288*G0_0_0_2_2_2 + 0.120535714285714*G0_0_1_0_0_0 + 0.01875*G0_0_1_0_0_1 + 0.0107142857142857*G0_0_1_0_0_2 + 0.01875*G0_0_1_0_1_0 - 0.00401785714285712*G0_0_1_0_1_1 + 0.0107142857142857*G0_0_1_0_2_0 + 0.00133928571428571*G0_0_1_0_2_2 + 0.01875*G0_0_1_1_0_0 - 0.00401785714285713*G0_0_1_1_0_1 - 0.00401785714285713*G0_0_1_1_1_0 - 0.0482142857142856*G0_0_1_1_1_1 - 0.0200892857142857*G0_0_1_1_1_2 - 0.0200892857142857*G0_0_1_1_2_1 - 0.01875*G0_0_1_1_2_2 + 0.0107142857142857*G0_0_1_2_0_0 + 0.00133928571428571*G0_0_1_2_0_2 - 0.0200892857142857*G0_0_1_2_1_1 - 0.01875*G0_0_1_2_1_2 + 0.00133928571428571*G0_0_1_2_2_0 - 0.01875*G0_0_1_2_2_1 - 0.0361607142857143*G0_0_1_2_2_2 - 0.291964285714285*G0_1_0_0_0_0 - 0.021875*G0_1_0_0_0_1 - 0.0263392857142856*G0_1_0_0_0_2 - 0.021875*G0_1_0_0_1_0 - 0.0107142857142857*G0_1_0_0_1_1 - 0.00424107142857142*G0_1_0_0_1_2 - 0.0263392857142856*G0_1_0_0_2_0 - 0.00424107142857142*G0_1_0_0_2_1 - 0.00491071428571429*G0_1_0_0_2_2 - 0.021875*G0_1_0_1_0_0 - 0.0107142857142857*G0_1_0_1_0_1 - 0.00424107142857142*G0_1_0_1_0_2 - 0.0107142857142857*G0_1_0_1_1_0 - 0.0468749999999999*G0_1_0_1_1_1 - 0.0147321428571428*G0_1_0_1_1_2 - 0.00424107142857142*G0_1_0_1_2_0 - 0.0147321428571428*G0_1_0_1_2_1 - 0.0165178571428572*G0_1_0_1_2_2 - 0.0263392857142856*G0_1_0_2_0_0 - 0.00424107142857142*G0_1_0_2_0_1 - 0.00491071428571429*G0_1_0_2_0_2 - 0.00424107142857142*G0_1_0_2_1_0 - 0.0147321428571428*G0_1_0_2_1_1 - 0.0165178571428572*G0_1_0_2_1_2 - 0.00491071428571429*G0_1_0_2_2_0 - 0.0165178571428571*G0_1_0_2_2_1 - 0.0629464285714288*G0_1_0_2_2_2 + 0.120535714285714*G0_1_1_0_0_0 + 0.01875*G0_1_1_0_0_1 + 0.0107142857142857*G0_1_1_0_0_2 + 0.01875*G0_1_1_0_1_0 - 0.00401785714285712*G0_1_1_0_1_1 + 0.0107142857142857*G0_1_1_0_2_0 + 0.00133928571428571*G0_1_1_0_2_2 + 0.01875*G0_1_1_1_0_0 - 0.00401785714285712*G0_1_1_1_0_1 - 0.00401785714285712*G0_1_1_1_1_0 - 0.0482142857142856*G0_1_1_1_1_1 - 0.0200892857142857*G0_1_1_1_1_2 - 0.0200892857142857*G0_1_1_1_2_1 - 0.01875*G0_1_1_1_2_2 + 0.0107142857142857*G0_1_1_2_0_0 + 0.00133928571428571*G0_1_1_2_0_2 - 0.0200892857142857*G0_1_1_2_1_1 - 0.01875*G0_1_1_2_1_2 + 0.00133928571428571*G0_1_1_2_2_0 - 0.01875*G0_1_1_2_2_1 - 0.0361607142857143*G0_1_1_2_2_2;
    A[60] = -A[40] + 0.141964285714286*G0_1_0_0_0_0 + 0.0191964285714286*G0_1_0_0_0_1 + 0.00758928571428569*G0_1_0_0_0_2 + 0.0191964285714286*G0_1_0_0_1_0 + 0.00312500000000001*G0_1_0_0_1_1 + 0.00379464285714285*G0_1_0_0_1_2 + 0.0075892857142857*G0_1_0_0_2_0 + 0.00379464285714285*G0_1_0_0_2_1 + 0.00803571428571427*G0_1_0_0_2_2 + 0.0191964285714286*G0_1_0_1_0_0 + 0.00312500000000001*G0_1_0_1_0_1 + 0.00379464285714285*G0_1_0_1_0_2 + 0.00312500000000001*G0_1_0_1_1_0 - 0.00267857142857137*G0_1_0_1_1_1 - 0.0044642857142857*G0_1_0_1_1_2 + 0.00379464285714285*G0_1_0_1_2_0 - 0.0044642857142857*G0_1_0_1_2_1 + 0.0075892857142857*G0_1_0_2_0_0 + 0.00379464285714285*G0_1_0_2_0_1 + 0.00803571428571427*G0_1_0_2_0_2 + 0.00379464285714285*G0_1_0_2_1_0 - 0.0044642857142857*G0_1_0_2_1_1 + 0.00803571428571428*G0_1_0_2_2_0 + 0.0375000000000001*G0_1_0_2_2_2 + 0.141964285714286*G0_1_1_0_0_0 + 0.0191964285714286*G0_1_1_0_0_1 + 0.0075892857142857*G0_1_1_0_0_2 + 0.0191964285714286*G0_1_1_0_1_0 + 0.00312500000000001*G0_1_1_0_1_1 + 0.00379464285714285*G0_1_1_0_1_2 + 0.0075892857142857*G0_1_1_0_2_0 + 0.00379464285714285*G0_1_1_0_2_1 + 0.00803571428571428*G0_1_1_0_2_2 + 0.0191964285714286*G0_1_1_1_0_0 + 0.00312500000000001*G0_1_1_1_0_1 + 0.00379464285714285*G0_1_1_1_0_2 + 0.00312500000000001*G0_1_1_1_1_0 - 0.0026785714285714*G0_1_1_1_1_1 - 0.0044642857142857*G0_1_1_1_1_2 + 0.00379464285714285*G0_1_1_1_2_0 - 0.0044642857142857*G0_1_1_1_2_1 + 0.0075892857142857*G0_1_1_2_0_0 + 0.00379464285714285*G0_1_1_2_0_1 + 0.00803571428571428*G0_1_1_2_0_2 + 0.00379464285714285*G0_1_1_2_1_0 - 0.0044642857142857*G0_1_1_2_1_1 + 0.00803571428571428*G0_1_1_2_2_0 + 0.0375000000000001*G0_1_1_2_2_2;
    A[71] = -A[41] + 0.0629464285714285*G0_0_0_0_0_0 + 0.00580357142857142*G0_0_0_0_0_1 + 0.0142857142857143*G0_0_0_0_0_2 + 0.00580357142857142*G0_0_0_0_1_0 + 0.00267857142857141*G0_0_0_0_1_1 + 0.00223214285714285*G0_0_0_0_1_2 + 0.0142857142857143*G0_0_0_0_2_0 + 0.00223214285714285*G0_0_0_0_2_1 + 0.0129464285714286*G0_0_0_0_2_2 + 0.00580357142857142*G0_0_0_1_0_0 + 0.00267857142857141*G0_0_0_1_0_1 + 0.00223214285714285*G0_0_0_1_0_2 + 0.00267857142857141*G0_0_0_1_1_0 + 0.107142857142857*G0_0_0_1_1_1 + 0.0107142857142857*G0_0_0_1_1_2 + 0.00223214285714285*G0_0_0_1_2_0 + 0.0107142857142857*G0_0_0_1_2_1 + 0.000446428571428581*G0_0_0_1_2_2 + 0.0142857142857143*G0_0_0_2_0_0 + 0.00223214285714285*G0_0_0_2_0_1 + 0.0129464285714286*G0_0_0_2_0_2 + 0.00223214285714285*G0_0_0_2_1_0 + 0.0107142857142857*G0_0_0_2_1_1 + 0.000446428571428581*G0_0_0_2_1_2 + 0.0129464285714286*G0_0_0_2_2_0 + 0.000446428571428581*G0_0_0_2_2_1 + 0.0508928571428573*G0_0_0_2_2_2 + 0.0120535714285714*G0_1_0_0_0_0 + 0.00535714285714286*G0_1_0_0_0_1 + 0.00133928571428571*G0_1_0_0_0_2 + 0.00535714285714286*G0_1_0_0_1_0 - 0.00803571428571428*G0_1_0_0_1_1 + 0.00133928571428571*G0_1_0_0_2_0 - 0.0013392857142857*G0_1_0_0_2_2 + 0.00535714285714286*G0_1_0_1_0_0 - 0.00803571428571428*G0_1_0_1_0_1 - 0.00803571428571428*G0_1_0_1_1_0 + 0.00803571428571429*G0_1_0_1_1_2 + 0.00803571428571428*G0_1_0_1_2_1 - 0.00535714285714284*G0_1_0_1_2_2 + 0.00133928571428572*G0_1_0_2_0_0 - 0.0013392857142857*G0_1_0_2_0_2 + 0.00803571428571429*G0_1_0_2_1_1 - 0.00535714285714284*G0_1_0_2_1_2 - 0.0013392857142857*G0_1_0_2_2_0 - 0.00535714285714284*G0_1_0_2_2_1 - 0.0120535714285713*G0_1_0_2_2_2;
    A[57] = A[75] - 0.289285714285714*G0_0_1_0_0_0 + 0.0120535714285715*G0_0_1_0_0_1 + 0.0120535714285715*G0_0_1_0_0_2 + 0.0120535714285715*G0_0_1_0_1_0 + 0.0160714285714286*G0_0_1_0_1_1 + 0.00803571428571428*G0_0_1_0_1_2 + 0.0120535714285715*G0_0_1_0_2_0 + 0.00803571428571428*G0_0_1_0_2_1 + 0.0160714285714285*G0_0_1_0_2_2 + 0.0120535714285715*G0_0_1_1_0_0 + 0.0160714285714286*G0_0_1_1_0_1 + 0.00803571428571428*G0_0_1_1_0_2 + 0.0160714285714286*G0_0_1_1_1_0 + 0.0180803571428571*G0_0_1_1_1_1 + 0.0060267857142857*G0_0_1_1_1_2 + 0.00803571428571428*G0_0_1_1_2_0 + 0.00602678571428571*G0_0_1_1_2_1 + 0.0060267857142857*G0_0_1_1_2_2 + 0.0120535714285715*G0_0_1_2_0_0 + 0.00803571428571428*G0_0_1_2_0_1 + 0.0160714285714285*G0_0_1_2_0_2 + 0.00803571428571428*G0_0_1_2_1_0 + 0.0060267857142857*G0_0_1_2_1_1 + 0.0060267857142857*G0_0_1_2_1_2 + 0.0160714285714285*G0_0_1_2_2_0 + 0.00602678571428571*G0_0_1_2_2_1 + 0.0180803571428571*G0_0_1_2_2_2 + 0.289285714285714*G0_1_0_0_0_0 - 0.0120535714285715*G0_1_0_0_0_1 - 0.0120535714285715*G0_1_0_0_0_2 - 0.0120535714285715*G0_1_0_0_1_0 - 0.0160714285714286*G0_1_0_0_1_1 - 0.00803571428571428*G0_1_0_0_1_2 - 0.0120535714285715*G0_1_0_0_2_0 - 0.00803571428571428*G0_1_0_0_2_1 - 0.0160714285714285*G0_1_0_0_2_2 - 0.0120535714285715*G0_1_0_1_0_0 - 0.0160714285714286*G0_1_0_1_0_1 - 0.00803571428571428*G0_1_0_1_0_2 - 0.0160714285714286*G0_1_0_1_1_0 - 0.0180803571428571*G0_1_0_1_1_1 - 0.00602678571428571*G0_1_0_1_1_2 - 0.00803571428571428*G0_1_0_1_2_0 - 0.00602678571428571*G0_1_0_1_2_1 - 0.0060267857142857*G0_1_0_1_2_2 - 0.0120535714285715*G0_1_0_2_0_0 - 0.00803571428571428*G0_1_0_2_0_1 - 0.0160714285714285*G0_1_0_2_0_2 - 0.00803571428571428*G0_1_0_2_1_0 - 0.00602678571428571*G0_1_0_2_1_1 - 0.00602678571428571*G0_1_0_2_1_2 - 0.0160714285714285*G0_1_0_2_2_0 - 0.0060267857142857*G0_1_0_2_2_1 - 0.0180803571428571*G0_1_0_2_2_2;
    A[74] = -A[86] - 0.0180803571428572*G0_0_0_0_0_0 + 0.00602678571428573*G0_0_0_0_0_1 + 0.00602678571428573*G0_0_0_0_1_0 + 0.00602678571428574*G0_0_0_0_1_1 + 0.00602678571428573*G0_0_0_1_0_0 + 0.00602678571428574*G0_0_0_1_0_1 + 0.00602678571428574*G0_0_0_1_1_0 - 0.0180803571428571*G0_0_0_1_1_1 - 0.00200892857142858*G0_0_1_0_0_1 - 0.0180803571428572*G0_0_1_0_0_2 - 0.00200892857142857*G0_0_1_0_1_0 + 0.00803571428571431*G0_0_1_0_1_1 - 0.0180803571428572*G0_0_1_0_2_0 - 0.0160714285714285*G0_0_1_0_2_2 - 0.00200892857142857*G0_0_1_1_0_0 + 0.00803571428571431*G0_0_1_1_0_1 + 0.00803571428571431*G0_0_1_1_1_0 - 0.018080357142857*G0_0_1_1_1_1 + 0.0180803571428571*G0_0_1_1_1_2 + 0.0180803571428571*G0_0_1_1_2_1 + 0.0160714285714286*G0_0_1_1_2_2 - 0.0180803571428572*G0_0_1_2_0_0 - 0.0160714285714285*G0_0_1_2_0_2 + 0.0180803571428571*G0_0_1_2_1_1 + 0.0160714285714286*G0_0_1_2_1_2 - 0.0160714285714285*G0_0_1_2_2_0 + 0.0160714285714286*G0_0_1_2_2_1 + 0.0180803571428572*G0_1_0_0_0_0 + 0.00602678571428573*G0_1_0_0_0_1 + 0.00602678571428573*G0_1_0_0_1_0 - 0.00602678571428564*G0_1_0_0_2_2 + 0.00602678571428573*G0_1_0_1_0_0 - 0.0361607142857143*G0_1_0_1_1_1 + 0.00602678571428574*G0_1_0_1_2_2 - 0.00602678571428564*G0_1_0_2_0_2 + 0.00602678571428574*G0_1_0_2_1_2 - 0.00602678571428564*G0_1_0_2_2_0 + 0.00602678571428574*G0_1_0_2_2_1 + 0.0361607142857146*G0_1_1_0_0_0 + 0.0281250000000001*G0_1_1_0_0_1 + 0.0281250000000001*G0_1_1_0_1_0 + 0.0301339285714286*G0_1_1_0_1_1 - 0.00200892857142854*G0_1_1_0_2_2 + 0.0281250000000001*G0_1_1_1_0_0 + 0.0301339285714286*G0_1_1_1_0_1 + 0.0301339285714286*G0_1_1_1_1_0 + 0.0180803571428571*G0_1_1_1_1_2 + 0.0180803571428571*G0_1_1_1_2_1 + 0.0200892857142857*G0_1_1_1_2_2 - 0.00200892857142854*G0_1_1_2_0_2 + 0.0180803571428571*G0_1_1_2_1_1 + 0.0200892857142857*G0_1_1_2_1_2 - 0.00200892857142854*G0_1_1_2_2_0 + 0.0200892857142857*G0_1_1_2_2_1 + 0.0180803571428573*G0_1_1_2_2_2;
    A[76] = -A[74] - 0.1265625*G0_0_1_0_0_0 + 0.0100446428571429*G0_0_1_0_0_1 + 0.0100446428571429*G0_0_1_0_1_0 + 0.0261160714285714*G0_0_1_0_1_1 - 0.0060267857142857*G0_0_1_0_1_2 - 0.0060267857142857*G0_0_1_0_2_1 - 0.00401785714285713*G0_0_1_0_2_2 + 0.0100446428571429*G0_0_1_1_0_0 + 0.0261160714285714*G0_0_1_1_0_1 - 0.0060267857142857*G0_0_1_1_0_2 + 0.0261160714285714*G0_0_1_1_1_0 + 0.0180803571428572*G0_0_1_1_1_1 + 0.0120535714285714*G0_0_1_1_1_2 - 0.0060267857142857*G0_0_1_1_2_0 + 0.0120535714285714*G0_0_1_1_2_1 + 0.00401785714285714*G0_0_1_1_2_2 - 0.0060267857142857*G0_0_1_2_0_1 - 0.00401785714285712*G0_0_1_2_0_2 - 0.0060267857142857*G0_0_1_2_1_0 + 0.0120535714285714*G0_0_1_2_1_1 + 0.00401785714285714*G0_0_1_2_1_2 - 0.00401785714285712*G0_0_1_2_2_0 + 0.00401785714285714*G0_0_1_2_2_1 + 0.114508928571429*G0_1_1_0_0_0 + 0.0522321428571429*G0_1_1_0_0_1 - 0.0180803571428571*G0_1_1_0_0_2 + 0.0522321428571429*G0_1_1_0_1_0 + 0.0421875*G0_1_1_0_1_1 - 0.0060267857142857*G0_1_1_0_1_2 - 0.0180803571428571*G0_1_1_0_2_0 - 0.00602678571428569*G0_1_1_0_2_1 - 0.0200892857142857*G0_1_1_0_2_2 + 0.0522321428571429*G0_1_1_1_0_0 + 0.0421875*G0_1_1_1_0_1 - 0.0060267857142857*G0_1_1_1_0_2 + 0.0421875*G0_1_1_1_1_0 + 0.0241071428571431*G0_1_1_1_1_1 + 0.0180803571428571*G0_1_1_1_1_2 - 0.0060267857142857*G0_1_1_1_2_0 + 0.0180803571428571*G0_1_1_1_2_1 + 0.00803571428571428*G0_1_1_1_2_2 - 0.0180803571428571*G0_1_1_2_0_0 - 0.0060267857142857*G0_1_1_2_0_1 - 0.0200892857142857*G0_1_1_2_0_2 - 0.0060267857142857*G0_1_1_2_1_0 + 0.0180803571428571*G0_1_1_2_1_1 + 0.00803571428571427*G0_1_1_2_1_2 - 0.0200892857142857*G0_1_1_2_2_0 + 0.00803571428571427*G0_1_1_2_2_1 - 0.0120535714285715*G0_1_1_2_2_2;
    A[67] = A[76] + 0.108482142857142*G0_0_1_0_0_0 - 0.00602678571428577*G0_0_1_0_0_1 - 0.00602678571428576*G0_0_1_0_0_2 - 0.00602678571428577*G0_0_1_0_1_0 - 0.0140625*G0_0_1_0_1_1 + 0.00200892857142856*G0_0_1_0_1_2 - 0.00602678571428576*G0_0_1_0_2_0 + 0.00200892857142856*G0_0_1_0_2_1 - 0.00200892857142861*G0_0_1_0_2_2 - 0.00602678571428577*G0_0_1_1_0_0 - 0.0140625*G0_0_1_1_0_1 + 0.00200892857142856*G0_0_1_1_0_2 - 0.0140625*G0_0_1_1_1_0 - 0.0180803571428571*G0_0_1_1_1_1 + 0.00200892857142856*G0_0_1_1_2_0 - 0.00602678571428576*G0_0_1_2_0_0 + 0.00200892857142856*G0_0_1_2_0_1 - 0.00200892857142862*G0_0_1_2_0_2 + 0.00200892857142856*G0_0_1_2_1_0 - 0.00200892857142862*G0_0_1_2_2_0 - 0.0180803571428573*G0_0_1_2_2_2 - 0.108482142857143*G0_1_0_0_0_0 + 0.00602678571428577*G0_1_0_0_0_1 + 0.00602678571428576*G0_1_0_0_0_2 + 0.00602678571428577*G0_1_0_0_1_0 + 0.0140625*G0_1_0_0_1_1 - 0.00200892857142856*G0_1_0_0_1_2 + 0.00602678571428576*G0_1_0_0_2_0 - 0.00200892857142856*G0_1_0_0_2_1 + 0.00200892857142862*G0_1_0_0_2_2 + 0.00602678571428577*G0_1_0_1_0_0 + 0.0140625*G0_1_0_1_0_1 - 0.00200892857142856*G0_1_0_1_0_2 + 0.0140625*G0_1_0_1_1_0 + 0.0180803571428571*G0_1_0_1_1_1 - 0.00200892857142856*G0_1_0_1_2_0 + 0.00602678571428576*G0_1_0_2_0_0 - 0.00200892857142856*G0_1_0_2_0_1 + 0.00200892857142862*G0_1_0_2_0_2 - 0.00200892857142856*G0_1_0_2_1_0 + 0.00200892857142862*G0_1_0_2_2_0 + 0.0180803571428573*G0_1_0_2_2_2;
    A[45] = A[67] - 0.0120535714285715*G0_0_0_0_0_0 + 0.00401785714285716*G0_0_0_0_0_1 - 0.0160714285714285*G0_0_0_0_0_2 + 0.00401785714285716*G0_0_0_0_1_0 + 0.00602678571428573*G0_0_0_0_1_1 - 0.0160714285714285*G0_0_0_0_2_0 - 0.018080357142857*G0_0_0_0_2_2 + 0.00401785714285715*G0_0_0_1_0_0 + 0.00602678571428573*G0_0_0_1_0_1 + 0.00602678571428573*G0_0_0_1_1_0 + 0.00602678571428578*G0_0_0_1_1_1 + 0.0160714285714286*G0_0_0_1_1_2 + 0.0160714285714286*G0_0_0_1_2_1 + 0.0421875000000001*G0_0_0_1_2_2 - 0.0160714285714285*G0_0_0_2_0_0 - 0.018080357142857*G0_0_0_2_0_2 + 0.0160714285714286*G0_0_0_2_1_1 + 0.0421875000000001*G0_0_0_2_1_2 - 0.018080357142857*G0_0_0_2_2_0 + 0.0421875000000001*G0_0_0_2_2_1 + 0.241071428571429*G0_0_0_2_2_2 - 0.0120535714285715*G0_0_1_0_0_0 + 0.00401785714285716*G0_0_1_0_0_1 - 0.0160714285714285*G0_0_1_0_0_2 + 0.00401785714285716*G0_0_1_0_1_0 + 0.00602678571428573*G0_0_1_0_1_1 - 0.0160714285714285*G0_0_1_0_2_0 - 0.018080357142857*G0_0_1_0_2_2 + 0.00401785714285716*G0_0_1_1_0_0 + 0.00602678571428573*G0_0_1_1_0_1 + 0.00602678571428573*G0_0_1_1_1_0 + 0.00602678571428577*G0_0_1_1_1_1 + 0.0160714285714286*G0_0_1_1_1_2 + 0.0160714285714286*G0_0_1_1_2_1 + 0.0421875000000001*G0_0_1_1_2_2 - 0.0160714285714285*G0_0_1_2_0_0 - 0.018080357142857*G0_0_1_2_0_2 + 0.0160714285714286*G0_0_1_2_1_1 + 0.0421875000000001*G0_0_1_2_1_2 - 0.018080357142857*G0_0_1_2_2_0 + 0.0421875000000001*G0_0_1_2_2_1 + 0.241071428571429*G0_0_1_2_2_2 + 0.114508928571428*G0_1_0_0_0_0 - 0.00200892857142856*G0_1_0_0_0_1 - 0.0200892857142857*G0_1_0_0_0_2 - 0.00200892857142855*G0_1_0_0_1_0 - 0.00803571428571424*G0_1_0_0_1_1 - 0.0200892857142857*G0_1_0_0_2_0 - 0.0140624999999999*G0_1_0_0_2_2 - 0.00200892857142855*G0_1_0_1_0_0 - 0.00803571428571424*G0_1_0_1_0_1 - 0.00803571428571425*G0_1_0_1_1_0 + 0.00602678571428588*G0_1_0_1_1_1 + 0.0301339285714286*G0_1_0_1_1_2 + 0.0301339285714286*G0_1_0_1_2_1 + 0.0482142857142857*G0_1_0_1_2_2 - 0.0200892857142857*G0_1_0_2_0_0 - 0.0140624999999999*G0_1_0_2_0_2 + 0.0301339285714286*G0_1_0_2_1_1 + 0.0482142857142857*G0_1_0_2_1_2 - 0.0140624999999999*G0_1_0_2_2_0 + 0.0482142857142857*G0_1_0_2_2_1 + 0.114508928571429*G0_1_0_2_2_2 - 0.1265625*G0_1_1_0_0_0 - 0.0441964285714286*G0_1_1_0_0_1 - 0.00200892857142856*G0_1_1_0_0_2 - 0.0441964285714286*G0_1_1_0_1_0 - 0.0241071428571428*G0_1_1_0_1_1 - 0.00200892857142856*G0_1_1_0_2_0 + 0.0020089285714286*G0_1_1_0_2_2 - 0.0441964285714286*G0_1_1_1_0_0 - 0.0241071428571428*G0_1_1_1_0_1 - 0.0241071428571428*G0_1_1_1_1_0 + 0.0241071428571428*G0_1_1_1_1_2 + 0.0241071428571428*G0_1_1_1_2_1 + 0.0441964285714286*G0_1_1_1_2_2 - 0.00200892857142856*G0_1_1_2_0_0 + 0.0020089285714286*G0_1_1_2_0_2 + 0.0241071428571428*G0_1_1_2_1_1 + 0.0441964285714286*G0_1_1_2_1_2 + 0.00200892857142861*G0_1_1_2_2_0 + 0.0441964285714286*G0_1_1_2_2_1 + 0.1265625*G0_1_1_2_2_2;
    A[54] = A[45] + 0.0180803571428574*G0_0_1_0_0_0 + 0.00200892857142861*G0_0_1_0_0_2 - 0.00200892857142856*G0_0_1_0_1_2 + 0.00200892857142861*G0_0_1_0_2_0 - 0.00200892857142856*G0_0_1_0_2_1 + 0.00602678571428567*G0_0_1_0_2_2 - 0.00200892857142856*G0_0_1_1_0_2 + 0.0180803571428572*G0_0_1_1_1_1 + 0.0140625*G0_0_1_1_1_2 - 0.00200892857142856*G0_0_1_1_2_0 + 0.0140625*G0_0_1_1_2_1 + 0.00602678571428563*G0_0_1_1_2_2 + 0.00200892857142861*G0_0_1_2_0_0 - 0.00200892857142856*G0_0_1_2_0_1 + 0.00602678571428567*G0_0_1_2_0_2 - 0.00200892857142856*G0_0_1_2_1_0 + 0.0140625*G0_0_1_2_1_1 + 0.00602678571428563*G0_0_1_2_1_2 + 0.00602678571428567*G0_0_1_2_2_0 + 0.00602678571428563*G0_0_1_2_2_1 - 0.108482142857143*G0_0_1_2_2_2 - 0.0180803571428574*G0_1_0_0_0_0 - 0.00200892857142861*G0_1_0_0_0_2 + 0.00200892857142856*G0_1_0_0_1_2 - 0.00200892857142861*G0_1_0_0_2_0 + 0.00200892857142856*G0_1_0_0_2_1 - 0.00602678571428566*G0_1_0_0_2_2 + 0.00200892857142856*G0_1_0_1_0_2 - 0.0180803571428572*G0_1_0_1_1_1 - 0.0140625*G0_1_0_1_1_2 + 0.00200892857142856*G0_1_0_1_2_0 - 0.0140625*G0_1_0_1_2_1 - 0.00602678571428563*G0_1_0_1_2_2 - 0.00200892857142861*G0_1_0_2_0_0 + 0.00200892857142856*G0_1_0_2_0_1 - 0.00602678571428566*G0_1_0_2_0_2 + 0.00200892857142856*G0_1_0_2_1_0 - 0.0140625*G0_1_0_2_1_1 - 0.00602678571428563*G0_1_0_2_1_2 - 0.00602678571428566*G0_1_0_2_2_0 - 0.00602678571428562*G0_1_0_2_2_1 + 0.108482142857143*G0_1_0_2_2_2;
    A[13] = -A[15] - 0.0361607142857143*G0_0_0_0_0_0 + 0.00133928571428572*G0_0_0_0_0_1 - 0.01875*G0_0_0_0_0_2 + 0.00133928571428571*G0_0_0_0_1_0 + 0.0107142857142857*G0_0_0_0_1_1 - 0.01875*G0_0_0_0_2_0 - 0.0200892857142857*G0_0_0_0_2_2 + 0.00133928571428571*G0_0_0_1_0_0 + 0.0107142857142857*G0_0_0_1_0_1 + 0.0107142857142857*G0_0_0_1_1_0 + 0.120535714285714*G0_0_0_1_1_1 + 0.01875*G0_0_0_1_1_2 + 0.01875*G0_0_0_1_2_1 - 0.00401785714285713*G0_0_0_1_2_2 - 0.01875*G0_0_0_2_0_0 - 0.0200892857142857*G0_0_0_2_0_2 + 0.01875*G0_0_0_2_1_1 - 0.00401785714285713*G0_0_0_2_1_2 - 0.0200892857142857*G0_0_0_2_2_0 - 0.00401785714285713*G0_0_0_2_2_1 - 0.0482142857142856*G0_0_0_2_2_2 + 0.0267857142857142*G0_0_1_0_0_0 + 0.00624999999999998*G0_0_1_0_0_1 - 0.00223214285714286*G0_0_1_0_0_2 + 0.00624999999999998*G0_0_1_0_1_0 + 0.0370535714285714*G0_0_1_0_1_1 + 0.00424107142857142*G0_0_1_0_1_2 - 0.00223214285714286*G0_0_1_0_2_0 + 0.00424107142857142*G0_0_1_0_2_1 - 0.00535714285714283*G0_0_1_0_2_2 + 0.00624999999999998*G0_0_1_1_0_0 + 0.0370535714285714*G0_0_1_1_0_1 + 0.00424107142857142*G0_0_1_1_0_2 + 0.0370535714285714*G0_0_1_1_1_0 + 0.412499999999999*G0_0_1_1_1_1 + 0.0406249999999999*G0_0_1_1_1_2 + 0.00424107142857142*G0_0_1_1_2_0 + 0.0406249999999999*G0_0_1_1_2_1 + 0.00669642857142856*G0_0_1_1_2_2 - 0.00223214285714286*G0_0_1_2_0_0 + 0.00424107142857142*G0_0_1_2_0_1 - 0.00535714285714283*G0_0_1_2_0_2 + 0.00424107142857142*G0_0_1_2_1_0 + 0.0406249999999999*G0_0_1_2_1_1 + 0.00669642857142856*G0_0_1_2_1_2 - 0.00535714285714283*G0_0_1_2_2_0 + 0.00669642857142856*G0_0_1_2_2_1 - 0.00133928571428558*G0_0_1_2_2_2;
    A[30] = A[4] + 0.00133928571428571*G0_0_0_0_0_1 - 0.00133928571428571*G0_0_0_0_0_2 + 0.00133928571428571*G0_0_0_0_1_0 + 0.00267857142857142*G0_0_0_0_1_1 + 0.000669642857142857*G0_0_0_0_1_2 - 0.00133928571428571*G0_0_0_0_2_0 + 0.000669642857142858*G0_0_0_0_2_1 - 0.00401785714285711*G0_0_0_0_2_2 + 0.00133928571428571*G0_0_0_1_0_0 + 0.00267857142857142*G0_0_0_1_0_1 + 0.000669642857142857*G0_0_0_1_0_2 + 0.00267857142857142*G0_0_0_1_1_0 - 0.0241071428571428*G0_0_0_1_1_1 - 0.00803571428571425*G0_0_0_1_1_2 + 0.000669642857142857*G0_0_0_1_2_0 - 0.00803571428571425*G0_0_0_1_2_1 - 0.00133928571428571*G0_0_0_2_0_0 + 0.000669642857142858*G0_0_0_2_0_1 - 0.00401785714285711*G0_0_0_2_0_2 + 0.000669642857142857*G0_0_0_2_1_0 - 0.00803571428571425*G0_0_0_2_1_1 - 0.00401785714285711*G0_0_0_2_2_0 + 0.0482142857142861*G0_0_0_2_2_2 - 0.000446428571428585*G0_0_1_0_0_1 + 0.00044642857142857*G0_0_1_0_0_2 - 0.000446428571428586*G0_0_1_0_1_0 + 0.00312499999999999*G0_0_1_0_1_1 + 0.00044642857142857*G0_0_1_0_2_0 - 0.00312499999999998*G0_0_1_0_2_2 - 0.000446428571428585*G0_0_1_1_0_0 + 0.00312499999999999*G0_0_1_1_0_1 + 0.00312499999999999*G0_0_1_1_1_0 - 0.0220982142857143*G0_0_1_1_1_1 - 0.00245535714285712*G0_0_1_1_1_2 - 0.00245535714285712*G0_0_1_1_2_1 + 0.00245535714285717*G0_0_1_1_2_2 + 0.00044642857142857*G0_0_1_2_0_0 - 0.00312499999999998*G0_0_1_2_0_2 - 0.00245535714285712*G0_0_1_2_1_1 + 0.00245535714285717*G0_0_1_2_1_2 - 0.00312499999999998*G0_0_1_2_2_0 + 0.00245535714285717*G0_0_1_2_2_1 + 0.0220982142857145*G0_0_1_2_2_2 + 0.00312499999999997*G0_1_0_0_0_1 - 0.00312500000000001*G0_1_0_0_0_2 + 0.00312499999999997*G0_1_0_0_1_0 + 0.00357142857142855*G0_1_0_0_1_1 - 0.00312500000000001*G0_1_0_0_2_0 - 0.00357142857142855*G0_1_0_0_2_2 + 0.00312499999999997*G0_1_0_1_0_0 + 0.00357142857142855*G0_1_0_1_0_1 + 0.00357142857142855*G0_1_0_1_1_0 - 0.0502232142857143*G0_1_0_1_1_1 - 0.00558035714285712*G0_1_0_1_1_2 - 0.00558035714285712*G0_1_0_1_2_1 + 0.00558035714285718*G0_1_0_1_2_2 - 0.00312500000000001*G0_1_0_2_0_0 - 0.00357142857142855*G0_1_0_2_0_2 - 0.00558035714285712*G0_1_0_2_1_1 + 0.00558035714285718*G0_1_0_2_1_2 - 0.00357142857142855*G0_1_0_2_2_0 + 0.00558035714285718*G0_1_0_2_2_1 + 0.0502232142857145*G0_1_0_2_2_2 + 0.00133928571428568*G0_1_1_0_0_1 - 0.00133928571428573*G0_1_1_0_0_2 + 0.00133928571428568*G0_1_1_0_1_0 + 0.00401785714285713*G0_1_1_0_1_1 - 0.000669642857142856*G0_1_1_0_1_2 - 0.00133928571428573*G0_1_1_0_2_0 - 0.000669642857142857*G0_1_1_0_2_1 - 0.00267857142857142*G0_1_1_0_2_2 + 0.00133928571428568*G0_1_1_1_0_0 + 0.00401785714285713*G0_1_1_1_0_1 - 0.000669642857142856*G0_1_1_1_0_2 + 0.00401785714285713*G0_1_1_1_1_0 - 0.0482142857142856*G0_1_1_1_1_1 - 0.000669642857142856*G0_1_1_1_2_0 + 0.0080357142857143*G0_1_1_1_2_2 - 0.00133928571428573*G0_1_1_2_0_0 - 0.000669642857142857*G0_1_1_2_0_1 - 0.00267857142857142*G0_1_1_2_0_2 - 0.000669642857142857*G0_1_1_2_1_0 + 0.0080357142857143*G0_1_1_2_1_2 - 0.00267857142857142*G0_1_1_2_2_0 + 0.0080357142857143*G0_1_1_2_2_1 + 0.0241071428571429*G0_1_1_2_2_2;
    A[80] = -A[30] + 0.141964285714286*G0_0_0_0_0_0 + 0.0075892857142857*G0_0_0_0_0_1 + 0.0191964285714285*G0_0_0_0_0_2 + 0.0075892857142857*G0_0_0_0_1_0 + 0.00803571428571428*G0_0_0_0_1_1 + 0.00379464285714285*G0_0_0_0_1_2 + 0.0191964285714285*G0_0_0_0_2_0 + 0.00379464285714285*G0_0_0_0_2_1 + 0.003125*G0_0_0_0_2_2 + 0.0075892857142857*G0_0_0_1_0_0 + 0.00803571428571428*G0_0_0_1_0_1 + 0.00379464285714285*G0_0_0_1_0_2 + 0.00803571428571428*G0_0_0_1_1_0 + 0.0375*G0_0_0_1_1_1 + 0.00379464285714285*G0_0_0_1_2_0 - 0.0044642857142857*G0_0_0_1_2_2 + 0.0191964285714285*G0_0_0_2_0_0 + 0.00379464285714285*G0_0_0_2_0_1 + 0.003125*G0_0_0_2_0_2 + 0.00379464285714285*G0_0_0_2_1_0 - 0.00446428571428569*G0_0_0_2_1_2 + 0.003125*G0_0_0_2_2_0 - 0.0044642857142857*G0_0_0_2_2_1 - 0.0026785714285713*G0_0_0_2_2_2 + 0.141964285714286*G0_0_1_0_0_0 + 0.00758928571428569*G0_0_1_0_0_1 + 0.0191964285714285*G0_0_1_0_0_2 + 0.00758928571428569*G0_0_1_0_1_0 + 0.00803571428571427*G0_0_1_0_1_1 + 0.00379464285714285*G0_0_1_0_1_2 + 0.0191964285714285*G0_0_1_0_2_0 + 0.00379464285714285*G0_0_1_0_2_1 + 0.003125*G0_0_1_0_2_2 + 0.00758928571428569*G0_0_1_1_0_0 + 0.00803571428571427*G0_0_1_1_0_1 + 0.00379464285714285*G0_0_1_1_0_2 + 0.00803571428571427*G0_0_1_1_1_0 + 0.0375*G0_0_1_1_1_1 + 0.00379464285714285*G0_0_1_1_2_0 - 0.00446428571428569*G0_0_1_1_2_2 + 0.0191964285714285*G0_0_1_2_0_0 + 0.00379464285714285*G0_0_1_2_0_1 + 0.003125*G0_0_1_2_0_2 + 0.00379464285714285*G0_0_1_2_1_0 - 0.00446428571428569*G0_0_1_2_1_2 + 0.003125*G0_0_1_2_2_0 - 0.00446428571428569*G0_0_1_2_2_1 - 0.00267857142857132*G0_0_1_2_2_2;
    A[14] = -A[16] + 0.00267857142857143*G0_0_1_0_0_0 - 0.00312499999999999*G0_0_1_0_0_1 + 0.00446428571428571*G0_0_1_0_0_2 - 0.00312499999999999*G0_0_1_0_1_0 - 0.0191964285714285*G0_0_1_0_1_1 - 0.00379464285714284*G0_0_1_0_1_2 + 0.00446428571428571*G0_0_1_0_2_0 - 0.00379464285714284*G0_0_1_0_2_1 - 0.00312499999999999*G0_0_1_1_0_0 - 0.0191964285714285*G0_0_1_1_0_1 - 0.00379464285714284*G0_0_1_1_0_2 - 0.0191964285714285*G0_0_1_1_1_0 - 0.141964285714285*G0_0_1_1_1_1 - 0.00758928571428568*G0_0_1_1_1_2 - 0.00379464285714285*G0_0_1_1_2_0 - 0.00758928571428568*G0_0_1_1_2_1 - 0.00803571428571428*G0_0_1_1_2_2 + 0.00446428571428571*G0_0_1_2_0_0 - 0.00379464285714285*G0_0_1_2_0_1 - 0.00379464285714284*G0_0_1_2_1_0 - 0.00758928571428568*G0_0_1_2_1_1 - 0.00803571428571428*G0_0_1_2_1_2 - 0.00803571428571428*G0_0_1_2_2_1 - 0.0375000000000001*G0_0_1_2_2_2;
    A[23] = -A[25] + 0.0120535714285714*G0_1_0_0_0_0 + 0.00133928571428571*G0_1_0_0_0_1 + 0.00535714285714286*G0_1_0_0_0_2 + 0.00133928571428571*G0_1_0_0_1_0 - 0.00133928571428572*G0_1_0_0_1_1 + 0.00535714285714286*G0_1_0_0_2_0 - 0.00803571428571426*G0_1_0_0_2_2 + 0.00133928571428571*G0_1_0_1_0_0 - 0.00133928571428572*G0_1_0_1_0_1 - 0.00133928571428572*G0_1_0_1_1_0 - 0.0120535714285714*G0_1_0_1_1_1 - 0.00535714285714284*G0_1_0_1_1_2 - 0.00535714285714284*G0_1_0_1_2_1 + 0.0080357142857143*G0_1_0_1_2_2 + 0.00535714285714286*G0_1_0_2_0_0 - 0.00803571428571426*G0_1_0_2_0_2 - 0.00535714285714284*G0_1_0_2_1_1 + 0.0080357142857143*G0_1_0_2_1_2 - 0.00803571428571426*G0_1_0_2_2_0 + 0.0080357142857143*G0_1_0_2_2_1 + 0.0629464285714285*G0_1_1_0_0_0 + 0.0142857142857143*G0_1_1_0_0_1 + 0.00580357142857142*G0_1_1_0_0_2 + 0.0142857142857143*G0_1_1_0_1_0 + 0.0129464285714285*G0_1_1_0_1_1 + 0.00223214285714285*G0_1_1_0_1_2 + 0.00580357142857142*G0_1_1_0_2_0 + 0.00223214285714285*G0_1_1_0_2_1 + 0.00267857142857147*G0_1_1_0_2_2 + 0.0142857142857143*G0_1_1_1_0_0 + 0.0129464285714285*G0_1_1_1_0_1 + 0.00223214285714285*G0_1_1_1_0_2 + 0.0129464285714285*G0_1_1_1_1_0 + 0.050892857142857*G0_1_1_1_1_1 + 0.000446428571428571*G0_1_1_1_1_2 + 0.00223214285714285*G0_1_1_1_2_0 + 0.000446428571428571*G0_1_1_1_2_1 + 0.0107142857142858*G0_1_1_1_2_2 + 0.00580357142857142*G0_1_1_2_0_0 + 0.00223214285714285*G0_1_1_2_0_1 + 0.00267857142857147*G0_1_1_2_0_2 + 0.00223214285714285*G0_1_1_2_1_0 + 0.000446428571428571*G0_1_1_2_1_1 + 0.0107142857142858*G0_1_1_2_1_2 + 0.00267857142857147*G0_1_1_2_2_0 + 0.0107142857142858*G0_1_1_2_2_1 + 0.107142857142858*G0_1_1_2_2_2;
    A[28] = -A[23] + 0.00267857142857145*G0_1_0_0_0_0 + 0.00446428571428571*G0_1_0_0_0_1 - 0.00312499999999999*G0_1_0_0_0_2 + 0.00446428571428571*G0_1_0_0_1_0 - 0.00379464285714285*G0_1_0_0_1_2 - 0.00312499999999999*G0_1_0_0_2_0 - 0.00379464285714285*G0_1_0_0_2_1 - 0.0191964285714286*G0_1_0_0_2_2 + 0.00446428571428571*G0_1_0_1_0_0 - 0.00379464285714285*G0_1_0_1_0_2 - 0.0374999999999999*G0_1_0_1_1_1 - 0.00803571428571427*G0_1_0_1_1_2 - 0.00379464285714285*G0_1_0_1_2_0 - 0.00803571428571427*G0_1_0_1_2_1 - 0.00758928571428574*G0_1_0_1_2_2 - 0.00312499999999999*G0_1_0_2_0_0 - 0.00379464285714285*G0_1_0_2_0_1 - 0.0191964285714286*G0_1_0_2_0_2 - 0.00379464285714285*G0_1_0_2_1_0 - 0.00803571428571427*G0_1_0_2_1_1 - 0.00758928571428574*G0_1_0_2_1_2 - 0.0191964285714286*G0_1_0_2_2_0 - 0.00758928571428574*G0_1_0_2_2_1 - 0.141964285714286*G0_1_0_2_2_2;
    A[82] = A[28] + 0.00200892857142861*G0_0_1_0_0_0 + 0.00558035714285714*G0_0_1_0_0_1 + 0.000446428571428572*G0_0_1_0_0_2 + 0.00558035714285714*G0_0_1_0_1_0 + 0.00245535714285715*G0_0_1_0_1_1 - 0.000669642857142856*G0_0_1_0_1_2 + 0.000446428571428573*G0_0_1_0_2_0 - 0.000669642857142856*G0_0_1_0_2_1 - 0.0017857142857143*G0_0_1_0_2_2 + 0.00558035714285714*G0_0_1_1_0_0 + 0.00245535714285715*G0_0_1_1_0_1 - 0.000669642857142856*G0_0_1_1_0_2 + 0.00245535714285715*G0_0_1_1_1_0 - 0.0261160714285713*G0_0_1_1_1_1 + 0.000892857142857141*G0_0_1_1_1_2 - 0.000669642857142856*G0_0_1_1_2_0 + 0.000892857142857142*G0_0_1_1_2_1 + 0.00178571428571427*G0_0_1_1_2_2 + 0.000446428571428573*G0_0_1_2_0_0 - 0.000669642857142856*G0_0_1_2_0_1 - 0.0017857142857143*G0_0_1_2_0_2 - 0.000669642857142856*G0_0_1_2_1_0 + 0.000892857142857141*G0_0_1_2_1_1 + 0.00178571428571427*G0_0_1_2_1_2 - 0.0017857142857143*G0_0_1_2_2_0 + 0.00178571428571427*G0_0_1_2_2_1 - 0.00200892857142861*G0_1_0_0_0_0 - 0.00558035714285714*G0_1_0_0_0_1 - 0.000446428571428572*G0_1_0_0_0_2 - 0.00558035714285714*G0_1_0_0_1_0 - 0.00245535714285715*G0_1_0_0_1_1 + 0.000669642857142856*G0_1_0_0_1_2 - 0.000446428571428573*G0_1_0_0_2_0 + 0.000669642857142856*G0_1_0_0_2_1 + 0.0017857142857143*G0_1_0_0_2_2 - 0.00558035714285714*G0_1_0_1_0_0 - 0.00245535714285715*G0_1_0_1_0_1 + 0.000669642857142856*G0_1_0_1_0_2 - 0.00245535714285715*G0_1_0_1_1_0 + 0.0261160714285713*G0_1_0_1_1_1 - 0.000892857142857142*G0_1_0_1_1_2 + 0.000669642857142856*G0_1_0_1_2_0 - 0.000892857142857142*G0_1_0_1_2_1 - 0.00178571428571427*G0_1_0_1_2_2 - 0.000446428571428572*G0_1_0_2_0_0 + 0.000669642857142856*G0_1_0_2_0_1 + 0.0017857142857143*G0_1_0_2_0_2 + 0.000669642857142856*G0_1_0_2_1_0 - 0.000892857142857142*G0_1_0_2_1_1 - 0.00178571428571427*G0_1_0_2_1_2 + 0.0017857142857143*G0_1_0_2_2_0 - 0.00178571428571427*G0_1_0_2_2_1;
    A[32] = -A[82] + 0.00267857142857145*G0_0_1_0_0_0 + 0.00446428571428571*G0_0_1_0_0_1 - 0.00312499999999999*G0_0_1_0_0_2 + 0.00446428571428571*G0_0_1_0_1_0 - 0.00379464285714285*G0_0_1_0_1_2 - 0.00312499999999999*G0_0_1_0_2_0 - 0.00379464285714285*G0_0_1_0_2_1 - 0.0191964285714286*G0_0_1_0_2_2 + 0.00446428571428571*G0_0_1_1_0_0 - 0.00379464285714285*G0_0_1_1_0_2 - 0.0374999999999999*G0_0_1_1_1_1 - 0.00803571428571427*G0_0_1_1_1_2 - 0.00379464285714285*G0_0_1_1_2_0 - 0.00803571428571427*G0_0_1_1_2_1 - 0.00758928571428574*G0_0_1_1_2_2 - 0.00312499999999999*G0_0_1_2_0_0 - 0.00379464285714285*G0_0_1_2_0_1 - 0.0191964285714286*G0_0_1_2_0_2 - 0.00379464285714285*G0_0_1_2_1_0 - 0.00803571428571427*G0_0_1_2_1_1 - 0.00758928571428574*G0_0_1_2_1_2 - 0.0191964285714286*G0_0_1_2_2_0 - 0.00758928571428574*G0_0_1_2_2_1 - 0.141964285714286*G0_0_1_2_2_2;
    A[72] = A[82] + 0.0241071428571427*G0_0_1_0_0_0 - 0.00803571428571429*G0_0_1_0_0_1 - 0.00133928571428572*G0_0_1_0_0_2 - 0.00803571428571428*G0_0_1_0_1_0 - 0.00803571428571428*G0_0_1_0_1_1 + 0.00133928571428571*G0_0_1_0_1_2 - 0.00133928571428572*G0_0_1_0_2_0 + 0.00133928571428571*G0_0_1_0_2_1 - 0.00803571428571429*G0_0_1_1_0_0 - 0.00803571428571429*G0_0_1_1_0_1 + 0.00133928571428571*G0_0_1_1_0_2 - 0.00803571428571428*G0_0_1_1_1_0 + 0.0241071428571427*G0_0_1_1_1_1 - 0.00133928571428571*G0_0_1_1_1_2 + 0.00133928571428571*G0_0_1_1_2_0 - 0.00133928571428571*G0_0_1_1_2_1 - 0.00133928571428572*G0_0_1_2_0_0 + 0.00133928571428571*G0_0_1_2_0_1 + 0.00133928571428571*G0_0_1_2_1_0 - 0.00133928571428571*G0_0_1_2_1_1 - 0.0241071428571428*G0_1_1_0_0_0 - 0.00803571428571427*G0_1_1_0_0_1 + 0.00267857142857144*G0_1_1_0_0_2 - 0.00803571428571427*G0_1_1_0_1_0 + 0.000669642857142856*G0_1_1_0_1_2 + 0.00267857142857144*G0_1_1_0_2_0 + 0.000669642857142856*G0_1_1_0_2_1 + 0.00133928571428574*G0_1_1_0_2_2 - 0.00803571428571426*G0_1_1_1_0_0 + 0.000669642857142857*G0_1_1_1_0_2 + 0.0482142857142856*G0_1_1_1_1_1 - 0.00401785714285715*G0_1_1_1_1_2 + 0.000669642857142857*G0_1_1_1_2_0 - 0.00401785714285715*G0_1_1_1_2_1 - 0.0013392857142857*G0_1_1_1_2_2 + 0.00267857142857144*G0_1_1_2_0_0 + 0.000669642857142856*G0_1_1_2_0_1 + 0.00133928571428574*G0_1_1_2_0_2 + 0.000669642857142856*G0_1_1_2_1_0 - 0.00401785714285715*G0_1_1_2_1_1 - 0.0013392857142857*G0_1_1_2_1_2 + 0.00133928571428574*G0_1_1_2_2_0 - 0.0013392857142857*G0_1_1_2_2_1;
    A[27] = A[28] + 0.0241071428571427*G0_1_0_0_0_0 - 0.00803571428571429*G0_1_0_0_0_1 - 0.00133928571428572*G0_1_0_0_0_2 - 0.00803571428571429*G0_1_0_0_1_0 - 0.00803571428571429*G0_1_0_0_1_1 + 0.00133928571428571*G0_1_0_0_1_2 - 0.00133928571428572*G0_1_0_0_2_0 + 0.00133928571428571*G0_1_0_0_2_1 - 0.00803571428571429*G0_1_0_1_0_0 - 0.00803571428571429*G0_1_0_1_0_1 + 0.00133928571428571*G0_1_0_1_0_2 - 0.00803571428571429*G0_1_0_1_1_0 + 0.0241071428571427*G0_1_0_1_1_1 - 0.00133928571428571*G0_1_0_1_1_2 + 0.00133928571428571*G0_1_0_1_2_0 - 0.00133928571428571*G0_1_0_1_2_1 - 0.00133928571428572*G0_1_0_2_0_0 + 0.00133928571428571*G0_1_0_2_0_1 + 0.00133928571428571*G0_1_0_2_1_0 - 0.00133928571428571*G0_1_0_2_1_1 - 0.0241071428571428*G0_1_1_0_0_0 - 0.00803571428571427*G0_1_1_0_0_1 + 0.00267857142857144*G0_1_1_0_0_2 - 0.00803571428571427*G0_1_1_0_1_0 + 0.000669642857142856*G0_1_1_0_1_2 + 0.00267857142857144*G0_1_1_0_2_0 + 0.000669642857142856*G0_1_1_0_2_1 + 0.00133928571428574*G0_1_1_0_2_2 - 0.00803571428571426*G0_1_1_1_0_0 + 0.000669642857142857*G0_1_1_1_0_2 + 0.0482142857142856*G0_1_1_1_1_1 - 0.00401785714285715*G0_1_1_1_1_2 + 0.000669642857142857*G0_1_1_1_2_0 - 0.00401785714285715*G0_1_1_1_2_1 - 0.0013392857142857*G0_1_1_1_2_2 + 0.00267857142857144*G0_1_1_2_0_0 + 0.000669642857142857*G0_1_1_2_0_1 + 0.00133928571428574*G0_1_1_2_0_2 + 0.000669642857142856*G0_1_1_2_1_0 - 0.00401785714285715*G0_1_1_2_1_1 - 0.0013392857142857*G0_1_1_2_1_2 + 0.00133928571428574*G0_1_1_2_2_0 - 0.0013392857142857*G0_1_1_2_2_1;
    A[87] = A[78] - 0.108482142857143*G0_0_1_0_0_0 - 0.0904017857142857*G0_0_1_0_0_1 - 0.0180803571428571*G0_0_1_0_0_2 - 0.0904017857142857*G0_0_1_0_1_0 - 0.0904017857142857*G0_0_1_0_1_1 - 0.0180803571428571*G0_0_1_0_1_2 - 0.0180803571428571*G0_0_1_0_2_0 - 0.0180803571428571*G0_0_1_0_2_1 - 0.00602678571428571*G0_0_1_0_2_2 - 0.0904017857142857*G0_0_1_1_0_0 - 0.0904017857142857*G0_0_1_1_0_1 - 0.0180803571428571*G0_0_1_1_0_2 - 0.0904017857142857*G0_0_1_1_1_0 - 0.108482142857143*G0_0_1_1_1_1 - 0.0180803571428571*G0_0_1_1_1_2 - 0.0180803571428571*G0_0_1_1_2_0 - 0.0180803571428571*G0_0_1_1_2_1 - 0.00602678571428571*G0_0_1_1_2_2 - 0.0180803571428571*G0_0_1_2_0_0 - 0.0180803571428571*G0_0_1_2_0_1 - 0.00602678571428571*G0_0_1_2_0_2 - 0.0180803571428571*G0_0_1_2_1_0 - 0.0180803571428571*G0_0_1_2_1_1 - 0.00602678571428571*G0_0_1_2_1_2 - 0.00602678571428571*G0_0_1_2_2_0 - 0.00602678571428571*G0_0_1_2_2_1 + 0.108482142857143*G0_1_0_0_0_0 + 0.0904017857142857*G0_1_0_0_0_1 + 0.0180803571428571*G0_1_0_0_0_2 + 0.0904017857142857*G0_1_0_0_1_0 + 0.0904017857142857*G0_1_0_0_1_1 + 0.0180803571428571*G0_1_0_0_1_2 + 0.0180803571428571*G0_1_0_0_2_0 + 0.0180803571428571*G0_1_0_0_2_1 + 0.00602678571428571*G0_1_0_0_2_2 + 0.0904017857142857*G0_1_0_1_0_0 + 0.0904017857142857*G0_1_0_1_0_1 + 0.0180803571428571*G0_1_0_1_0_2 + 0.0904017857142857*G0_1_0_1_1_0 + 0.108482142857143*G0_1_0_1_1_1 + 0.0180803571428571*G0_1_0_1_1_2 + 0.0180803571428571*G0_1_0_1_2_0 + 0.0180803571428571*G0_1_0_1_2_1 + 0.00602678571428572*G0_1_0_1_2_2 + 0.0180803571428571*G0_1_0_2_0_0 + 0.0180803571428571*G0_1_0_2_0_1 + 0.00602678571428571*G0_1_0_2_0_2 + 0.0180803571428571*G0_1_0_2_1_0 + 0.0180803571428571*G0_1_0_2_1_1 + 0.00602678571428571*G0_1_0_2_1_2 + 0.00602678571428571*G0_1_0_2_2_0 + 0.00602678571428571*G0_1_0_2_2_1;
    A[42] = -A[72] + 0.0267857142857142*G0_0_1_0_0_0 - 0.00223214285714286*G0_0_1_0_0_1 + 0.00624999999999999*G0_0_1_0_0_2 - 0.00223214285714286*G0_0_1_0_1_0 - 0.00535714285714286*G0_0_1_0_1_1 + 0.00424107142857143*G0_0_1_0_1_2 + 0.00624999999999999*G0_0_1_0_2_0 + 0.00424107142857143*G0_0_1_0_2_1 + 0.0370535714285714*G0_0_1_0_2_2 - 0.00223214285714286*G0_0_1_1_0_0 - 0.00535714285714286*G0_0_1_1_0_1 + 0.00424107142857143*G0_0_1_1_0_2 - 0.00535714285714286*G0_0_1_1_1_0 - 0.00133928571428575*G0_0_1_1_1_1 + 0.00669642857142858*G0_0_1_1_1_2 + 0.00424107142857143*G0_0_1_1_2_0 + 0.00669642857142858*G0_0_1_1_2_1 + 0.0406250000000001*G0_0_1_1_2_2 + 0.00624999999999999*G0_0_1_2_0_0 + 0.00424107142857143*G0_0_1_2_0_1 + 0.0370535714285714*G0_0_1_2_0_2 + 0.00424107142857143*G0_0_1_2_1_0 + 0.00669642857142858*G0_0_1_2_1_1 + 0.0406250000000001*G0_0_1_2_1_2 + 0.0370535714285714*G0_0_1_2_2_0 + 0.0406250000000001*G0_0_1_2_2_1 + 0.4125*G0_0_1_2_2_2 - 0.0361607142857143*G0_1_1_0_0_0 - 0.01875*G0_1_1_0_0_1 + 0.00133928571428572*G0_1_1_0_0_2 - 0.01875*G0_1_1_0_1_0 - 0.0200892857142857*G0_1_1_0_1_1 + 0.00133928571428572*G0_1_1_0_2_0 + 0.0107142857142857*G0_1_1_0_2_2 - 0.01875*G0_1_1_1_0_0 - 0.0200892857142857*G0_1_1_1_0_1 - 0.0200892857142857*G0_1_1_1_1_0 - 0.0482142857142857*G0_1_1_1_1_1 - 0.00401785714285712*G0_1_1_1_1_2 - 0.00401785714285712*G0_1_1_1_2_1 + 0.01875*G0_1_1_1_2_2 + 0.00133928571428572*G0_1_1_2_0_0 + 0.0107142857142857*G0_1_1_2_0_2 - 0.00401785714285712*G0_1_1_2_1_1 + 0.01875*G0_1_1_2_1_2 + 0.0107142857142857*G0_1_1_2_2_0 + 0.01875*G0_1_1_2_2_1 + 0.120535714285715*G0_1_1_2_2_2;
    A[62] = -A[42] - 0.0575892857142857*G0_1_1_0_0_0 - 0.0191964285714285*G0_1_1_0_0_1 - 0.0116071428571428*G0_1_1_0_0_2 - 0.0191964285714285*G0_1_1_0_1_0 - 0.0191964285714285*G0_1_1_0_1_1 - 0.00580357142857143*G0_1_1_0_1_2 - 0.0116071428571428*G0_1_1_0_2_0 - 0.00580357142857143*G0_1_1_0_2_1 - 0.0160714285714286*G0_1_1_0_2_2 - 0.0191964285714285*G0_1_1_1_0_0 - 0.0191964285714285*G0_1_1_1_0_1 - 0.00580357142857142*G0_1_1_1_0_2 - 0.0191964285714285*G0_1_1_1_1_0 - 0.0575892857142856*G0_1_1_1_1_1 - 0.0116071428571428*G0_1_1_1_1_2 - 0.00580357142857142*G0_1_1_1_2_0 - 0.0116071428571428*G0_1_1_1_2_1 - 0.0160714285714286*G0_1_1_1_2_2 - 0.0116071428571428*G0_1_1_2_0_0 - 0.00580357142857143*G0_1_1_2_0_1 - 0.0160714285714286*G0_1_1_2_0_2 - 0.00580357142857142*G0_1_1_2_1_0 - 0.0116071428571428*G0_1_1_2_1_1 - 0.0160714285714286*G0_1_1_2_1_2 - 0.0160714285714286*G0_1_1_2_2_0 - 0.0160714285714286*G0_1_1_2_2_1 - 0.20625*G0_1_1_2_2_2;
    A[96] = A[69] + 0.0723214285714283*G0_0_1_0_0_0 + 0.0723214285714284*G0_0_1_0_0_2 - 0.0120535714285715*G0_0_1_0_1_1 + 0.0723214285714284*G0_0_1_0_2_0 + 0.0361607142857141*G0_0_1_0_2_2 - 0.0120535714285715*G0_0_1_1_0_1 - 0.0120535714285715*G0_0_1_1_1_0 - 0.0361607142857144*G0_0_1_1_1_1 - 0.0241071428571429*G0_0_1_1_1_2 - 0.0241071428571429*G0_0_1_1_2_1 - 0.0361607142857144*G0_0_1_1_2_2 + 0.0723214285714284*G0_0_1_2_0_0 + 0.0361607142857141*G0_0_1_2_0_2 - 0.0241071428571429*G0_0_1_2_1_1 - 0.0361607142857144*G0_0_1_2_1_2 + 0.0361607142857141*G0_0_1_2_2_0 - 0.0361607142857144*G0_0_1_2_2_1 - 0.144642857142858*G0_0_1_2_2_2 - 0.0723214285714283*G0_1_0_0_0_0 - 0.0723214285714284*G0_1_0_0_0_2 + 0.0120535714285715*G0_1_0_0_1_1 - 0.0723214285714284*G0_1_0_0_2_0 - 0.0361607142857141*G0_1_0_0_2_2 + 0.0120535714285715*G0_1_0_1_0_1 + 0.0120535714285715*G0_1_0_1_1_0 + 0.0361607142857144*G0_1_0_1_1_1 + 0.0241071428571429*G0_1_0_1_1_2 + 0.0241071428571429*G0_1_0_1_2_1 + 0.0361607142857144*G0_1_0_1_2_2 - 0.0723214285714284*G0_1_0_2_0_0 - 0.0361607142857141*G0_1_0_2_0_2 + 0.0241071428571429*G0_1_0_2_1_1 + 0.0361607142857144*G0_1_0_2_1_2 - 0.0361607142857141*G0_1_0_2_2_0 + 0.0361607142857144*G0_1_0_2_2_1 + 0.144642857142858*G0_1_0_2_2_2;
    A[89] = A[96] - 0.120535714285714*G0_0_0_0_0_0 - 0.0241071428571429*G0_0_0_0_0_2 + 0.0120535714285714*G0_0_0_0_1_1 - 0.0241071428571429*G0_0_0_0_2_0 + 0.0120535714285714*G0_0_0_0_2_2 + 0.0120535714285714*G0_0_0_1_0_1 + 0.0120535714285714*G0_0_0_1_1_0 + 0.204910714285714*G0_0_0_1_1_1 + 0.0241071428571427*G0_0_0_1_1_2 + 0.0241071428571427*G0_0_0_1_2_1 - 0.0361607142857143*G0_0_0_1_2_2 - 0.0241071428571429*G0_0_0_2_0_0 + 0.0120535714285714*G0_0_0_2_0_2 + 0.0241071428571427*G0_0_0_2_1_1 - 0.0361607142857143*G0_0_0_2_1_2 + 0.0120535714285714*G0_0_0_2_2_0 - 0.0361607142857143*G0_0_0_2_2_1 - 0.048214285714286*G0_0_0_2_2_2 + 0.0723214285714286*G0_0_1_0_0_1 - 0.0723214285714285*G0_0_1_0_0_2 + 0.0723214285714286*G0_0_1_0_1_0 + 0.0723214285714286*G0_0_1_0_1_1 - 0.0723214285714285*G0_0_1_0_2_0 - 0.0723214285714284*G0_0_1_0_2_2 + 0.0723214285714286*G0_0_1_1_0_0 + 0.0723214285714286*G0_0_1_1_0_1 + 0.0723214285714286*G0_0_1_1_1_0 + 0.180803571428571*G0_0_1_1_1_1 + 0.0361607142857142*G0_0_1_1_1_2 + 0.0361607142857142*G0_0_1_1_2_1 - 0.0361607142857142*G0_0_1_1_2_2 - 0.0723214285714285*G0_0_1_2_0_0 - 0.0723214285714284*G0_0_1_2_0_2 + 0.0361607142857142*G0_0_1_2_1_1 - 0.0361607142857142*G0_0_1_2_1_2 - 0.0723214285714284*G0_0_1_2_2_0 - 0.0361607142857142*G0_0_1_2_2_1 - 0.180803571428571*G0_0_1_2_2_2 + 0.0241071428571428*G0_1_0_0_1_1 - 0.0241071428571429*G0_1_0_0_2_2 + 0.0241071428571428*G0_1_0_1_0_1 + 0.0241071428571428*G0_1_0_1_1_0 + 0.289285714285713*G0_1_0_1_1_1 + 0.0482142857142855*G0_1_0_1_1_2 + 0.0482142857142855*G0_1_0_1_2_1 - 0.0482142857142858*G0_1_0_1_2_2 - 0.0241071428571429*G0_1_0_2_0_2 + 0.0482142857142855*G0_1_0_2_1_1 - 0.0482142857142858*G0_1_0_2_1_2 - 0.0241071428571429*G0_1_0_2_2_0 - 0.0482142857142858*G0_1_0_2_2_1 - 0.289285714285715*G0_1_0_2_2_2 + 0.120535714285714*G0_1_1_0_0_0 + 0.0241071428571428*G0_1_1_0_0_1 + 0.0241071428571428*G0_1_1_0_1_0 - 0.0120535714285715*G0_1_1_0_1_1 - 0.0120535714285714*G0_1_1_0_2_2 + 0.0241071428571428*G0_1_1_1_0_0 - 0.0120535714285715*G0_1_1_1_0_1 - 0.0120535714285715*G0_1_1_1_1_0 + 0.0482142857142852*G0_1_1_1_1_1 + 0.0361607142857142*G0_1_1_1_1_2 + 0.0361607142857142*G0_1_1_1_2_1 - 0.0241071428571429*G0_1_1_1_2_2 - 0.0120535714285714*G0_1_1_2_0_2 + 0.0361607142857142*G0_1_1_2_1_1 - 0.0241071428571429*G0_1_1_2_1_2 - 0.0120535714285714*G0_1_1_2_2_0 - 0.0241071428571429*G0_1_1_2_2_1 - 0.204910714285715*G0_1_1_2_2_2;
    A[94] = -A[96] + 0.108482142857143*G0_0_1_0_0_0 + 0.0120535714285714*G0_0_1_0_0_1 + 0.0964285714285713*G0_0_1_0_0_2 + 0.0120535714285714*G0_0_1_0_1_0 - 0.0120535714285715*G0_0_1_0_1_1 + 0.0964285714285713*G0_0_1_0_2_0 + 0.0723214285714284*G0_0_1_0_2_2 + 0.0120535714285714*G0_0_1_1_0_0 - 0.0120535714285715*G0_0_1_1_0_1 - 0.0120535714285715*G0_0_1_1_1_0 - 0.108482142857143*G0_0_1_1_1_1 - 0.0964285714285714*G0_0_1_1_1_2 - 0.0964285714285714*G0_0_1_1_2_1 - 0.0723214285714285*G0_0_1_1_2_2 + 0.0964285714285712*G0_0_1_2_0_0 + 0.0723214285714284*G0_0_1_2_0_2 - 0.0964285714285714*G0_0_1_2_1_1 - 0.0723214285714285*G0_0_1_2_1_2 + 0.0723214285714284*G0_0_1_2_2_0 - 0.0723214285714285*G0_0_1_2_2_1 - 0.180803571428572*G0_1_1_0_0_0 - 0.0964285714285715*G0_1_1_0_0_1 - 0.0120535714285715*G0_1_1_0_0_2 - 0.0964285714285715*G0_1_1_0_1_0 - 0.108482142857143*G0_1_1_0_1_1 - 0.0241071428571428*G0_1_1_0_1_2 - 0.0120535714285715*G0_1_1_0_2_0 - 0.0241071428571428*G0_1_1_0_2_1 - 0.0241071428571428*G0_1_1_0_2_2 - 0.0964285714285715*G0_1_1_1_0_0 - 0.108482142857143*G0_1_1_1_0_1 - 0.0241071428571428*G0_1_1_1_0_2 - 0.108482142857143*G0_1_1_1_1_0 - 0.289285714285714*G0_1_1_1_1_1 - 0.108482142857143*G0_1_1_1_1_2 - 0.0241071428571428*G0_1_1_1_2_0 - 0.108482142857143*G0_1_1_1_2_1 - 0.0964285714285713*G0_1_1_1_2_2 - 0.0120535714285715*G0_1_1_2_0_0 - 0.0241071428571428*G0_1_1_2_0_1 - 0.0241071428571428*G0_1_1_2_0_2 - 0.0241071428571428*G0_1_1_2_1_0 - 0.108482142857143*G0_1_1_2_1_1 - 0.0964285714285713*G0_1_1_2_1_2 - 0.0241071428571428*G0_1_1_2_2_0 - 0.0964285714285713*G0_1_1_2_2_1 - 0.0723214285714281*G0_1_1_2_2_2;
    A[98] = A[89] - 0.0723214285714286*G0_0_1_0_0_0 - 0.0723214285714286*G0_0_1_0_0_1 - 0.0723214285714286*G0_0_1_0_1_0 - 0.0361607142857143*G0_0_1_0_1_1 + 0.0120535714285714*G0_0_1_0_2_2 - 0.0723214285714286*G0_0_1_1_0_0 - 0.0361607142857143*G0_0_1_1_0_1 - 0.0361607142857143*G0_0_1_1_1_0 + 0.144642857142857*G0_0_1_1_1_1 + 0.0361607142857142*G0_0_1_1_1_2 + 0.0361607142857142*G0_0_1_1_2_1 + 0.0241071428571428*G0_0_1_1_2_2 + 0.0120535714285714*G0_0_1_2_0_2 + 0.0361607142857142*G0_0_1_2_1_1 + 0.0241071428571428*G0_0_1_2_1_2 + 0.0120535714285714*G0_0_1_2_2_0 + 0.0241071428571428*G0_0_1_2_2_1 + 0.0361607142857141*G0_0_1_2_2_2 + 0.0723214285714286*G0_1_0_0_0_0 + 0.0723214285714286*G0_1_0_0_0_1 + 0.0723214285714286*G0_1_0_0_1_0 + 0.0361607142857143*G0_1_0_0_1_1 - 0.0120535714285714*G0_1_0_0_2_2 + 0.0723214285714286*G0_1_0_1_0_0 + 0.0361607142857143*G0_1_0_1_0_1 + 0.0361607142857143*G0_1_0_1_1_0 - 0.144642857142857*G0_1_0_1_1_1 - 0.0361607142857142*G0_1_0_1_1_2 - 0.0361607142857142*G0_1_0_1_2_1 - 0.0241071428571428*G0_1_0_1_2_2 - 0.0120535714285714*G0_1_0_2_0_2 - 0.0361607142857142*G0_1_0_2_1_1 - 0.0241071428571428*G0_1_0_2_1_2 - 0.0120535714285714*G0_1_0_2_2_0 - 0.0241071428571428*G0_1_0_2_2_1 - 0.036160714285714*G0_1_0_2_2_2;
    A[93] = -A[98] - 0.180803571428571*G0_0_0_0_0_0 - 0.0120535714285714*G0_0_0_0_0_1 - 0.0964285714285714*G0_0_0_0_0_2 - 0.0120535714285714*G0_0_0_0_1_0 - 0.0241071428571429*G0_0_0_0_1_1 - 0.0241071428571428*G0_0_0_0_1_2 - 0.0964285714285714*G0_0_0_0_2_0 - 0.0241071428571428*G0_0_0_0_2_1 - 0.108482142857143*G0_0_0_0_2_2 - 0.0120535714285714*G0_0_0_1_0_0 - 0.0241071428571429*G0_0_0_1_0_1 - 0.0241071428571428*G0_0_0_1_0_2 - 0.0241071428571429*G0_0_0_1_1_0 - 0.0723214285714288*G0_0_0_1_1_1 - 0.0964285714285713*G0_0_0_1_1_2 - 0.0241071428571428*G0_0_0_1_2_0 - 0.0964285714285713*G0_0_0_1_2_1 - 0.108482142857143*G0_0_0_1_2_2 - 0.0964285714285714*G0_0_0_2_0_0 - 0.0241071428571428*G0_0_0_2_0_1 - 0.108482142857143*G0_0_0_2_0_2 - 0.0241071428571428*G0_0_0_2_1_0 - 0.0964285714285713*G0_0_0_2_1_1 - 0.108482142857143*G0_0_0_2_1_2 - 0.108482142857143*G0_0_0_2_2_0 - 0.108482142857143*G0_0_0_2_2_1 - 0.289285714285714*G0_0_0_2_2_2 + 0.108482142857143*G0_1_0_0_0_0 + 0.0964285714285714*G0_1_0_0_0_1 + 0.0120535714285714*G0_1_0_0_0_2 + 0.0964285714285715*G0_1_0_0_1_0 + 0.0723214285714286*G0_1_0_0_1_1 + 0.0120535714285714*G0_1_0_0_2_0 - 0.0120535714285714*G0_1_0_0_2_2 + 0.0964285714285715*G0_1_0_1_0_0 + 0.0723214285714286*G0_1_0_1_0_1 + 0.0723214285714286*G0_1_0_1_1_0 - 0.0723214285714285*G0_1_0_1_1_2 - 0.0723214285714285*G0_1_0_1_2_1 - 0.0964285714285712*G0_1_0_1_2_2 + 0.0120535714285714*G0_1_0_2_0_0 - 0.0120535714285714*G0_1_0_2_0_2 - 0.0723214285714285*G0_1_0_2_1_1 - 0.0964285714285712*G0_1_0_2_1_2 - 0.0120535714285714*G0_1_0_2_2_0 - 0.0964285714285712*G0_1_0_2_2_1 - 0.108482142857142*G0_1_0_2_2_2;
    A[39] = A[93] + 0.0361607142857141*G0_0_1_0_0_0 + 0.0241071428571428*G0_0_1_0_0_1 + 0.0120535714285714*G0_0_1_0_0_2 + 0.0241071428571428*G0_0_1_0_1_0 + 0.0361607142857142*G0_0_1_0_1_1 + 0.0120535714285714*G0_0_1_0_2_0 + 0.0241071428571428*G0_0_1_1_0_0 + 0.0361607142857142*G0_0_1_1_0_1 + 0.0361607142857142*G0_0_1_1_1_0 + 0.144642857142857*G0_0_1_1_1_1 - 0.0361607142857143*G0_0_1_1_1_2 - 0.0361607142857143*G0_0_1_1_2_1 - 0.0723214285714284*G0_0_1_1_2_2 + 0.0120535714285714*G0_0_1_2_0_0 - 0.0361607142857143*G0_0_1_2_1_1 - 0.0723214285714284*G0_0_1_2_1_2 - 0.0723214285714284*G0_0_1_2_2_1 - 0.0723214285714283*G0_0_1_2_2_2 - 0.0361607142857141*G0_1_0_0_0_0 - 0.0241071428571428*G0_1_0_0_0_1 - 0.0120535714285714*G0_1_0_0_0_2 - 0.0241071428571428*G0_1_0_0_1_0 - 0.0361607142857142*G0_1_0_0_1_1 - 0.0120535714285714*G0_1_0_0_2_0 - 0.0241071428571428*G0_1_0_1_0_0 - 0.0361607142857142*G0_1_0_1_0_1 - 0.0361607142857142*G0_1_0_1_1_0 - 0.144642857142857*G0_1_0_1_1_1 + 0.0361607142857143*G0_1_0_1_1_2 + 0.0361607142857143*G0_1_0_1_2_1 + 0.0723214285714284*G0_1_0_1_2_2 - 0.0120535714285714*G0_1_0_2_0_0 + 0.0361607142857143*G0_1_0_2_1_1 + 0.0723214285714284*G0_1_0_2_1_2 + 0.0723214285714284*G0_1_0_2_2_1 + 0.0723214285714283*G0_1_0_2_2_2;
    A[65] = -A[45] - 0.0904017857142855*G0_1_0_0_0_0 - 0.0180803571428571*G0_1_0_0_0_1 - 0.088392857142857*G0_1_0_0_0_2 - 0.0180803571428571*G0_1_0_0_1_0 - 0.00602678571428567*G0_1_0_0_1_1 - 0.0200892857142857*G0_1_0_0_1_2 - 0.088392857142857*G0_1_0_0_2_0 - 0.0200892857142857*G0_1_0_0_2_1 - 0.0843749999999999*G0_1_0_0_2_2 - 0.0180803571428571*G0_1_0_1_0_0 - 0.00602678571428567*G0_1_0_1_0_1 - 0.0200892857142857*G0_1_0_1_0_2 - 0.00602678571428567*G0_1_0_1_1_0 + 0.0180803571428573*G0_1_0_1_1_1 + 0.0080357142857143*G0_1_0_1_1_2 - 0.0200892857142857*G0_1_0_1_2_0 + 0.0080357142857143*G0_1_0_1_2_1 - 0.0120535714285714*G0_1_0_1_2_2 - 0.088392857142857*G0_1_0_2_0_0 - 0.0200892857142857*G0_1_0_2_0_1 - 0.0843749999999999*G0_1_0_2_0_2 - 0.0200892857142857*G0_1_0_2_1_0 + 0.0080357142857143*G0_1_0_2_1_1 - 0.0120535714285714*G0_1_0_2_1_2 - 0.0843749999999999*G0_1_0_2_2_0 - 0.0120535714285714*G0_1_0_2_2_1 - 0.216964285714286*G0_1_0_2_2_2 - 0.331473214285714*G0_1_1_0_0_0 - 0.0602678571428571*G0_1_1_0_0_1 - 0.0703124999999998*G0_1_1_0_0_2 - 0.0602678571428571*G0_1_1_0_1_0 - 0.0220982142857143*G0_1_1_0_1_1 - 0.0200892857142857*G0_1_1_0_1_2 - 0.0703124999999998*G0_1_1_0_2_0 - 0.0200892857142857*G0_1_1_0_2_1 - 0.0683035714285713*G0_1_1_0_2_2 - 0.0602678571428571*G0_1_1_1_0_0 - 0.0220982142857143*G0_1_1_1_0_1 - 0.0200892857142857*G0_1_1_1_0_2 - 0.0220982142857143*G0_1_1_1_1_0 + 0.0120535714285714*G0_1_1_1_1_1 + 0.00200892857142857*G0_1_1_1_1_2 - 0.0200892857142857*G0_1_1_1_2_0 + 0.00200892857142857*G0_1_1_1_2_1 - 0.0160714285714286*G0_1_1_1_2_2 - 0.0703124999999998*G0_1_1_2_0_0 - 0.0200892857142857*G0_1_1_2_0_1 - 0.0683035714285713*G0_1_1_2_0_2 - 0.0200892857142857*G0_1_1_2_1_0 + 0.00200892857142857*G0_1_1_2_1_1 - 0.0160714285714286*G0_1_1_2_1_2 - 0.0683035714285713*G0_1_1_2_2_0 - 0.0160714285714286*G0_1_1_2_2_1 - 0.204910714285714*G0_1_1_2_2_2;
    A[53] = -A[58] + 0.114508928571429*G0_0_0_0_0_0 - 0.0180803571428571*G0_0_0_0_0_1 + 0.0522321428571429*G0_0_0_0_0_2 - 0.0180803571428571*G0_0_0_0_1_0 - 0.0200892857142857*G0_0_0_0_1_1 - 0.00602678571428569*G0_0_0_0_1_2 + 0.0522321428571429*G0_0_0_0_2_0 - 0.00602678571428569*G0_0_0_0_2_1 + 0.0421874999999999*G0_0_0_0_2_2 - 0.0180803571428571*G0_0_0_1_0_0 - 0.0200892857142857*G0_0_0_1_0_1 - 0.00602678571428569*G0_0_0_1_0_2 - 0.0200892857142857*G0_0_0_1_1_0 - 0.0120535714285712*G0_0_0_1_1_1 + 0.00803571428571429*G0_0_0_1_1_2 - 0.00602678571428569*G0_0_0_1_2_0 + 0.00803571428571429*G0_0_0_1_2_1 + 0.0180803571428571*G0_0_0_1_2_2 + 0.0522321428571429*G0_0_0_2_0_0 - 0.00602678571428569*G0_0_0_2_0_1 + 0.0421874999999999*G0_0_0_2_0_2 - 0.00602678571428569*G0_0_0_2_1_0 + 0.00803571428571429*G0_0_0_2_1_1 + 0.0180803571428571*G0_0_0_2_1_2 + 0.0421874999999999*G0_0_0_2_2_0 + 0.0180803571428571*G0_0_0_2_2_1 + 0.0241071428571424*G0_0_0_2_2_2 - 0.1265625*G0_1_0_0_0_0 + 0.0100446428571429*G0_1_0_0_0_2 - 0.00401785714285712*G0_1_0_0_1_1 - 0.0060267857142857*G0_1_0_0_1_2 + 0.0100446428571429*G0_1_0_0_2_0 - 0.0060267857142857*G0_1_0_0_2_1 + 0.0261160714285714*G0_1_0_0_2_2 - 0.00401785714285712*G0_1_0_1_0_1 - 0.0060267857142857*G0_1_0_1_0_2 - 0.00401785714285712*G0_1_0_1_1_0 + 0.00401785714285714*G0_1_0_1_1_2 - 0.0060267857142857*G0_1_0_1_2_0 + 0.00401785714285713*G0_1_0_1_2_1 + 0.0120535714285714*G0_1_0_1_2_2 + 0.0100446428571429*G0_1_0_2_0_0 - 0.0060267857142857*G0_1_0_2_0_1 + 0.0261160714285714*G0_1_0_2_0_2 - 0.0060267857142857*G0_1_0_2_1_0 + 0.00401785714285714*G0_1_0_2_1_1 + 0.0120535714285714*G0_1_0_2_1_2 + 0.0261160714285714*G0_1_0_2_2_0 + 0.0120535714285714*G0_1_0_2_2_1 + 0.0180803571428567*G0_1_0_2_2_2;
    A[70] = A[7] - 0.412499999999999*G0_0_1_0_0_0 - 0.0388392857142857*G0_0_1_0_0_1 - 0.0388392857142856*G0_0_1_0_0_2 - 0.0388392857142857*G0_0_1_0_1_0 - 0.00714285714285713*G0_0_1_0_1_1 - 0.00357142857142856*G0_0_1_0_1_2 - 0.0388392857142856*G0_0_1_0_2_0 - 0.00357142857142856*G0_0_1_0_2_1 - 0.00714285714285713*G0_0_1_0_2_2 - 0.0388392857142857*G0_0_1_1_0_0 - 0.00714285714285713*G0_0_1_1_0_1 - 0.00357142857142856*G0_0_1_1_0_2 - 0.00714285714285713*G0_0_1_1_1_0 - 0.000669642857142813*G0_0_1_1_1_1 - 0.000223214285714281*G0_0_1_1_1_2 - 0.00357142857142856*G0_0_1_1_2_0 - 0.000223214285714283*G0_0_1_1_2_1 - 0.000223214285714288*G0_0_1_1_2_2 - 0.0388392857142856*G0_0_1_2_0_0 - 0.00357142857142856*G0_0_1_2_0_1 - 0.00714285714285713*G0_0_1_2_0_2 - 0.00357142857142856*G0_0_1_2_1_0 - 0.00022321428571428*G0_0_1_2_1_1 - 0.000223214285714287*G0_0_1_2_1_2 - 0.00714285714285713*G0_0_1_2_2_0 - 0.000223214285714287*G0_0_1_2_2_1 - 0.000669642857142858*G0_0_1_2_2_2 + 0.412499999999999*G0_1_0_0_0_0 + 0.0388392857142857*G0_1_0_0_0_1 + 0.0388392857142856*G0_1_0_0_0_2 + 0.0388392857142857*G0_1_0_0_1_0 + 0.00714285714285713*G0_1_0_0_1_1 + 0.00357142857142856*G0_1_0_0_1_2 + 0.0388392857142856*G0_1_0_0_2_0 + 0.00357142857142856*G0_1_0_0_2_1 + 0.00714285714285713*G0_1_0_0_2_2 + 0.0388392857142857*G0_1_0_1_0_0 + 0.00714285714285713*G0_1_0_1_0_1 + 0.00357142857142856*G0_1_0_1_0_2 + 0.00714285714285713*G0_1_0_1_1_0 + 0.000669642857142813*G0_1_0_1_1_1 + 0.00022321428571428*G0_1_0_1_1_2 + 0.00357142857142856*G0_1_0_1_2_0 + 0.000223214285714278*G0_1_0_1_2_1 + 0.000223214285714286*G0_1_0_1_2_2 + 0.0388392857142856*G0_1_0_2_0_0 + 0.00357142857142856*G0_1_0_2_0_1 + 0.00714285714285713*G0_1_0_2_0_2 + 0.00357142857142856*G0_1_0_2_1_0 + 0.00022321428571428*G0_1_0_2_1_1 + 0.000223214285714288*G0_1_0_2_1_2 + 0.00714285714285713*G0_1_0_2_2_0 + 0.000223214285714287*G0_1_0_2_2_1 + 0.00066964285714286*G0_1_0_2_2_2;
    A[5] = A[70] + 0.412499999999999*G0_0_0_0_0_0 + 0.03125*G0_0_0_0_0_1 + 0.0464285714285714*G0_0_0_0_0_2 + 0.03125*G0_0_0_0_1_0 + 0.00937499999999999*G0_0_0_0_1_1 + 0.00357142857142857*G0_0_0_0_1_2 + 0.0464285714285714*G0_0_0_0_2_0 + 0.00357142857142857*G0_0_0_0_2_1 + 0.00491071428571428*G0_0_0_0_2_2 + 0.03125*G0_0_0_1_0_0 + 0.00937499999999999*G0_0_0_1_0_1 + 0.00357142857142857*G0_0_0_1_0_2 + 0.00937499999999999*G0_0_0_1_1_0 + 0.0348214285714286*G0_0_0_1_1_1 + 0.00401785714285713*G0_0_0_1_1_2 + 0.00357142857142857*G0_0_0_1_2_0 + 0.00401785714285713*G0_0_0_1_2_1 - 0.00357142857142858*G0_0_0_1_2_2 + 0.0464285714285714*G0_0_0_2_0_0 + 0.00357142857142857*G0_0_0_2_0_1 + 0.00491071428571428*G0_0_0_2_0_2 + 0.00357142857142857*G0_0_0_2_1_0 + 0.00401785714285713*G0_0_0_2_1_1 - 0.00357142857142858*G0_0_0_2_1_2 + 0.00491071428571428*G0_0_0_2_2_0 - 0.00357142857142858*G0_0_0_2_2_1 - 0.0334821428571429*G0_0_0_2_2_2 - 0.0075892857142857*G0_0_1_0_0_1 + 0.00758928571428571*G0_0_1_0_0_2 - 0.0075892857142857*G0_0_1_0_1_0 + 0.00223214285714285*G0_0_1_0_1_1 + 0.00758928571428572*G0_0_1_0_2_0 - 0.00223214285714286*G0_0_1_0_2_2 - 0.0075892857142857*G0_0_1_1_0_0 + 0.00223214285714285*G0_0_1_1_0_1 + 0.00223214285714285*G0_0_1_1_1_0 + 0.0341517857142857*G0_0_1_1_1_1 + 0.00379464285714284*G0_0_1_1_1_2 + 0.00379464285714284*G0_0_1_1_2_1 - 0.00379464285714287*G0_0_1_1_2_2 + 0.00758928571428572*G0_0_1_2_0_0 - 0.00223214285714286*G0_0_1_2_0_2 + 0.00379464285714284*G0_0_1_2_1_1 - 0.00379464285714286*G0_0_1_2_1_2 - 0.00223214285714286*G0_0_1_2_2_0 - 0.00379464285714286*G0_0_1_2_2_1 - 0.0341517857142858*G0_0_1_2_2_2 - 0.00758928571428571*G0_1_0_0_0_1 + 0.00758928571428571*G0_1_0_0_0_2 - 0.00758928571428571*G0_1_0_0_1_0 + 0.00223214285714285*G0_1_0_0_1_1 + 0.00758928571428571*G0_1_0_0_2_0 - 0.00223214285714286*G0_1_0_0_2_2 - 0.00758928571428571*G0_1_0_1_0_0 + 0.00223214285714286*G0_1_0_1_0_1 + 0.00223214285714286*G0_1_0_1_1_0 + 0.0341517857142857*G0_1_0_1_1_1 + 0.00379464285714285*G0_1_0_1_1_2 + 0.00379464285714285*G0_1_0_1_2_1 - 0.00379464285714287*G0_1_0_1_2_2 + 0.00758928571428571*G0_1_0_2_0_0 - 0.00223214285714286*G0_1_0_2_0_2 + 0.00379464285714285*G0_1_0_2_1_1 - 0.00379464285714287*G0_1_0_2_1_2 - 0.00223214285714286*G0_1_0_2_2_0 - 0.00379464285714287*G0_1_0_2_2_1 - 0.0341517857142858*G0_1_0_2_2_2 - 0.4125*G0_1_1_0_0_0 - 0.0464285714285714*G0_1_1_0_0_1 - 0.0312499999999999*G0_1_1_0_0_2 - 0.0464285714285714*G0_1_1_0_1_0 - 0.00491071428571429*G0_1_1_0_1_1 - 0.00357142857142857*G0_1_1_0_1_2 - 0.0312499999999999*G0_1_1_0_2_0 - 0.00357142857142857*G0_1_1_0_2_1 - 0.009375*G0_1_1_0_2_2 - 0.0464285714285714*G0_1_1_1_0_0 - 0.00491071428571429*G0_1_1_1_0_1 - 0.00357142857142857*G0_1_1_1_0_2 - 0.00491071428571429*G0_1_1_1_1_0 + 0.0334821428571428*G0_1_1_1_1_1 + 0.00357142857142855*G0_1_1_1_1_2 - 0.00357142857142857*G0_1_1_1_2_0 + 0.00357142857142856*G0_1_1_1_2_1 - 0.00401785714285715*G0_1_1_1_2_2 - 0.0312499999999999*G0_1_1_2_0_0 - 0.00357142857142857*G0_1_1_2_0_1 - 0.009375*G0_1_1_2_0_2 - 0.00357142857142857*G0_1_1_2_1_0 + 0.00357142857142856*G0_1_1_2_1_1 - 0.00401785714285716*G0_1_1_2_1_2 - 0.009375*G0_1_1_2_2_0 - 0.00401785714285715*G0_1_1_2_2_1 - 0.0348214285714286*G0_1_1_2_2_2;
    A[50] = A[5] + 0.412499999999999*G0_0_1_0_0_0 + 0.0388392857142857*G0_0_1_0_0_1 + 0.0388392857142856*G0_0_1_0_0_2 + 0.0388392857142857*G0_0_1_0_1_0 + 0.00714285714285714*G0_0_1_0_1_1 + 0.00357142857142856*G0_0_1_0_1_2 + 0.0388392857142856*G0_0_1_0_2_0 + 0.00357142857142856*G0_0_1_0_2_1 + 0.00714285714285713*G0_0_1_0_2_2 + 0.0388392857142857*G0_0_1_1_0_0 + 0.00714285714285714*G0_0_1_1_0_1 + 0.00357142857142856*G0_0_1_1_0_2 + 0.00714285714285714*G0_0_1_1_1_0 + 0.000669642857142872*G0_0_1_1_1_1 + 0.000223214285714287*G0_0_1_1_1_2 + 0.00357142857142856*G0_0_1_1_2_0 + 0.000223214285714285*G0_0_1_1_2_1 + 0.000223214285714285*G0_0_1_1_2_2 + 0.0388392857142856*G0_0_1_2_0_0 + 0.00357142857142856*G0_0_1_2_0_1 + 0.00714285714285713*G0_0_1_2_0_2 + 0.00357142857142856*G0_0_1_2_1_0 + 0.000223214285714287*G0_0_1_2_1_1 + 0.000223214285714283*G0_0_1_2_1_2 + 0.00714285714285713*G0_0_1_2_2_0 + 0.000223214285714283*G0_0_1_2_2_1 + 0.000669642857142855*G0_0_1_2_2_2 - 0.412499999999999*G0_1_0_0_0_0 - 0.0388392857142857*G0_1_0_0_0_1 - 0.0388392857142856*G0_1_0_0_0_2 - 0.0388392857142857*G0_1_0_0_1_0 - 0.00714285714285714*G0_1_0_0_1_1 - 0.00357142857142856*G0_1_0_0_1_2 - 0.0388392857142856*G0_1_0_0_2_0 - 0.00357142857142856*G0_1_0_0_2_1 - 0.00714285714285713*G0_1_0_0_2_2 - 0.0388392857142857*G0_1_0_1_0_0 - 0.00714285714285714*G0_1_0_1_0_1 - 0.00357142857142856*G0_1_0_1_0_2 - 0.00714285714285714*G0_1_0_1_1_0 - 0.00066964285714287*G0_1_0_1_1_1 - 0.000223214285714287*G0_1_0_1_1_2 - 0.00357142857142856*G0_1_0_1_2_0 - 0.000223214285714285*G0_1_0_1_2_1 - 0.000223214285714283*G0_1_0_1_2_2 - 0.0388392857142856*G0_1_0_2_0_0 - 0.00357142857142856*G0_1_0_2_0_1 - 0.00714285714285713*G0_1_0_2_0_2 - 0.00357142857142856*G0_1_0_2_1_0 - 0.000223214285714287*G0_1_0_2_1_1 - 0.000223214285714283*G0_1_0_2_1_2 - 0.00714285714285713*G0_1_0_2_2_0 - 0.000223214285714283*G0_1_0_2_2_1 - 0.000669642857142848*G0_1_0_2_2_2;
    A[6] = -A[4] + 0.141964285714286*G0_0_1_0_0_0 + 0.0191964285714286*G0_0_1_0_0_1 + 0.00758928571428569*G0_0_1_0_0_2 + 0.0191964285714286*G0_0_1_0_1_0 + 0.00312500000000001*G0_0_1_0_1_1 + 0.00379464285714285*G0_0_1_0_1_2 + 0.00758928571428569*G0_0_1_0_2_0 + 0.00379464285714285*G0_0_1_0_2_1 + 0.00803571428571427*G0_0_1_0_2_2 + 0.0191964285714286*G0_0_1_1_0_0 + 0.00312500000000001*G0_0_1_1_0_1 + 0.00379464285714285*G0_0_1_1_0_2 + 0.00312500000000001*G0_0_1_1_1_0 - 0.00267857142857137*G0_0_1_1_1_1 - 0.0044642857142857*G0_0_1_1_1_2 + 0.00379464285714285*G0_0_1_1_2_0 - 0.0044642857142857*G0_0_1_1_2_1 + 0.0075892857142857*G0_0_1_2_0_0 + 0.00379464285714285*G0_0_1_2_0_1 + 0.00803571428571427*G0_0_1_2_0_2 + 0.00379464285714285*G0_0_1_2_1_0 - 0.0044642857142857*G0_0_1_2_1_1 + 0.00803571428571427*G0_0_1_2_2_0 + 0.0375000000000001*G0_0_1_2_2_2 + 0.141964285714286*G0_1_1_0_0_0 + 0.0191964285714286*G0_1_1_0_0_1 + 0.0075892857142857*G0_1_1_0_0_2 + 0.0191964285714286*G0_1_1_0_1_0 + 0.00312500000000001*G0_1_1_0_1_1 + 0.00379464285714285*G0_1_1_0_1_2 + 0.0075892857142857*G0_1_1_0_2_0 + 0.00379464285714285*G0_1_1_0_2_1 + 0.00803571428571428*G0_1_1_0_2_2 + 0.0191964285714286*G0_1_1_1_0_0 + 0.00312500000000001*G0_1_1_1_0_1 + 0.00379464285714285*G0_1_1_1_0_2 + 0.00312500000000001*G0_1_1_1_1_0 - 0.0026785714285714*G0_1_1_1_1_1 - 0.0044642857142857*G0_1_1_1_1_2 + 0.00379464285714285*G0_1_1_1_2_0 - 0.0044642857142857*G0_1_1_1_2_1 + 0.0075892857142857*G0_1_1_2_0_0 + 0.00379464285714285*G0_1_1_2_0_1 + 0.00803571428571428*G0_1_1_2_0_2 + 0.00379464285714285*G0_1_1_2_1_0 - 0.0044642857142857*G0_1_1_2_1_1 + 0.00803571428571428*G0_1_1_2_2_0 + 0.0375000000000001*G0_1_1_2_2_2;
    A[17] = -A[14] + 0.0629464285714285*G0_0_0_0_0_0 + 0.00580357142857142*G0_0_0_0_0_1 + 0.0142857142857143*G0_0_0_0_0_2 + 0.00580357142857142*G0_0_0_0_1_0 + 0.00267857142857141*G0_0_0_0_1_1 + 0.00223214285714285*G0_0_0_0_1_2 + 0.0142857142857143*G0_0_0_0_2_0 + 0.00223214285714285*G0_0_0_0_2_1 + 0.0129464285714286*G0_0_0_0_2_2 + 0.00580357142857142*G0_0_0_1_0_0 + 0.00267857142857141*G0_0_0_1_0_1 + 0.00223214285714285*G0_0_0_1_0_2 + 0.00267857142857141*G0_0_0_1_1_0 + 0.107142857142857*G0_0_0_1_1_1 + 0.0107142857142857*G0_0_0_1_1_2 + 0.00223214285714285*G0_0_0_1_2_0 + 0.0107142857142857*G0_0_0_1_2_1 + 0.000446428571428581*G0_0_0_1_2_2 + 0.0142857142857143*G0_0_0_2_0_0 + 0.00223214285714285*G0_0_0_2_0_1 + 0.0129464285714286*G0_0_0_2_0_2 + 0.00223214285714285*G0_0_0_2_1_0 + 0.0107142857142857*G0_0_0_2_1_1 + 0.00044642857142858*G0_0_0_2_1_2 + 0.0129464285714286*G0_0_0_2_2_0 + 0.00044642857142858*G0_0_0_2_2_1 + 0.0508928571428573*G0_0_0_2_2_2 + 0.0120535714285714*G0_0_1_0_0_0 + 0.00535714285714286*G0_0_1_0_0_1 + 0.00133928571428571*G0_0_1_0_0_2 + 0.00535714285714286*G0_0_1_0_1_0 - 0.00803571428571428*G0_0_1_0_1_1 + 0.00133928571428572*G0_0_1_0_2_0 - 0.0013392857142857*G0_0_1_0_2_2 + 0.00535714285714286*G0_0_1_1_0_0 - 0.00803571428571428*G0_0_1_1_0_1 - 0.00803571428571428*G0_0_1_1_1_0 + 0.00803571428571428*G0_0_1_1_1_2 + 0.00803571428571428*G0_0_1_1_2_1 - 0.00535714285714284*G0_0_1_1_2_2 + 0.00133928571428572*G0_0_1_2_0_0 - 0.0013392857142857*G0_0_1_2_0_2 + 0.00803571428571428*G0_0_1_2_1_1 - 0.00535714285714284*G0_0_1_2_1_2 - 0.0013392857142857*G0_0_1_2_2_0 - 0.00535714285714285*G0_0_1_2_2_1 - 0.0120535714285713*G0_0_1_2_2_2;
    A[34] = -A[36] - 0.0180803571428571*G0_0_1_0_0_0 + 0.00602678571428571*G0_0_1_0_0_1 - 0.00803571428571428*G0_0_1_0_0_2 + 0.00602678571428571*G0_0_1_0_1_0 + 0.0180803571428571*G0_0_1_0_1_1 + 0.0200892857142857*G0_0_1_0_1_2 - 0.00803571428571428*G0_0_1_0_2_0 + 0.0200892857142857*G0_0_1_0_2_1 + 0.0120535714285714*G0_0_1_0_2_2 + 0.00602678571428571*G0_0_1_1_0_0 + 0.0180803571428571*G0_0_1_1_0_1 + 0.0200892857142857*G0_0_1_1_0_2 + 0.0180803571428571*G0_0_1_1_1_0 + 0.0904017857142858*G0_0_1_1_1_1 + 0.0883928571428571*G0_0_1_1_1_2 + 0.0200892857142857*G0_0_1_1_2_0 + 0.0883928571428571*G0_0_1_1_2_1 + 0.0843749999999999*G0_0_1_1_2_2 - 0.00803571428571428*G0_0_1_2_0_0 + 0.0200892857142857*G0_0_1_2_0_1 + 0.0120535714285714*G0_0_1_2_0_2 + 0.0200892857142857*G0_0_1_2_1_0 + 0.0883928571428571*G0_0_1_2_1_1 + 0.0843749999999999*G0_0_1_2_1_2 + 0.0120535714285714*G0_0_1_2_2_0 + 0.0843749999999999*G0_0_1_2_2_1 + 0.216964285714286*G0_0_1_2_2_2 - 0.00602678571428579*G0_1_1_0_0_0 - 0.0160714285714286*G0_1_1_0_0_1 - 0.00602678571428569*G0_1_1_0_0_2 - 0.0160714285714286*G0_1_1_0_1_0 - 0.0421874999999999*G0_1_1_0_1_1 - 0.00602678571428569*G0_1_1_0_2_0 - 0.00401785714285712*G0_1_1_0_2_2 - 0.0160714285714286*G0_1_1_1_0_0 - 0.0421874999999999*G0_1_1_1_0_1 - 0.0421874999999999*G0_1_1_1_1_0 - 0.241071428571428*G0_1_1_1_1_1 + 0.0180803571428572*G0_1_1_1_1_2 + 0.0180803571428572*G0_1_1_1_2_1 + 0.0160714285714286*G0_1_1_1_2_2 - 0.00602678571428569*G0_1_1_2_0_0 - 0.00401785714285712*G0_1_1_2_0_2 + 0.0180803571428572*G0_1_1_2_1_1 + 0.0160714285714286*G0_1_1_2_1_2 - 0.00401785714285712*G0_1_1_2_2_0 + 0.0160714285714286*G0_1_1_2_2_1 + 0.0120535714285713*G0_1_1_2_2_2;
    A[24] = -A[27] + 0.0267857142857142*G0_1_0_0_0_0 - 0.00223214285714286*G0_1_0_0_0_1 + 0.00624999999999999*G0_1_0_0_0_2 - 0.00223214285714287*G0_1_0_0_1_0 - 0.00535714285714286*G0_1_0_0_1_1 + 0.00424107142857143*G0_1_0_0_1_2 + 0.00624999999999999*G0_1_0_0_2_0 + 0.00424107142857143*G0_1_0_0_2_1 + 0.0370535714285714*G0_1_0_0_2_2 - 0.00223214285714287*G0_1_0_1_0_0 - 0.00535714285714286*G0_1_0_1_0_1 + 0.00424107142857143*G0_1_0_1_0_2 - 0.00535714285714286*G0_1_0_1_1_0 - 0.00133928571428575*G0_1_0_1_1_1 + 0.00669642857142858*G0_1_0_1_1_2 + 0.00424107142857143*G0_1_0_1_2_0 + 0.00669642857142858*G0_1_0_1_2_1 + 0.0406250000000001*G0_1_0_1_2_2 + 0.00624999999999999*G0_1_0_2_0_0 + 0.00424107142857143*G0_1_0_2_0_1 + 0.0370535714285714*G0_1_0_2_0_2 + 0.00424107142857143*G0_1_0_2_1_0 + 0.00669642857142858*G0_1_0_2_1_1 + 0.0406250000000001*G0_1_0_2_1_2 + 0.0370535714285714*G0_1_0_2_2_0 + 0.0406250000000001*G0_1_0_2_2_1 + 0.4125*G0_1_0_2_2_2 - 0.0361607142857143*G0_1_1_0_0_0 - 0.01875*G0_1_1_0_0_1 + 0.00133928571428572*G0_1_1_0_0_2 - 0.01875*G0_1_1_0_1_0 - 0.0200892857142857*G0_1_1_0_1_1 + 0.00133928571428572*G0_1_1_0_2_0 + 0.0107142857142857*G0_1_1_0_2_2 - 0.01875*G0_1_1_1_0_0 - 0.0200892857142857*G0_1_1_1_0_1 - 0.0200892857142857*G0_1_1_1_1_0 - 0.0482142857142857*G0_1_1_1_1_1 - 0.00401785714285712*G0_1_1_1_1_2 - 0.00401785714285712*G0_1_1_1_2_1 + 0.01875*G0_1_1_1_2_2 + 0.00133928571428572*G0_1_1_2_0_0 + 0.0107142857142857*G0_1_1_2_0_2 - 0.00401785714285712*G0_1_1_2_1_1 + 0.01875*G0_1_1_2_1_2 + 0.0107142857142857*G0_1_1_2_2_0 + 0.01875*G0_1_1_2_2_1 + 0.120535714285715*G0_1_1_2_2_2;
    A[26] = -A[24] - 0.0575892857142857*G0_1_1_0_0_0 - 0.0191964285714285*G0_1_1_0_0_1 - 0.0116071428571428*G0_1_1_0_0_2 - 0.0191964285714285*G0_1_1_0_1_0 - 0.0191964285714286*G0_1_1_0_1_1 - 0.00580357142857143*G0_1_1_0_1_2 - 0.0116071428571428*G0_1_1_0_2_0 - 0.00580357142857143*G0_1_1_0_2_1 - 0.0160714285714286*G0_1_1_0_2_2 - 0.0191964285714285*G0_1_1_1_0_0 - 0.0191964285714286*G0_1_1_1_0_1 - 0.00580357142857142*G0_1_1_1_0_2 - 0.0191964285714286*G0_1_1_1_1_0 - 0.0575892857142856*G0_1_1_1_1_1 - 0.0116071428571428*G0_1_1_1_1_2 - 0.00580357142857143*G0_1_1_1_2_0 - 0.0116071428571428*G0_1_1_1_2_1 - 0.0160714285714286*G0_1_1_1_2_2 - 0.0116071428571428*G0_1_1_2_0_0 - 0.00580357142857143*G0_1_1_2_0_1 - 0.0160714285714286*G0_1_1_2_0_2 - 0.00580357142857143*G0_1_1_2_1_0 - 0.0116071428571428*G0_1_1_2_1_1 - 0.0160714285714286*G0_1_1_2_1_2 - 0.0160714285714286*G0_1_1_2_2_0 - 0.0160714285714286*G0_1_1_2_2_1 - 0.20625*G0_1_1_2_2_2;
    A[59] = A[93] - 0.216964285714286*G0_0_0_0_0_0 - 0.0241071428571428*G0_0_0_0_0_1 - 0.0964285714285713*G0_0_0_0_0_2 - 0.0241071428571428*G0_0_0_0_1_0 + 0.0241071428571428*G0_0_0_0_1_1 - 0.0964285714285713*G0_0_0_0_2_0 - 0.0482142857142857*G0_0_0_0_2_2 - 0.0241071428571428*G0_0_0_1_0_0 + 0.0241071428571428*G0_0_0_1_0_1 + 0.0241071428571429*G0_0_0_1_1_0 + 0.216964285714285*G0_0_0_1_1_1 + 0.0964285714285713*G0_0_0_1_1_2 + 0.0964285714285713*G0_0_0_1_2_1 + 0.0482142857142856*G0_0_0_1_2_2 - 0.0964285714285713*G0_0_0_2_0_0 - 0.0482142857142857*G0_0_0_2_0_2 + 0.0964285714285713*G0_0_0_2_1_1 + 0.0482142857142856*G0_0_0_2_1_2 - 0.0482142857142856*G0_0_0_2_2_0 + 0.0482142857142856*G0_0_0_2_2_1 - 0.0843750000000002*G0_0_1_0_0_0 - 0.0241071428571429*G0_0_1_0_0_1 - 0.0120535714285714*G0_0_1_0_0_2 - 0.0241071428571429*G0_0_1_0_1_0 + 0.0120535714285714*G0_0_1_0_1_1 - 0.0120535714285714*G0_0_1_0_2_0 - 0.0241071428571429*G0_0_1_1_0_0 + 0.0120535714285714*G0_0_1_1_0_1 + 0.0120535714285714*G0_0_1_1_1_0 + 0.241071428571428*G0_0_1_1_1_1 + 0.0361607142857142*G0_0_1_1_1_2 + 0.0361607142857142*G0_0_1_1_2_1 - 0.0241071428571429*G0_0_1_1_2_2 - 0.0120535714285714*G0_0_1_2_0_0 + 0.0361607142857142*G0_0_1_2_1_1 - 0.0241071428571429*G0_0_1_2_1_2 - 0.0241071428571429*G0_0_1_2_2_1 - 0.120535714285714*G0_0_1_2_2_2 + 0.024107142857143*G0_1_0_0_0_0 - 0.0120535714285714*G0_1_0_0_0_1 - 0.060267857142857*G0_1_0_0_0_2 - 0.0120535714285714*G0_1_0_0_1_0 - 0.060267857142857*G0_1_0_0_2_0 - 0.0723214285714285*G0_1_0_0_2_2 - 0.0120535714285714*G0_1_0_1_0_0 + 0.132589285714286*G0_1_0_1_1_1 + 0.0843749999999999*G0_1_0_1_1_2 + 0.0843749999999999*G0_1_0_1_2_1 + 0.0482142857142856*G0_1_0_1_2_2 - 0.060267857142857*G0_1_0_2_0_0 - 0.0723214285714285*G0_1_0_2_0_2 + 0.0843749999999999*G0_1_0_2_1_1 + 0.0482142857142856*G0_1_0_2_1_2 - 0.0723214285714285*G0_1_0_2_2_0 + 0.0482142857142856*G0_1_0_2_2_1 - 0.120535714285714*G0_1_0_2_2_2 + 0.204910714285715*G0_1_1_0_0_0 + 0.0241071428571428*G0_1_1_0_0_1 + 0.0120535714285714*G0_1_1_0_0_2 + 0.0241071428571428*G0_1_1_0_1_0 - 0.0361607142857143*G0_1_1_0_1_1 + 0.0120535714285714*G0_1_1_0_2_0 + 0.0241071428571428*G0_1_1_1_0_0 - 0.0361607142857143*G0_1_1_1_0_1 - 0.0361607142857143*G0_1_1_1_1_0 - 0.0482142857142861*G0_1_1_1_1_1 + 0.0120535714285714*G0_1_1_1_1_2 + 0.0120535714285714*G0_1_1_1_2_1 - 0.0241071428571429*G0_1_1_1_2_2 + 0.0120535714285714*G0_1_1_2_0_0 + 0.0120535714285714*G0_1_1_2_1_1 - 0.0241071428571429*G0_1_1_2_1_2 - 0.0241071428571429*G0_1_1_2_2_1 - 0.120535714285714*G0_1_1_2_2_2;
    A[47] = -A[67] - 0.1265625*G0_1_0_0_0_0 + 0.0100446428571429*G0_1_0_0_0_1 + 0.0100446428571429*G0_1_0_0_1_0 + 0.0261160714285714*G0_1_0_0_1_1 - 0.0060267857142857*G0_1_0_0_1_2 - 0.0060267857142857*G0_1_0_0_2_1 - 0.00401785714285713*G0_1_0_0_2_2 + 0.0100446428571429*G0_1_0_1_0_0 + 0.0261160714285714*G0_1_0_1_0_1 - 0.0060267857142857*G0_1_0_1_0_2 + 0.0261160714285714*G0_1_0_1_1_0 + 0.0180803571428572*G0_1_0_1_1_1 + 0.0120535714285714*G0_1_0_1_1_2 - 0.0060267857142857*G0_1_0_1_2_0 + 0.0120535714285714*G0_1_0_1_2_1 + 0.00401785714285714*G0_1_0_1_2_2 - 0.0060267857142857*G0_1_0_2_0_1 - 0.00401785714285712*G0_1_0_2_0_2 - 0.0060267857142857*G0_1_0_2_1_0 + 0.0120535714285714*G0_1_0_2_1_1 + 0.00401785714285714*G0_1_0_2_1_2 - 0.00401785714285712*G0_1_0_2_2_0 + 0.00401785714285714*G0_1_0_2_2_1 + 0.114508928571429*G0_1_1_0_0_0 + 0.0522321428571429*G0_1_1_0_0_1 - 0.0180803571428571*G0_1_1_0_0_2 + 0.0522321428571429*G0_1_1_0_1_0 + 0.0421875*G0_1_1_0_1_1 - 0.0060267857142857*G0_1_1_0_1_2 - 0.0180803571428571*G0_1_1_0_2_0 - 0.0060267857142857*G0_1_1_0_2_1 - 0.0200892857142857*G0_1_1_0_2_2 + 0.0522321428571429*G0_1_1_1_0_0 + 0.0421875*G0_1_1_1_0_1 - 0.0060267857142857*G0_1_1_1_0_2 + 0.0421875*G0_1_1_1_1_0 + 0.024107142857143*G0_1_1_1_1_1 + 0.0180803571428571*G0_1_1_1_1_2 - 0.0060267857142857*G0_1_1_1_2_0 + 0.0180803571428571*G0_1_1_1_2_1 + 0.00803571428571427*G0_1_1_1_2_2 - 0.0180803571428571*G0_1_1_2_0_0 - 0.0060267857142857*G0_1_1_2_0_1 - 0.0200892857142857*G0_1_1_2_0_2 - 0.0060267857142857*G0_1_1_2_1_0 + 0.0180803571428571*G0_1_1_2_1_1 + 0.00803571428571427*G0_1_1_2_1_2 - 0.0200892857142857*G0_1_1_2_2_0 + 0.00803571428571427*G0_1_1_2_2_1 - 0.0120535714285715*G0_1_1_2_2_2;
    A[91] = 0.0361607142857143*G0_0_0_0_0_0 - 0.00133928571428571*G0_0_0_0_0_1 + 0.01875*G0_0_0_0_0_2 - 0.00133928571428571*G0_0_0_0_1_0 - 0.0107142857142857*G0_0_0_0_1_1 + 0.01875*G0_0_0_0_2_0 + 0.0200892857142857*G0_0_0_0_2_2 - 0.00133928571428571*G0_0_0_1_0_0 - 0.0107142857142857*G0_0_0_1_0_1 - 0.0107142857142857*G0_0_0_1_1_0 - 0.120535714285714*G0_0_0_1_1_1 - 0.01875*G0_0_0_1_1_2 - 0.01875*G0_0_0_1_2_1 + 0.00401785714285713*G0_0_0_1_2_2 + 0.01875*G0_0_0_2_0_0 + 0.0200892857142857*G0_0_0_2_0_2 - 0.01875*G0_0_0_2_1_1 + 0.00401785714285713*G0_0_0_2_1_2 + 0.0200892857142857*G0_0_0_2_2_0 + 0.00401785714285713*G0_0_0_2_2_1 + 0.0482142857142856*G0_0_0_2_2_2 - 0.0120535714285714*G0_1_0_0_0_0 - 0.00535714285714286*G0_1_0_0_0_1 - 0.00133928571428571*G0_1_0_0_0_2 - 0.00535714285714286*G0_1_0_0_1_0 + 0.00803571428571429*G0_1_0_0_1_1 - 0.00133928571428571*G0_1_0_0_2_0 + 0.0013392857142857*G0_1_0_0_2_2 - 0.00535714285714286*G0_1_0_1_0_0 + 0.00803571428571429*G0_1_0_1_0_1 + 0.00803571428571429*G0_1_0_1_1_0 - 0.00803571428571426*G0_1_0_1_1_2 - 0.00803571428571427*G0_1_0_1_2_1 + 0.00535714285714285*G0_1_0_1_2_2 - 0.00133928571428571*G0_1_0_2_0_0 + 0.0013392857142857*G0_1_0_2_0_2 - 0.00803571428571427*G0_1_0_2_1_1 + 0.00535714285714285*G0_1_0_2_1_2 + 0.0013392857142857*G0_1_0_2_2_0 + 0.00535714285714285*G0_1_0_2_2_1 + 0.0120535714285713*G0_1_0_2_2_2;
    A[90] = -A[91] - 0.084375*G0_0_0_0_0_0 - 0.0120535714285714*G0_0_0_0_0_1 - 0.0120535714285714*G0_0_0_0_1_0 - 0.0120535714285714*G0_0_0_0_1_1 + 0.0241071428571428*G0_0_0_0_2_2 - 0.0120535714285714*G0_0_0_1_0_0 - 0.0120535714285714*G0_0_0_1_0_1 - 0.0120535714285714*G0_0_0_1_1_0 - 0.0843749999999998*G0_0_0_1_1_1 + 0.0241071428571428*G0_0_0_1_2_2 + 0.0241071428571428*G0_0_0_2_0_2 + 0.0241071428571428*G0_0_0_2_1_2 + 0.0241071428571428*G0_0_0_2_2_0 + 0.0241071428571428*G0_0_0_2_2_1 + 0.0964285714285712*G0_0_0_2_2_2 - 0.120535714285714*G0_0_1_0_0_0 - 0.0107142857142857*G0_0_1_0_0_1 - 0.01875*G0_0_1_0_0_2 - 0.0107142857142857*G0_0_1_0_1_0 - 0.00133928571428572*G0_0_1_0_1_1 - 0.01875*G0_0_1_0_2_0 + 0.00401785714285714*G0_0_1_0_2_2 - 0.0107142857142857*G0_0_1_1_0_0 - 0.00133928571428572*G0_0_1_1_0_1 - 0.00133928571428572*G0_0_1_1_1_0 + 0.0361607142857141*G0_0_1_1_1_1 + 0.0187499999999999*G0_0_1_1_1_2 + 0.0187499999999999*G0_0_1_1_2_1 + 0.0200892857142857*G0_0_1_1_2_2 - 0.01875*G0_0_1_2_0_0 + 0.00401785714285713*G0_0_1_2_0_2 + 0.0187499999999999*G0_0_1_2_1_1 + 0.0200892857142857*G0_0_1_2_1_2 + 0.00401785714285714*G0_0_1_2_2_0 + 0.0200892857142857*G0_0_1_2_2_1 + 0.0482142857142856*G0_0_1_2_2_2 - 0.132589285714286*G0_1_0_0_0_0 - 0.0241071428571429*G0_1_0_0_0_1 - 0.0120535714285714*G0_1_0_0_0_2 - 0.0241071428571429*G0_1_0_0_1_0 + 0.0120535714285714*G0_1_0_0_1_1 - 0.0120535714285714*G0_1_0_0_2_0 - 0.0241071428571429*G0_1_0_1_0_0 + 0.0120535714285714*G0_1_0_1_0_1 + 0.0120535714285714*G0_1_0_1_1_0 + 0.0482142857142858*G0_1_0_1_1_1 + 0.0120535714285714*G0_1_0_1_1_2 + 0.0120535714285714*G0_1_0_1_2_1 + 0.0241071428571428*G0_1_0_1_2_2 - 0.0120535714285714*G0_1_0_2_0_0 + 0.0120535714285714*G0_1_0_2_1_1 + 0.0241071428571428*G0_1_0_2_1_2 + 0.0241071428571428*G0_1_0_2_2_1 + 0.0482142857142856*G0_1_0_2_2_2 - 0.120535714285715*G0_1_1_0_0_0 - 0.01875*G0_1_1_0_0_1 - 0.0107142857142857*G0_1_1_0_0_2 - 0.01875*G0_1_1_0_1_0 + 0.00401785714285711*G0_1_1_0_1_1 - 0.0107142857142857*G0_1_1_0_2_0 - 0.00133928571428571*G0_1_1_0_2_2 - 0.01875*G0_1_1_1_0_0 + 0.00401785714285711*G0_1_1_1_0_1 + 0.00401785714285711*G0_1_1_1_1_0 + 0.0482142857142855*G0_1_1_1_1_1 + 0.0200892857142857*G0_1_1_1_1_2 + 0.0200892857142857*G0_1_1_1_2_1 + 0.01875*G0_1_1_1_2_2 - 0.0107142857142857*G0_1_1_2_0_0 - 0.00133928571428571*G0_1_1_2_0_2 + 0.0200892857142857*G0_1_1_2_1_1 + 0.01875*G0_1_1_2_1_2 - 0.00133928571428571*G0_1_1_2_2_0 + 0.01875*G0_1_1_2_2_1 + 0.0361607142857143*G0_1_1_2_2_2;
    A[19] = A[91] - 0.0120535714285714*G0_0_1_0_0_0 - 0.00535714285714286*G0_0_1_0_0_1 - 0.00133928571428572*G0_0_1_0_0_2 - 0.00535714285714286*G0_0_1_0_1_0 + 0.00803571428571429*G0_0_1_0_1_1 - 0.00133928571428572*G0_0_1_0_2_0 + 0.0013392857142857*G0_0_1_0_2_2 - 0.00535714285714286*G0_0_1_1_0_0 + 0.00803571428571429*G0_0_1_1_0_1 + 0.00803571428571429*G0_0_1_1_1_0 - 0.00803571428571425*G0_0_1_1_1_2 - 0.00803571428571426*G0_0_1_1_2_1 + 0.00535714285714285*G0_0_1_1_2_2 - 0.00133928571428572*G0_0_1_2_0_0 + 0.0013392857142857*G0_0_1_2_0_2 - 0.00803571428571425*G0_0_1_2_1_1 + 0.00535714285714285*G0_0_1_2_1_2 + 0.0013392857142857*G0_0_1_2_2_0 + 0.00535714285714285*G0_0_1_2_2_1 + 0.0120535714285713*G0_0_1_2_2_2 + 0.0120535714285714*G0_1_0_0_0_0 + 0.00535714285714286*G0_1_0_0_0_1 + 0.00133928571428572*G0_1_0_0_0_2 + 0.00535714285714286*G0_1_0_0_1_0 - 0.00803571428571429*G0_1_0_0_1_1 + 0.00133928571428572*G0_1_0_0_2_0 - 0.0013392857142857*G0_1_0_0_2_2 + 0.00535714285714286*G0_1_0_1_0_0 - 0.00803571428571429*G0_1_0_1_0_1 - 0.00803571428571429*G0_1_0_1_1_0 + 0.00803571428571425*G0_1_0_1_1_2 + 0.00803571428571426*G0_1_0_1_2_1 - 0.00535714285714285*G0_1_0_1_2_2 + 0.00133928571428572*G0_1_0_2_0_0 - 0.0013392857142857*G0_1_0_2_0_2 + 0.00803571428571426*G0_1_0_2_1_1 - 0.00535714285714285*G0_1_0_2_1_2 - 0.0013392857142857*G0_1_0_2_2_0 - 0.00535714285714285*G0_1_0_2_2_1 - 0.0120535714285713*G0_1_0_2_2_2;
    A[9] = A[90] - 0.00803571428571431*G0_0_1_0_0_1 + 0.00803571428571427*G0_0_1_0_0_2 - 0.00803571428571431*G0_0_1_0_1_0 + 0.00535714285714284*G0_0_1_0_1_1 + 0.00803571428571427*G0_0_1_0_2_0 - 0.00535714285714285*G0_0_1_0_2_2 - 0.00803571428571431*G0_0_1_1_0_0 + 0.00535714285714284*G0_0_1_1_0_1 + 0.00535714285714284*G0_0_1_1_1_0 + 0.0120535714285714*G0_0_1_1_1_1 + 0.00133928571428572*G0_0_1_1_1_2 + 0.00133928571428573*G0_0_1_1_2_1 - 0.0013392857142857*G0_0_1_1_2_2 + 0.00803571428571428*G0_0_1_2_0_0 - 0.00535714285714284*G0_0_1_2_0_2 + 0.00133928571428573*G0_0_1_2_1_1 - 0.0013392857142857*G0_0_1_2_1_2 - 0.00535714285714285*G0_0_1_2_2_0 - 0.0013392857142857*G0_0_1_2_2_1 - 0.0120535714285713*G0_0_1_2_2_2 + 0.00803571428571431*G0_1_0_0_0_1 - 0.00803571428571427*G0_1_0_0_0_2 + 0.00803571428571431*G0_1_0_0_1_0 - 0.00535714285714284*G0_1_0_0_1_1 - 0.00803571428571427*G0_1_0_0_2_0 + 0.00535714285714285*G0_1_0_0_2_2 + 0.00803571428571431*G0_1_0_1_0_0 - 0.00535714285714284*G0_1_0_1_0_1 - 0.00535714285714284*G0_1_0_1_1_0 - 0.0120535714285714*G0_1_0_1_1_1 - 0.00133928571428572*G0_1_0_1_1_2 - 0.00133928571428573*G0_1_0_1_2_1 + 0.0013392857142857*G0_1_0_1_2_2 - 0.00803571428571427*G0_1_0_2_0_0 + 0.00535714285714284*G0_1_0_2_0_2 - 0.00133928571428573*G0_1_0_2_1_1 + 0.0013392857142857*G0_1_0_2_1_2 + 0.00535714285714285*G0_1_0_2_2_0 + 0.00133928571428569*G0_1_0_2_2_1 + 0.0120535714285713*G0_1_0_2_2_2;
    A[66] = -A[64] - 0.0180803571428573*G0_0_1_0_0_0 - 0.00602678571428575*G0_0_1_0_0_1 - 0.0160714285714285*G0_0_1_0_0_2 - 0.00602678571428575*G0_0_1_0_1_0 - 0.00602678571428575*G0_0_1_0_1_1 - 0.00803571428571427*G0_0_1_0_1_2 - 0.0160714285714285*G0_0_1_0_2_0 - 0.00803571428571427*G0_0_1_0_2_1 - 0.0120535714285713*G0_0_1_0_2_2 - 0.00602678571428575*G0_0_1_1_0_0 - 0.00602678571428575*G0_0_1_1_0_1 - 0.00803571428571427*G0_0_1_1_0_2 - 0.00602678571428575*G0_0_1_1_1_0 - 0.0180803571428573*G0_0_1_1_1_1 - 0.0160714285714286*G0_0_1_1_1_2 - 0.00803571428571427*G0_0_1_1_2_0 - 0.0160714285714286*G0_0_1_1_2_1 - 0.0120535714285713*G0_0_1_1_2_2 - 0.0160714285714285*G0_0_1_2_0_0 - 0.00803571428571427*G0_0_1_2_0_1 - 0.0120535714285713*G0_0_1_2_0_2 - 0.00803571428571427*G0_0_1_2_1_0 - 0.0160714285714286*G0_0_1_2_1_1 - 0.0120535714285713*G0_0_1_2_1_2 - 0.0120535714285713*G0_0_1_2_2_0 - 0.0120535714285713*G0_0_1_2_2_1 + 0.289285714285715*G0_0_1_2_2_2 + 0.247098214285714*G0_1_1_0_0_0 + 0.0602678571428571*G0_1_1_0_0_1 + 0.074330357142857*G0_1_1_0_0_2 + 0.0602678571428571*G0_1_1_0_1_0 + 0.0381696428571428*G0_1_1_0_1_1 + 0.0220982142857143*G0_1_1_0_1_2 + 0.074330357142857*G0_1_1_0_2_0 + 0.0220982142857143*G0_1_1_0_2_1 + 0.0763392857142857*G0_1_1_0_2_2 + 0.0602678571428571*G0_1_1_1_0_0 + 0.0381696428571428*G0_1_1_1_0_1 + 0.0220982142857143*G0_1_1_1_0_2 + 0.0381696428571428*G0_1_1_1_1_0 + 0.0482142857142856*G0_1_1_1_1_1 + 0.0140625*G0_1_1_1_1_2 + 0.0220982142857143*G0_1_1_1_2_0 + 0.0140625*G0_1_1_1_2_1 + 0.0321428571428572*G0_1_1_1_2_2 + 0.074330357142857*G0_1_1_2_0_0 + 0.0220982142857143*G0_1_1_2_0_1 + 0.0763392857142857*G0_1_1_2_0_2 + 0.0220982142857143*G0_1_1_2_1_0 + 0.0140625*G0_1_1_2_1_1 + 0.0321428571428572*G0_1_1_2_1_2 + 0.0763392857142857*G0_1_1_2_2_0 + 0.0321428571428572*G0_1_1_2_2_1 + 0.373660714285715*G0_1_1_2_2_2;
    A[55] = A[66] + 0.229017857142857*G0_0_0_0_0_0 + 0.0361607142857143*G0_0_0_0_0_1 + 0.0964285714285713*G0_0_0_0_0_2 + 0.0361607142857143*G0_0_0_0_1_0 + 0.0120535714285714*G0_0_0_0_1_1 + 0.0120535714285714*G0_0_0_0_1_2 + 0.0964285714285713*G0_0_0_0_2_0 + 0.0120535714285714*G0_0_0_0_2_1 + 0.0361607142857143*G0_0_0_1_0_0 + 0.0120535714285714*G0_0_0_1_0_1 + 0.0120535714285714*G0_0_0_1_0_2 + 0.0120535714285714*G0_0_0_1_1_0 + 0.0120535714285714*G0_0_0_1_1_1 + 0.0120535714285714*G0_0_0_1_2_0 - 0.0482142857142859*G0_0_0_1_2_2 + 0.0964285714285713*G0_0_0_2_0_0 + 0.0120535714285714*G0_0_0_2_0_1 + 0.0120535714285714*G0_0_0_2_1_0 - 0.0482142857142859*G0_0_0_2_1_2 - 0.0482142857142859*G0_0_0_2_2_1 - 0.602678571428572*G0_0_0_2_2_2 - 0.0241071428571428*G0_0_1_0_0_0 + 0.00602678571428571*G0_0_1_0_0_1 + 0.0542410714285713*G0_0_1_0_0_2 + 0.00602678571428571*G0_0_1_0_1_0 + 0.00602678571428571*G0_0_1_0_1_1 + 0.0120535714285714*G0_0_1_0_1_2 + 0.0542410714285713*G0_0_1_0_2_0 + 0.0120535714285714*G0_0_1_0_2_1 + 0.0421874999999998*G0_0_1_0_2_2 + 0.00602678571428571*G0_0_1_1_0_0 + 0.00602678571428571*G0_0_1_1_0_1 + 0.0120535714285714*G0_0_1_1_0_2 + 0.00602678571428571*G0_0_1_1_1_0 + 0.0120535714285714*G0_0_1_1_1_1 + 0.00602678571428568*G0_0_1_1_1_2 + 0.0120535714285714*G0_0_1_1_2_0 + 0.00602678571428568*G0_0_1_1_2_1 - 0.0180803571428573*G0_0_1_1_2_2 + 0.0542410714285713*G0_0_1_2_0_0 + 0.0120535714285714*G0_0_1_2_0_1 + 0.0421874999999998*G0_0_1_2_0_2 + 0.0120535714285714*G0_0_1_2_1_0 + 0.00602678571428567*G0_0_1_2_1_1 - 0.0180803571428573*G0_0_1_2_1_2 + 0.0421874999999998*G0_0_1_2_2_0 - 0.0180803571428573*G0_0_1_2_2_1 - 0.349553571428573*G0_0_1_2_2_2 - 0.0241071428571428*G0_1_0_0_0_0 + 0.00602678571428571*G0_1_0_0_0_1 + 0.0542410714285713*G0_1_0_0_0_2 + 0.00602678571428571*G0_1_0_0_1_0 + 0.00602678571428571*G0_1_0_0_1_1 + 0.0120535714285714*G0_1_0_0_1_2 + 0.0542410714285713*G0_1_0_0_2_0 + 0.0120535714285714*G0_1_0_0_2_1 + 0.0421874999999998*G0_1_0_0_2_2 + 0.00602678571428571*G0_1_0_1_0_0 + 0.00602678571428571*G0_1_0_1_0_1 + 0.0120535714285714*G0_1_0_1_0_2 + 0.00602678571428571*G0_1_0_1_1_0 + 0.0120535714285714*G0_1_0_1_1_1 + 0.00602678571428567*G0_1_0_1_1_2 + 0.0120535714285714*G0_1_0_1_2_0 + 0.00602678571428567*G0_1_0_1_2_1 - 0.0180803571428573*G0_1_0_1_2_2 + 0.0542410714285713*G0_1_0_2_0_0 + 0.0120535714285714*G0_1_0_2_0_1 + 0.0421874999999998*G0_1_0_2_0_2 + 0.0120535714285714*G0_1_0_2_1_0 + 0.00602678571428567*G0_1_0_2_1_1 - 0.0180803571428573*G0_1_0_2_1_2 + 0.0421874999999998*G0_1_0_2_2_0 - 0.0180803571428573*G0_1_0_2_2_1 - 0.349553571428573*G0_1_0_2_2_2 + 0.325446428571428*G0_1_1_0_0_0 + 0.0241071428571428*G0_1_1_0_0_1 + 0.0120535714285713*G0_1_1_0_0_2 + 0.0241071428571428*G0_1_1_0_1_0 + 0.0120535714285713*G0_1_1_0_2_0 - 0.0120535714285716*G0_1_1_0_2_2 + 0.0241071428571428*G0_1_1_1_0_0 - 0.024107142857143*G0_1_1_1_2_2 + 0.0120535714285713*G0_1_1_2_0_0 - 0.0120535714285716*G0_1_1_2_0_2 - 0.024107142857143*G0_1_1_2_1_2 - 0.0120535714285715*G0_1_1_2_2_0 - 0.024107142857143*G0_1_1_2_2_1 - 0.325446428571429*G0_1_1_2_2_2;
    A[56] = A[65] - 0.108482142857143*G0_0_1_0_0_0 - 0.0180803571428571*G0_0_1_0_0_1 - 0.0904017857142856*G0_0_1_0_0_2 - 0.0180803571428571*G0_0_1_0_1_0 - 0.00602678571428571*G0_0_1_0_1_1 - 0.0180803571428571*G0_0_1_0_1_2 - 0.0904017857142856*G0_0_1_0_2_0 - 0.0180803571428571*G0_0_1_0_2_1 - 0.0904017857142855*G0_0_1_0_2_2 - 0.0180803571428571*G0_0_1_1_0_0 - 0.00602678571428571*G0_0_1_1_0_1 - 0.0180803571428571*G0_0_1_1_0_2 - 0.00602678571428571*G0_0_1_1_1_0 - 0.00602678571428568*G0_0_1_1_1_2 - 0.0180803571428571*G0_0_1_1_2_0 - 0.00602678571428568*G0_0_1_1_2_1 - 0.0180803571428571*G0_0_1_1_2_2 - 0.0904017857142856*G0_0_1_2_0_0 - 0.0180803571428571*G0_0_1_2_0_1 - 0.0904017857142856*G0_0_1_2_0_2 - 0.0180803571428571*G0_0_1_2_1_0 - 0.00602678571428569*G0_0_1_2_1_1 - 0.0180803571428571*G0_0_1_2_1_2 - 0.0904017857142856*G0_0_1_2_2_0 - 0.0180803571428571*G0_0_1_2_2_1 - 0.108482142857142*G0_0_1_2_2_2 + 0.108482142857143*G0_1_0_0_0_0 + 0.0180803571428571*G0_1_0_0_0_1 + 0.0904017857142856*G0_1_0_0_0_2 + 0.0180803571428571*G0_1_0_0_1_0 + 0.00602678571428571*G0_1_0_0_1_1 + 0.0180803571428571*G0_1_0_0_1_2 + 0.0904017857142856*G0_1_0_0_2_0 + 0.0180803571428571*G0_1_0_0_2_1 + 0.0904017857142856*G0_1_0_0_2_2 + 0.0180803571428571*G0_1_0_1_0_0 + 0.00602678571428571*G0_1_0_1_0_1 + 0.0180803571428571*G0_1_0_1_0_2 + 0.00602678571428571*G0_1_0_1_1_0 + 0.00602678571428568*G0_1_0_1_1_2 + 0.0180803571428571*G0_1_0_1_2_0 + 0.00602678571428568*G0_1_0_1_2_1 + 0.0180803571428571*G0_1_0_1_2_2 + 0.0904017857142856*G0_1_0_2_0_0 + 0.0180803571428571*G0_1_0_2_0_1 + 0.0904017857142856*G0_1_0_2_0_2 + 0.0180803571428571*G0_1_0_2_1_0 + 0.00602678571428569*G0_1_0_2_1_1 + 0.0180803571428571*G0_1_0_2_1_2 + 0.0904017857142856*G0_1_0_2_2_0 + 0.0180803571428571*G0_1_0_2_2_1 + 0.108482142857142*G0_1_0_2_2_2;
    A[3] = A[30] + 0.00178571428571426*G0_0_1_0_0_1 - 0.0017857142857143*G0_0_1_0_0_2 + 0.00178571428571426*G0_0_1_0_1_0 + 0.000892857142857125*G0_0_1_0_1_1 - 0.000669642857142857*G0_0_1_0_1_2 - 0.0017857142857143*G0_0_1_0_2_0 - 0.000669642857142859*G0_0_1_0_2_1 + 0.000446428571428568*G0_0_1_0_2_2 + 0.00178571428571426*G0_0_1_1_0_0 + 0.000892857142857125*G0_0_1_1_0_1 - 0.000669642857142857*G0_0_1_1_0_2 + 0.000892857142857126*G0_0_1_1_1_0 - 0.0261160714285715*G0_0_1_1_1_1 + 0.00245535714285713*G0_0_1_1_1_2 - 0.000669642857142858*G0_0_1_1_2_0 + 0.00245535714285712*G0_0_1_1_2_1 + 0.00558035714285712*G0_0_1_1_2_2 - 0.0017857142857143*G0_0_1_2_0_0 - 0.000669642857142859*G0_0_1_2_0_1 + 0.000446428571428568*G0_0_1_2_0_2 - 0.000669642857142858*G0_0_1_2_1_0 + 0.00245535714285713*G0_0_1_2_1_1 + 0.00558035714285712*G0_0_1_2_1_2 + 0.000446428571428568*G0_0_1_2_2_0 + 0.00558035714285712*G0_0_1_2_2_1 + 0.00200892857142848*G0_0_1_2_2_2 - 0.00178571428571426*G0_1_0_0_0_1 + 0.0017857142857143*G0_1_0_0_0_2 - 0.00178571428571426*G0_1_0_0_1_0 - 0.000892857142857126*G0_1_0_0_1_1 + 0.000669642857142857*G0_1_0_0_1_2 + 0.0017857142857143*G0_1_0_0_2_0 + 0.000669642857142858*G0_1_0_0_2_1 - 0.000446428571428567*G0_1_0_0_2_2 - 0.00178571428571426*G0_1_0_1_0_0 - 0.000892857142857125*G0_1_0_1_0_1 + 0.000669642857142858*G0_1_0_1_0_2 - 0.000892857142857126*G0_1_0_1_1_0 + 0.0261160714285715*G0_1_0_1_1_1 - 0.00245535714285713*G0_1_0_1_1_2 + 0.000669642857142858*G0_1_0_1_2_0 - 0.00245535714285713*G0_1_0_1_2_1 - 0.00558035714285712*G0_1_0_1_2_2 + 0.0017857142857143*G0_1_0_2_0_0 + 0.000669642857142858*G0_1_0_2_0_1 - 0.000446428571428567*G0_1_0_2_0_2 + 0.000669642857142858*G0_1_0_2_1_0 - 0.00245535714285712*G0_1_0_2_1_1 - 0.00558035714285712*G0_1_0_2_1_2 - 0.000446428571428567*G0_1_0_2_2_0 - 0.00558035714285712*G0_1_0_2_2_1 - 0.00200892857142847*G0_1_0_2_2_2;
    A[8] = -A[3] + 0.141964285714286*G0_0_0_0_0_0 + 0.0075892857142857*G0_0_0_0_0_1 + 0.0191964285714285*G0_0_0_0_0_2 + 0.0075892857142857*G0_0_0_0_1_0 + 0.00803571428571428*G0_0_0_0_1_1 + 0.00379464285714285*G0_0_0_0_1_2 + 0.0191964285714285*G0_0_0_0_2_0 + 0.00379464285714285*G0_0_0_0_2_1 + 0.003125*G0_0_0_0_2_2 + 0.0075892857142857*G0_0_0_1_0_0 + 0.00803571428571428*G0_0_0_1_0_1 + 0.00379464285714285*G0_0_0_1_0_2 + 0.00803571428571428*G0_0_0_1_1_0 + 0.0375*G0_0_0_1_1_1 + 0.00379464285714285*G0_0_0_1_2_0 - 0.00446428571428569*G0_0_0_1_2_2 + 0.0191964285714285*G0_0_0_2_0_0 + 0.00379464285714285*G0_0_0_2_0_1 + 0.003125*G0_0_0_2_0_2 + 0.00379464285714285*G0_0_0_2_1_0 - 0.00446428571428569*G0_0_0_2_1_2 + 0.003125*G0_0_0_2_2_0 - 0.00446428571428569*G0_0_0_2_2_1 - 0.0026785714285713*G0_0_0_2_2_2 + 0.141964285714286*G0_1_0_0_0_0 + 0.00758928571428569*G0_1_0_0_0_1 + 0.0191964285714285*G0_1_0_0_0_2 + 0.00758928571428569*G0_1_0_0_1_0 + 0.00803571428571427*G0_1_0_0_1_1 + 0.00379464285714285*G0_1_0_0_1_2 + 0.0191964285714285*G0_1_0_0_2_0 + 0.00379464285714285*G0_1_0_0_2_1 + 0.003125*G0_1_0_0_2_2 + 0.00758928571428569*G0_1_0_1_0_0 + 0.00803571428571427*G0_1_0_1_0_1 + 0.00379464285714285*G0_1_0_1_0_2 + 0.00803571428571427*G0_1_0_1_1_0 + 0.0375*G0_1_0_1_1_1 + 0.00379464285714285*G0_1_0_1_2_0 - 0.00446428571428569*G0_1_0_1_2_2 + 0.0191964285714285*G0_1_0_2_0_0 + 0.00379464285714285*G0_1_0_2_0_1 + 0.003125*G0_1_0_2_0_2 + 0.00379464285714285*G0_1_0_2_1_0 - 0.00446428571428569*G0_1_0_2_1_2 + 0.003125*G0_1_0_2_2_0 - 0.00446428571428569*G0_1_0_2_2_1 - 0.00267857142857131*G0_1_0_2_2_2;
    A[12] = 0.000446428571428567*G0_0_1_0_0_0 + 0.000595238095238094*G0_0_1_0_0_1 + 0.000595238095238094*G0_0_1_0_0_2 + 0.000595238095238094*G0_0_1_0_1_0 + 0.00372023809523809*G0_0_1_0_1_1 + 0.000520833333333332*G0_0_1_0_1_2 + 0.000595238095238095*G0_0_1_0_2_0 + 0.000520833333333332*G0_0_1_0_2_1 + 0.0037202380952381*G0_0_1_0_2_2 + 0.000595238095238095*G0_0_1_1_0_0 + 0.00372023809523809*G0_0_1_1_0_1 + 0.000520833333333332*G0_0_1_1_0_2 + 0.00372023809523809*G0_0_1_1_1_0 + 0.0285714285714285*G0_0_1_1_1_1 + 0.000148809523809518*G0_0_1_1_1_2 + 0.000520833333333332*G0_0_1_1_2_0 + 0.000148809523809518*G0_0_1_1_2_1 + 0.000148809523809532*G0_0_1_1_2_2 + 0.000595238095238095*G0_0_1_2_0_0 + 0.000520833333333332*G0_0_1_2_0_1 + 0.0037202380952381*G0_0_1_2_0_2 + 0.000520833333333332*G0_0_1_2_1_0 + 0.000148809523809518*G0_0_1_2_1_1 + 0.000148809523809532*G0_0_1_2_1_2 + 0.0037202380952381*G0_0_1_2_2_0 + 0.000148809523809532*G0_0_1_2_2_1 + 0.0285714285714287*G0_0_1_2_2_2;
    A[10] = -A[12] - 0.0285714285714285*G0_0_0_0_0_0 - 0.000148809523809523*G0_0_0_0_0_1 - 0.00372023809523809*G0_0_0_0_0_2 - 0.000148809523809523*G0_0_0_0_1_0 - 0.00014880952380953*G0_0_0_0_1_1 - 0.000520833333333332*G0_0_0_0_1_2 - 0.00372023809523809*G0_0_0_0_2_0 - 0.000520833333333332*G0_0_0_0_2_1 - 0.000595238095238096*G0_0_0_0_2_2 - 0.000148809523809523*G0_0_0_1_0_0 - 0.00014880952380953*G0_0_0_1_0_1 - 0.000520833333333332*G0_0_0_1_0_2 - 0.00014880952380953*G0_0_0_1_1_0 - 0.0285714285714286*G0_0_0_1_1_1 - 0.00372023809523809*G0_0_0_1_1_2 - 0.000520833333333332*G0_0_0_1_2_0 - 0.00372023809523809*G0_0_0_1_2_1 - 0.000595238095238095*G0_0_0_1_2_2 - 0.00372023809523809*G0_0_0_2_0_0 - 0.000520833333333332*G0_0_0_2_0_1 - 0.000595238095238096*G0_0_0_2_0_2 - 0.000520833333333332*G0_0_0_2_1_0 - 0.00372023809523809*G0_0_0_2_1_1 - 0.000595238095238095*G0_0_0_2_1_2 - 0.000595238095238096*G0_0_0_2_2_0 - 0.000595238095238095*G0_0_0_2_2_1 - 0.000446428571428594*G0_0_0_2_2_2 - 0.028125*G0_0_1_0_0_0 + 0.000446428571428572*G0_0_1_0_0_1 - 0.00312499999999999*G0_0_1_0_0_2 + 0.000446428571428572*G0_0_1_0_1_0 + 0.00357142857142856*G0_0_1_0_1_1 - 0.00312499999999999*G0_0_1_0_2_0 + 0.003125*G0_0_1_0_2_2 + 0.000446428571428573*G0_0_1_1_0_0 + 0.00357142857142856*G0_0_1_1_0_1 + 0.00357142857142856*G0_0_1_1_1_0 - 0.00357142857142857*G0_0_1_1_1_2 - 0.00357142857142857*G0_0_1_1_2_1 - 0.000446428571428562*G0_0_1_1_2_2 - 0.00312499999999999*G0_0_1_2_0_0 + 0.003125*G0_0_1_2_0_2 - 0.00357142857142857*G0_0_1_2_1_1 - 0.000446428571428562*G0_0_1_2_1_2 + 0.003125*G0_0_1_2_2_0 - 0.000446428571428562*G0_0_1_2_2_1 + 0.0281250000000001*G0_0_1_2_2_2;
    A[2] = -A[12] - 0.0281249999999999*G0_0_1_0_0_0 - 0.00312499999999999*G0_0_1_0_0_1 + 0.000446428571428575*G0_0_1_0_0_2 - 0.00312499999999999*G0_0_1_0_1_0 + 0.00312499999999999*G0_0_1_0_1_1 + 0.000446428571428575*G0_0_1_0_2_0 + 0.00357142857142857*G0_0_1_0_2_2 - 0.00312499999999999*G0_0_1_1_0_0 + 0.00312499999999999*G0_0_1_1_0_1 + 0.00312499999999999*G0_0_1_1_1_0 + 0.0281249999999999*G0_0_1_1_1_1 - 0.000446428571428582*G0_0_1_1_1_2 - 0.000446428571428582*G0_0_1_1_2_1 - 0.00357142857142858*G0_0_1_1_2_2 + 0.000446428571428576*G0_0_1_2_0_0 + 0.00357142857142857*G0_0_1_2_0_2 - 0.000446428571428582*G0_0_1_2_1_1 - 0.00357142857142858*G0_0_1_2_1_2 + 0.00357142857142857*G0_0_1_2_2_0 - 0.00357142857142858*G0_0_1_2_2_1 - 0.0285714285714285*G0_1_1_0_0_0 - 0.00372023809523809*G0_1_1_0_0_1 - 0.000148809523809521*G0_1_1_0_0_2 - 0.00372023809523809*G0_1_1_0_1_0 - 0.000595238095238096*G0_1_1_0_1_1 - 0.000520833333333334*G0_1_1_0_1_2 - 0.000148809523809521*G0_1_1_0_2_0 - 0.000520833333333333*G0_1_1_0_2_1 - 0.000148809523809536*G0_1_1_0_2_2 - 0.00372023809523809*G0_1_1_1_0_0 - 0.000595238095238096*G0_1_1_1_0_1 - 0.000520833333333334*G0_1_1_1_0_2 - 0.000595238095238096*G0_1_1_1_1_0 - 0.000446428571428581*G0_1_1_1_1_1 - 0.000595238095238101*G0_1_1_1_1_2 - 0.000520833333333334*G0_1_1_1_2_0 - 0.000595238095238101*G0_1_1_1_2_1 - 0.00372023809523811*G0_1_1_1_2_2 - 0.000148809523809522*G0_1_1_2_0_0 - 0.000520833333333333*G0_1_1_2_0_1 - 0.000148809523809536*G0_1_1_2_0_2 - 0.000520833333333334*G0_1_1_2_1_0 - 0.000595238095238101*G0_1_1_2_1_1 - 0.00372023809523811*G0_1_1_2_1_2 - 0.000148809523809536*G0_1_1_2_2_0 - 0.00372023809523811*G0_1_1_2_2_1 - 0.0285714285714287*G0_1_1_2_2_2;
    A[49] = A[94] - 0.0361607142857143*G0_0_1_0_0_0 - 0.0120535714285714*G0_0_1_0_0_1 - 0.0241071428571428*G0_0_1_0_0_2 - 0.0120535714285714*G0_0_1_0_1_0 - 0.0241071428571428*G0_0_1_0_2_0 - 0.0361607142857143*G0_0_1_0_2_2 - 0.0120535714285714*G0_0_1_1_0_0 + 0.0723214285714285*G0_0_1_1_1_1 + 0.0723214285714284*G0_0_1_1_1_2 + 0.0723214285714284*G0_0_1_1_2_1 + 0.0361607142857141*G0_0_1_1_2_2 - 0.0241071428571428*G0_0_1_2_0_0 - 0.0361607142857143*G0_0_1_2_0_2 + 0.0723214285714284*G0_0_1_2_1_1 + 0.0361607142857141*G0_0_1_2_1_2 - 0.0361607142857143*G0_0_1_2_2_0 + 0.0361607142857141*G0_0_1_2_2_1 - 0.144642857142858*G0_0_1_2_2_2 + 0.0361607142857142*G0_1_0_0_0_0 + 0.0120535714285714*G0_1_0_0_0_1 + 0.0241071428571428*G0_1_0_0_0_2 + 0.0120535714285714*G0_1_0_0_1_0 + 0.0241071428571428*G0_1_0_0_2_0 + 0.0361607142857143*G0_1_0_0_2_2 + 0.0120535714285714*G0_1_0_1_0_0 - 0.0723214285714285*G0_1_0_1_1_1 - 0.0723214285714284*G0_1_0_1_1_2 - 0.0723214285714284*G0_1_0_1_2_1 - 0.0361607142857141*G0_1_0_1_2_2 + 0.0241071428571428*G0_1_0_2_0_0 + 0.0361607142857143*G0_1_0_2_0_2 - 0.0723214285714284*G0_1_0_2_1_1 - 0.0361607142857141*G0_1_0_2_1_2 + 0.0361607142857143*G0_1_0_2_2_0 - 0.0361607142857141*G0_1_0_2_2_1 + 0.144642857142858*G0_1_0_2_2_2;
    A[97] = A[49] + 0.204910714285714*G0_0_0_0_0_0 + 0.0120535714285714*G0_0_0_0_0_1 + 0.0241071428571429*G0_0_0_0_0_2 + 0.0120535714285714*G0_0_0_0_1_0 + 0.0241071428571429*G0_0_0_0_2_0 - 0.0361607142857142*G0_0_0_0_2_2 + 0.0120535714285714*G0_0_0_1_0_0 - 0.120535714285714*G0_0_0_1_1_1 - 0.0241071428571427*G0_0_0_1_1_2 - 0.0241071428571427*G0_0_0_1_2_1 + 0.0120535714285715*G0_0_0_1_2_2 + 0.0241071428571429*G0_0_0_2_0_0 - 0.0361607142857142*G0_0_0_2_0_2 - 0.0241071428571427*G0_0_0_2_1_1 + 0.0120535714285715*G0_0_0_2_1_2 - 0.0361607142857142*G0_0_0_2_2_0 + 0.0120535714285715*G0_0_0_2_2_1 - 0.0482142857142852*G0_0_0_2_2_2 - 0.0843749999999999*G0_0_1_0_0_0 - 0.0120535714285714*G0_0_1_0_0_1 - 0.0241071428571428*G0_0_1_0_0_2 - 0.0120535714285714*G0_0_1_0_1_0 - 0.0241071428571428*G0_0_1_0_2_0 + 0.0120535714285715*G0_0_1_0_2_2 - 0.0120535714285714*G0_0_1_1_0_0 - 0.120535714285714*G0_0_1_1_1_1 - 0.0241071428571427*G0_0_1_1_1_2 - 0.0241071428571426*G0_0_1_1_2_1 + 0.0361607142857145*G0_0_1_1_2_2 - 0.0241071428571428*G0_0_1_2_0_0 + 0.0120535714285715*G0_0_1_2_0_2 - 0.0241071428571427*G0_0_1_2_1_1 + 0.0361607142857145*G0_0_1_2_1_2 + 0.0120535714285715*G0_0_1_2_2_0 + 0.0361607142857145*G0_0_1_2_2_1 + 0.241071428571429*G0_0_1_2_2_2 + 0.0241071428571432*G0_1_0_0_0_0 - 0.0602678571428571*G0_1_0_0_0_1 - 0.0120535714285714*G0_1_0_0_0_2 - 0.0602678571428571*G0_1_0_0_1_0 - 0.0723214285714285*G0_1_0_0_1_1 - 0.0120535714285714*G0_1_0_0_2_0 - 0.0602678571428571*G0_1_0_1_0_0 - 0.0723214285714285*G0_1_0_1_0_1 - 0.0723214285714285*G0_1_0_1_1_0 - 0.120535714285714*G0_1_0_1_1_1 + 0.0482142857142858*G0_1_0_1_1_2 + 0.0482142857142858*G0_1_0_1_2_1 + 0.0843749999999999*G0_1_0_1_2_2 - 0.0120535714285714*G0_1_0_2_0_0 + 0.0482142857142858*G0_1_0_2_1_1 + 0.0843749999999999*G0_1_0_2_1_2 + 0.0843749999999999*G0_1_0_2_2_1 + 0.132589285714286*G0_1_0_2_2_2 - 0.216964285714286*G0_1_1_0_0_0 - 0.0964285714285714*G0_1_1_0_0_1 - 0.0241071428571428*G0_1_1_0_0_2 - 0.0964285714285714*G0_1_1_0_1_0 - 0.0482142857142857*G0_1_1_0_1_1 - 0.0241071428571428*G0_1_1_0_2_0 + 0.0241071428571428*G0_1_1_0_2_2 - 0.0964285714285714*G0_1_1_1_0_0 - 0.0482142857142857*G0_1_1_1_0_1 - 0.0482142857142857*G0_1_1_1_1_0 + 0.0482142857142858*G0_1_1_1_1_2 + 0.0482142857142858*G0_1_1_1_2_1 + 0.0964285714285714*G0_1_1_1_2_2 - 0.0241071428571428*G0_1_1_2_0_0 + 0.0241071428571428*G0_1_1_2_0_2 + 0.0482142857142858*G0_1_1_2_1_1 + 0.0964285714285714*G0_1_1_2_1_2 + 0.0241071428571428*G0_1_1_2_2_0 + 0.0964285714285714*G0_1_1_2_2_1 + 0.216964285714286*G0_1_1_2_2_2;
    A[99] = -A[97] + 0.542410714285714*G0_0_0_0_0_0 + 0.0602678571428571*G0_0_0_0_0_1 + 0.241071428571428*G0_0_0_0_0_2 + 0.0602678571428571*G0_0_0_0_1_0 + 0.0482142857142857*G0_0_0_0_1_1 + 0.0482142857142857*G0_0_0_0_1_2 + 0.241071428571428*G0_0_0_0_2_0 + 0.0482142857142857*G0_0_0_0_2_1 + 0.204910714285714*G0_0_0_0_2_2 + 0.0602678571428571*G0_0_0_1_0_0 + 0.0482142857142857*G0_0_0_1_0_1 + 0.0482142857142857*G0_0_0_1_0_2 + 0.0482142857142857*G0_0_0_1_1_0 + 0.216964285714285*G0_0_0_1_1_1 + 0.144642857142857*G0_0_0_1_1_2 + 0.0482142857142857*G0_0_0_1_2_0 + 0.144642857142857*G0_0_0_1_2_1 + 0.156696428571428*G0_0_0_1_2_2 + 0.241071428571428*G0_0_0_2_0_0 + 0.0482142857142857*G0_0_0_2_0_1 + 0.204910714285714*G0_0_0_2_0_2 + 0.0482142857142857*G0_0_0_2_1_0 + 0.144642857142857*G0_0_0_2_1_1 + 0.156696428571428*G0_0_0_2_1_2 + 0.204910714285714*G0_0_0_2_2_0 + 0.156696428571428*G0_0_0_2_2_1 + 0.43392857142857*G0_0_0_2_2_2 - 0.0120535714285713*G0_0_1_0_0_0 - 0.0120535714285714*G0_0_1_0_0_2 + 0.0120535714285714*G0_0_1_0_1_1 + 0.0120535714285714*G0_0_1_0_1_2 - 0.0120535714285714*G0_0_1_0_2_0 + 0.0120535714285714*G0_0_1_0_2_1 + 0.0120535714285714*G0_0_1_1_0_1 + 0.0120535714285714*G0_0_1_1_0_2 + 0.0120535714285714*G0_0_1_1_1_0 + 0.0964285714285714*G0_0_1_1_1_1 + 0.108482142857143*G0_0_1_1_1_2 + 0.0120535714285714*G0_0_1_1_2_0 + 0.108482142857143*G0_0_1_1_2_1 + 0.120535714285714*G0_0_1_1_2_2 - 0.0120535714285714*G0_0_1_2_0_0 + 0.0120535714285714*G0_0_1_2_0_1 + 0.0120535714285714*G0_0_1_2_1_0 + 0.108482142857143*G0_0_1_2_1_1 + 0.120535714285714*G0_0_1_2_1_2 + 0.120535714285714*G0_0_1_2_2_1 + 0.16875*G0_0_1_2_2_2 + 0.132589285714286*G0_1_0_0_0_0 - 0.0361607142857143*G0_1_0_0_0_1 + 0.0241071428571428*G0_1_0_0_0_2 - 0.0361607142857142*G0_1_0_0_1_0 - 0.0602678571428571*G0_1_0_0_1_1 + 0.0120535714285714*G0_1_0_0_1_2 + 0.0241071428571429*G0_1_0_0_2_0 + 0.0120535714285714*G0_1_0_0_2_1 + 0.0241071428571428*G0_1_0_0_2_2 - 0.0361607142857143*G0_1_0_1_0_0 - 0.0602678571428571*G0_1_0_1_0_1 + 0.0120535714285714*G0_1_0_1_0_2 - 0.0602678571428571*G0_1_0_1_1_0 + 0.0241071428571428*G0_1_0_1_1_1 + 0.108482142857143*G0_1_0_1_1_2 + 0.0120535714285714*G0_1_0_1_2_0 + 0.108482142857143*G0_1_0_1_2_1 + 0.132589285714285*G0_1_0_1_2_2 + 0.0241071428571428*G0_1_0_2_0_0 + 0.0120535714285714*G0_1_0_2_0_1 + 0.0241071428571428*G0_1_0_2_0_2 + 0.0120535714285714*G0_1_0_2_1_0 + 0.108482142857143*G0_1_0_2_1_1 + 0.132589285714285*G0_1_0_2_1_2 + 0.0241071428571428*G0_1_0_2_2_0 + 0.132589285714285*G0_1_0_2_2_1 + 0.204910714285714*G0_1_0_2_2_2 + 0.0602678571428575*G0_1_1_0_0_0 + 0.0482142857142859*G0_1_1_0_0_1 + 0.0120535714285714*G0_1_1_0_0_2 + 0.0482142857142859*G0_1_1_0_1_0 + 0.0723214285714287*G0_1_1_0_1_1 + 0.0241071428571428*G0_1_1_0_1_2 + 0.0120535714285715*G0_1_1_0_2_0 + 0.0241071428571429*G0_1_1_0_2_1 + 0.0361607142857142*G0_1_1_0_2_2 + 0.0482142857142859*G0_1_1_1_0_0 + 0.0723214285714286*G0_1_1_1_0_1 + 0.0241071428571429*G0_1_1_1_0_2 + 0.0723214285714286*G0_1_1_1_1_0 + 0.241071428571429*G0_1_1_1_1_1 + 0.120535714285714*G0_1_1_1_1_2 + 0.0241071428571429*G0_1_1_1_2_0 + 0.120535714285714*G0_1_1_1_2_1 + 0.144642857142857*G0_1_1_1_2_2 + 0.0120535714285714*G0_1_1_2_0_0 + 0.0241071428571429*G0_1_1_2_0_1 + 0.0361607142857142*G0_1_1_2_0_2 + 0.0241071428571429*G0_1_1_2_1_0 + 0.120535714285714*G0_1_1_2_1_1 + 0.144642857142857*G0_1_1_2_1_2 + 0.0361607142857142*G0_1_1_2_2_0 + 0.144642857142857*G0_1_1_2_2_1 + 0.277232142857143*G0_1_1_2_2_2;
    A[29] = A[92] + 0.0120535714285714*G0_0_1_0_0_0 + 0.00133928571428571*G0_0_1_0_0_1 + 0.00535714285714285*G0_0_1_0_0_2 + 0.00133928571428571*G0_0_1_0_1_0 - 0.00133928571428572*G0_0_1_0_1_1 + 0.00535714285714285*G0_0_1_0_2_0 - 0.00803571428571426*G0_0_1_0_2_2 + 0.00133928571428571*G0_0_1_1_0_0 - 0.00133928571428572*G0_0_1_1_0_1 - 0.00133928571428572*G0_0_1_1_1_0 - 0.0120535714285714*G0_0_1_1_1_1 - 0.00535714285714284*G0_0_1_1_1_2 - 0.00535714285714284*G0_0_1_1_2_1 + 0.00803571428571429*G0_0_1_1_2_2 + 0.00535714285714285*G0_0_1_2_0_0 - 0.00803571428571426*G0_0_1_2_0_2 - 0.00535714285714284*G0_0_1_2_1_1 + 0.00803571428571429*G0_0_1_2_1_2 - 0.00803571428571426*G0_0_1_2_2_0 + 0.00803571428571429*G0_0_1_2_2_1 - 0.0120535714285714*G0_1_0_0_0_0 - 0.00133928571428571*G0_1_0_0_0_1 - 0.00535714285714285*G0_1_0_0_0_2 - 0.00133928571428571*G0_1_0_0_1_0 + 0.00133928571428572*G0_1_0_0_1_1 - 0.00535714285714285*G0_1_0_0_2_0 + 0.00803571428571426*G0_1_0_0_2_2 - 0.00133928571428571*G0_1_0_1_0_0 + 0.00133928571428572*G0_1_0_1_0_1 + 0.00133928571428572*G0_1_0_1_1_0 + 0.0120535714285714*G0_1_0_1_1_1 + 0.00535714285714284*G0_1_0_1_1_2 + 0.00535714285714284*G0_1_0_1_2_1 - 0.00803571428571429*G0_1_0_1_2_2 - 0.00535714285714285*G0_1_0_2_0_0 + 0.00803571428571426*G0_1_0_2_0_2 + 0.00535714285714284*G0_1_0_2_1_1 - 0.00803571428571429*G0_1_0_2_1_2 + 0.00803571428571426*G0_1_0_2_2_0 - 0.00803571428571429*G0_1_0_2_2_1;
    A[81] = -A[31] - 0.0575892857142857*G0_0_0_0_0_0 - 0.0116071428571428*G0_0_0_0_0_1 - 0.0191964285714285*G0_0_0_0_0_2 - 0.0116071428571428*G0_0_0_0_1_0 - 0.0160714285714285*G0_0_0_0_1_1 - 0.00580357142857141*G0_0_0_0_1_2 - 0.0191964285714285*G0_0_0_0_2_0 - 0.00580357142857141*G0_0_0_0_2_1 - 0.0191964285714285*G0_0_0_0_2_2 - 0.0116071428571428*G0_0_0_1_0_0 - 0.0160714285714285*G0_0_0_1_0_1 - 0.00580357142857141*G0_0_0_1_0_2 - 0.0160714285714285*G0_0_0_1_1_0 - 0.20625*G0_0_0_1_1_1 - 0.0160714285714285*G0_0_0_1_1_2 - 0.00580357142857141*G0_0_0_1_2_0 - 0.0160714285714285*G0_0_0_1_2_1 - 0.0116071428571428*G0_0_0_1_2_2 - 0.0191964285714285*G0_0_0_2_0_0 - 0.00580357142857141*G0_0_0_2_0_1 - 0.0191964285714285*G0_0_0_2_0_2 - 0.00580357142857141*G0_0_0_2_1_0 - 0.0160714285714285*G0_0_0_2_1_1 - 0.0116071428571428*G0_0_0_2_1_2 - 0.0191964285714285*G0_0_0_2_2_0 - 0.0116071428571428*G0_0_0_2_2_1 - 0.0575892857142857*G0_0_0_2_2_2;
    A[44] = -A[64] + 0.0180803571428573*G0_1_0_0_0_0 + 0.00602678571428575*G0_1_0_0_0_1 + 0.0160714285714285*G0_1_0_0_0_2 + 0.00602678571428575*G0_1_0_0_1_0 + 0.00602678571428575*G0_1_0_0_1_1 + 0.00803571428571428*G0_1_0_0_1_2 + 0.0160714285714285*G0_1_0_0_2_0 + 0.00803571428571428*G0_1_0_0_2_1 + 0.0120535714285713*G0_1_0_0_2_2 + 0.00602678571428575*G0_1_0_1_0_0 + 0.00602678571428575*G0_1_0_1_0_1 + 0.00803571428571428*G0_1_0_1_0_2 + 0.00602678571428575*G0_1_0_1_1_0 + 0.0180803571428573*G0_1_0_1_1_1 + 0.0160714285714286*G0_1_0_1_1_2 + 0.00803571428571428*G0_1_0_1_2_0 + 0.0160714285714286*G0_1_0_1_2_1 + 0.0120535714285714*G0_1_0_1_2_2 + 0.0160714285714285*G0_1_0_2_0_0 + 0.00803571428571428*G0_1_0_2_0_1 + 0.0120535714285713*G0_1_0_2_0_2 + 0.00803571428571428*G0_1_0_2_1_0 + 0.0160714285714286*G0_1_0_2_1_1 + 0.0120535714285714*G0_1_0_2_1_2 + 0.0120535714285713*G0_1_0_2_2_0 + 0.0120535714285714*G0_1_0_2_2_1 - 0.289285714285715*G0_1_0_2_2_2 + 0.066294642857143*G0_1_1_0_0_0 + 0.0441964285714286*G0_1_1_0_0_1 + 0.0301339285714285*G0_1_1_0_0_2 + 0.0441964285714286*G0_1_1_0_1_0 + 0.0662946428571428*G0_1_1_0_1_1 + 0.0301339285714285*G0_1_1_0_1_2 + 0.0301339285714285*G0_1_1_0_2_0 + 0.0301339285714285*G0_1_1_0_2_1 + 0.0441964285714284*G0_1_1_0_2_2 + 0.0441964285714286*G0_1_1_1_0_0 + 0.0662946428571428*G0_1_1_1_0_1 + 0.0301339285714285*G0_1_1_1_0_2 + 0.0662946428571428*G0_1_1_1_1_0 + 0.265178571428571*G0_1_1_1_1_1 + 0.0904017857142856*G0_1_1_1_1_2 + 0.0301339285714285*G0_1_1_1_2_0 + 0.0904017857142856*G0_1_1_1_2_1 + 0.088392857142857*G0_1_1_1_2_2 + 0.0301339285714285*G0_1_1_2_0_0 + 0.0301339285714285*G0_1_1_2_0_1 + 0.0441964285714284*G0_1_1_2_0_2 + 0.0301339285714285*G0_1_1_2_1_0 + 0.0904017857142856*G0_1_1_2_1_1 + 0.0883928571428571*G0_1_1_2_1_2 + 0.0441964285714284*G0_1_1_2_2_0 + 0.0883928571428571*G0_1_1_2_2_1 + 0.0843749999999996*G0_1_1_2_2_2;
    A[33] = A[44] + 0.0120535714285714*G0_0_0_0_0_0 + 0.0120535714285714*G0_0_0_0_0_1 + 0.0120535714285714*G0_0_0_0_1_0 + 0.0361607142857142*G0_0_0_0_1_1 + 0.0120535714285714*G0_0_0_0_1_2 + 0.0120535714285714*G0_0_0_0_2_1 - 0.0482142857142858*G0_0_0_0_2_2 + 0.0120535714285714*G0_0_0_1_0_0 + 0.0361607142857142*G0_0_0_1_0_1 + 0.0120535714285714*G0_0_0_1_0_2 + 0.0361607142857142*G0_0_0_1_1_0 + 0.229017857142857*G0_0_0_1_1_1 + 0.0964285714285712*G0_0_0_1_1_2 + 0.0120535714285714*G0_0_0_1_2_0 + 0.0964285714285712*G0_0_0_1_2_1 + 0.0120535714285714*G0_0_0_2_0_1 - 0.0482142857142858*G0_0_0_2_0_2 + 0.0120535714285714*G0_0_0_2_1_0 + 0.0964285714285712*G0_0_0_2_1_1 - 0.0482142857142858*G0_0_0_2_2_0 - 0.602678571428572*G0_0_0_2_2_2 + 0.0060267857142857*G0_0_1_0_0_1 - 0.00602678571428571*G0_0_1_0_0_2 + 0.0060267857142857*G0_0_1_0_1_0 + 0.0301339285714285*G0_0_1_0_1_1 - 0.00602678571428571*G0_0_1_0_2_0 - 0.0301339285714286*G0_0_1_0_2_2 + 0.0060267857142857*G0_0_1_1_0_0 + 0.0301339285714285*G0_0_1_1_0_1 + 0.0301339285714285*G0_0_1_1_1_0 + 0.253124999999999*G0_0_1_1_1_1 + 0.0421874999999998*G0_0_1_1_1_2 + 0.0421874999999998*G0_0_1_1_2_1 - 0.0421875000000001*G0_0_1_1_2_2 - 0.00602678571428571*G0_0_1_2_0_0 - 0.0301339285714286*G0_0_1_2_0_2 + 0.0421874999999998*G0_0_1_2_1_1 - 0.0421875000000001*G0_0_1_2_1_2 - 0.0301339285714286*G0_0_1_2_2_0 - 0.0421875000000001*G0_0_1_2_2_1 - 0.253125*G0_0_1_2_2_2 + 0.0060267857142857*G0_1_0_0_0_1 - 0.00602678571428571*G0_1_0_0_0_2 + 0.0060267857142857*G0_1_0_0_1_0 + 0.0301339285714285*G0_1_0_0_1_1 - 0.00602678571428571*G0_1_0_0_2_0 - 0.0301339285714286*G0_1_0_0_2_2 + 0.0060267857142857*G0_1_0_1_0_0 + 0.0301339285714285*G0_1_0_1_0_1 + 0.0301339285714285*G0_1_0_1_1_0 + 0.253124999999999*G0_1_0_1_1_1 + 0.0421874999999998*G0_1_0_1_1_2 + 0.0421874999999998*G0_1_0_1_2_1 - 0.0421875000000001*G0_1_0_1_2_2 - 0.00602678571428571*G0_1_0_2_0_0 - 0.0301339285714286*G0_1_0_2_0_2 + 0.0421874999999998*G0_1_0_2_1_1 - 0.0421875000000001*G0_1_0_2_1_2 - 0.0301339285714286*G0_1_0_2_2_0 - 0.0421875000000001*G0_1_0_2_2_1 - 0.253125*G0_1_0_2_2_2 - 0.0120535714285715*G0_1_1_0_0_0 - 0.0120535714285714*G0_1_1_0_0_2 + 0.0482142857142856*G0_1_1_0_1_1 - 0.0120535714285714*G0_1_1_0_1_2 - 0.0120535714285714*G0_1_1_0_2_0 - 0.0120535714285714*G0_1_1_0_2_1 - 0.0361607142857142*G0_1_1_0_2_2 + 0.0482142857142856*G0_1_1_1_0_1 - 0.0120535714285714*G0_1_1_1_0_2 + 0.0482142857142856*G0_1_1_1_1_0 + 0.60267857142857*G0_1_1_1_1_1 - 0.0120535714285714*G0_1_1_1_2_0 - 0.0964285714285714*G0_1_1_1_2_2 - 0.0120535714285714*G0_1_1_2_0_0 - 0.0120535714285714*G0_1_1_2_0_1 - 0.0361607142857142*G0_1_1_2_0_2 - 0.0120535714285714*G0_1_1_2_1_0 - 0.0964285714285714*G0_1_1_2_1_2 - 0.0361607142857142*G0_1_1_2_2_0 - 0.0964285714285714*G0_1_1_2_2_1 - 0.229017857142857*G0_1_1_2_2_2;
    A[83] = -A[33] + 0.0662946428571429*G0_0_0_0_0_0 + 0.0301339285714285*G0_0_0_0_0_1 + 0.0441964285714285*G0_0_0_0_0_2 + 0.0301339285714285*G0_0_0_0_1_0 + 0.0441964285714285*G0_0_0_0_1_1 + 0.0301339285714285*G0_0_0_0_1_2 + 0.0441964285714285*G0_0_0_0_2_0 + 0.0301339285714285*G0_0_0_0_2_1 + 0.0662946428571428*G0_0_0_0_2_2 + 0.0301339285714285*G0_0_0_1_0_0 + 0.0441964285714285*G0_0_0_1_0_1 + 0.0301339285714285*G0_0_0_1_0_2 + 0.0441964285714285*G0_0_0_1_1_0 + 0.0843749999999999*G0_0_0_1_1_1 + 0.088392857142857*G0_0_0_1_1_2 + 0.0301339285714285*G0_0_0_1_2_0 + 0.088392857142857*G0_0_0_1_2_1 + 0.0904017857142855*G0_0_0_1_2_2 + 0.0441964285714285*G0_0_0_2_0_0 + 0.0301339285714285*G0_0_0_2_0_1 + 0.0662946428571428*G0_0_0_2_0_2 + 0.0301339285714285*G0_0_0_2_1_0 + 0.088392857142857*G0_0_0_2_1_1 + 0.0904017857142855*G0_0_0_2_1_2 + 0.0662946428571428*G0_0_0_2_2_0 + 0.0904017857142855*G0_0_0_2_2_1 + 0.265178571428571*G0_0_0_2_2_2 + 0.0180803571428572*G0_0_1_0_0_0 + 0.0160714285714286*G0_0_1_0_0_1 + 0.00602678571428572*G0_0_1_0_0_2 + 0.0160714285714286*G0_0_1_0_1_0 + 0.0120535714285715*G0_0_1_0_1_1 + 0.0080357142857143*G0_0_1_0_1_2 + 0.00602678571428572*G0_0_1_0_2_0 + 0.0080357142857143*G0_0_1_0_2_1 + 0.00602678571428569*G0_0_1_0_2_2 + 0.0160714285714286*G0_0_1_1_0_0 + 0.0120535714285715*G0_0_1_1_0_1 + 0.0080357142857143*G0_0_1_1_0_2 + 0.0120535714285715*G0_0_1_1_1_0 - 0.289285714285714*G0_0_1_1_1_1 + 0.0120535714285715*G0_0_1_1_1_2 + 0.0080357142857143*G0_0_1_1_2_0 + 0.0120535714285715*G0_0_1_1_2_1 + 0.0160714285714285*G0_0_1_1_2_2 + 0.00602678571428572*G0_0_1_2_0_0 + 0.0080357142857143*G0_0_1_2_0_1 + 0.00602678571428569*G0_0_1_2_0_2 + 0.0080357142857143*G0_0_1_2_1_0 + 0.0120535714285715*G0_0_1_2_1_1 + 0.0160714285714285*G0_0_1_2_1_2 + 0.00602678571428569*G0_0_1_2_2_0 + 0.0160714285714286*G0_0_1_2_2_1 + 0.018080357142857*G0_0_1_2_2_2;
    A[63] = A[36] + 0.0180803571428571*G0_0_1_0_0_0 + 0.0140625*G0_0_1_0_0_2 - 0.00200892857142858*G0_0_1_0_1_2 + 0.0140625*G0_0_1_0_2_0 - 0.00200892857142858*G0_0_1_0_2_1 + 0.00602678571428561*G0_0_1_0_2_2 - 0.00200892857142858*G0_0_1_1_0_2 + 0.0180803571428572*G0_0_1_1_1_1 + 0.00200892857142855*G0_0_1_1_1_2 - 0.00200892857142858*G0_0_1_1_2_0 + 0.00200892857142855*G0_0_1_1_2_1 + 0.00602678571428562*G0_0_1_1_2_2 + 0.0140625*G0_0_1_2_0_0 - 0.00200892857142858*G0_0_1_2_0_1 + 0.00602678571428561*G0_0_1_2_0_2 - 0.00200892857142858*G0_0_1_2_1_0 + 0.00200892857142855*G0_0_1_2_1_1 + 0.00602678571428562*G0_0_1_2_1_2 + 0.00602678571428561*G0_0_1_2_2_0 + 0.00602678571428561*G0_0_1_2_2_1 - 0.108482142857144*G0_0_1_2_2_2 - 0.0180803571428571*G0_1_0_0_0_0 - 0.0140625*G0_1_0_0_0_2 + 0.00200892857142858*G0_1_0_0_1_2 - 0.0140625*G0_1_0_0_2_0 + 0.00200892857142858*G0_1_0_0_2_1 - 0.00602678571428561*G0_1_0_0_2_2 + 0.00200892857142858*G0_1_0_1_0_2 - 0.0180803571428572*G0_1_0_1_1_1 - 0.00200892857142855*G0_1_0_1_1_2 + 0.00200892857142858*G0_1_0_1_2_0 - 0.00200892857142855*G0_1_0_1_2_1 - 0.00602678571428562*G0_1_0_1_2_2 - 0.0140625*G0_1_0_2_0_0 + 0.00200892857142858*G0_1_0_2_0_1 - 0.00602678571428561*G0_1_0_2_0_2 + 0.00200892857142858*G0_1_0_2_1_0 - 0.00200892857142855*G0_1_0_2_1_1 - 0.00602678571428562*G0_1_0_2_1_2 - 0.00602678571428561*G0_1_0_2_2_0 - 0.00602678571428562*G0_1_0_2_2_1 + 0.108482142857144*G0_1_0_2_2_2;
    A[68] = -A[63] + 0.00602678571428572*G0_0_0_0_0_0 + 0.00602678571428574*G0_0_0_0_0_1 + 0.0160714285714286*G0_0_0_0_0_2 + 0.00602678571428574*G0_0_0_0_1_0 + 0.00401785714285716*G0_0_0_0_1_1 + 0.0160714285714286*G0_0_0_0_2_0 + 0.0421875*G0_0_0_0_2_2 + 0.00602678571428574*G0_0_0_1_0_0 + 0.00401785714285716*G0_0_0_1_0_1 + 0.00401785714285716*G0_0_0_1_1_0 - 0.0120535714285715*G0_0_0_1_1_1 - 0.0160714285714286*G0_0_0_1_1_2 - 0.0160714285714286*G0_0_0_1_2_1 - 0.0180803571428571*G0_0_0_1_2_2 + 0.0160714285714286*G0_0_0_2_0_0 + 0.0421875*G0_0_0_2_0_2 - 0.0160714285714286*G0_0_0_2_1_1 - 0.018080357142857*G0_0_0_2_1_2 + 0.0421875*G0_0_0_2_2_0 - 0.018080357142857*G0_0_0_2_2_1 + 0.241071428571429*G0_0_0_2_2_2 - 0.0180803571428572*G0_1_0_0_0_0 - 0.0120535714285714*G0_1_0_0_0_1 - 0.0261160714285714*G0_1_0_0_0_2 - 0.0120535714285714*G0_1_0_0_1_0 - 0.00401785714285714*G0_1_0_0_1_1 + 0.00602678571428571*G0_1_0_0_1_2 - 0.0261160714285714*G0_1_0_0_2_0 + 0.00602678571428571*G0_1_0_0_2_1 - 0.0100446428571428*G0_1_0_0_2_2 - 0.0120535714285714*G0_1_0_1_0_0 - 0.00401785714285714*G0_1_0_1_0_1 + 0.00602678571428571*G0_1_0_1_0_2 - 0.00401785714285714*G0_1_0_1_1_0 + 0.00401785714285715*G0_1_0_1_1_2 + 0.00602678571428571*G0_1_0_1_2_0 + 0.00401785714285716*G0_1_0_1_2_1 - 0.0261160714285714*G0_1_0_2_0_0 + 0.00602678571428571*G0_1_0_2_0_1 - 0.0100446428571428*G0_1_0_2_0_2 + 0.00602678571428571*G0_1_0_2_1_0 + 0.00401785714285715*G0_1_0_2_1_1 - 0.0100446428571428*G0_1_0_2_2_0 + 0.126562500000001*G0_1_0_2_2_2;
    A[48] = -A[68] - 0.0180803571428573*G0_1_0_0_0_0 - 0.0261160714285715*G0_1_0_0_0_1 - 0.0120535714285714*G0_1_0_0_0_2 - 0.0261160714285715*G0_1_0_0_1_0 - 0.0100446428571429*G0_1_0_0_1_1 + 0.00602678571428569*G0_1_0_0_1_2 - 0.0120535714285714*G0_1_0_0_2_0 + 0.00602678571428569*G0_1_0_0_2_1 - 0.00401785714285714*G0_1_0_0_2_2 - 0.0261160714285715*G0_1_0_1_0_0 - 0.0100446428571429*G0_1_0_1_0_1 + 0.00602678571428569*G0_1_0_1_0_2 - 0.0100446428571429*G0_1_0_1_1_0 + 0.1265625*G0_1_0_1_1_1 + 0.00602678571428569*G0_1_0_1_2_0 + 0.00401785714285713*G0_1_0_1_2_2 - 0.0120535714285714*G0_1_0_2_0_0 + 0.00602678571428569*G0_1_0_2_0_1 - 0.00401785714285714*G0_1_0_2_0_2 + 0.00602678571428569*G0_1_0_2_1_0 + 0.00401785714285713*G0_1_0_2_1_2 - 0.00401785714285714*G0_1_0_2_2_0 + 0.00401785714285713*G0_1_0_2_2_1 + 0.00602678571428582*G0_1_1_0_0_0 + 0.0160714285714286*G0_1_1_0_0_1 + 0.00602678571428571*G0_1_1_0_0_2 + 0.0160714285714286*G0_1_1_0_1_0 + 0.0421874999999999*G0_1_1_0_1_1 + 0.00602678571428571*G0_1_1_0_2_0 + 0.00401785714285714*G0_1_1_0_2_2 + 0.0160714285714286*G0_1_1_1_0_0 + 0.0421874999999999*G0_1_1_1_0_1 + 0.0421874999999999*G0_1_1_1_1_0 + 0.241071428571428*G0_1_1_1_1_1 - 0.0180803571428572*G0_1_1_1_1_2 - 0.0180803571428572*G0_1_1_1_2_1 - 0.0160714285714286*G0_1_1_1_2_2 + 0.00602678571428571*G0_1_1_2_0_0 + 0.00401785714285714*G0_1_1_2_0_2 - 0.0180803571428572*G0_1_1_2_1_1 - 0.0160714285714286*G0_1_1_2_1_2 + 0.00401785714285714*G0_1_1_2_2_0 - 0.0160714285714286*G0_1_1_2_2_1 - 0.0120535714285713*G0_1_1_2_2_2;
    A[84] = A[48] - 0.0180803571428571*G0_0_1_0_0_0 - 0.0140625000000001*G0_0_1_0_0_1 - 0.0140625000000001*G0_0_1_0_1_0 - 0.00602678571428579*G0_0_1_0_1_1 + 0.00200892857142856*G0_0_1_0_1_2 + 0.00200892857142856*G0_0_1_0_2_1 - 0.0140625000000001*G0_0_1_1_0_0 - 0.0060267857142858*G0_0_1_1_0_1 + 0.00200892857142856*G0_0_1_1_0_2 - 0.0060267857142858*G0_0_1_1_1_0 + 0.108482142857142*G0_0_1_1_1_1 - 0.00602678571428573*G0_0_1_1_1_2 + 0.00200892857142856*G0_0_1_1_2_0 - 0.00602678571428572*G0_0_1_1_2_1 - 0.00200892857142857*G0_0_1_1_2_2 + 0.00200892857142856*G0_0_1_2_0_1 + 0.00200892857142856*G0_0_1_2_1_0 - 0.00602678571428572*G0_0_1_2_1_1 - 0.00200892857142857*G0_0_1_2_1_2 - 0.00200892857142857*G0_0_1_2_2_1 - 0.018080357142857*G0_0_1_2_2_2 + 0.0180803571428571*G0_1_0_0_0_0 + 0.0140625000000001*G0_1_0_0_0_1 + 0.0140625000000001*G0_1_0_0_1_0 + 0.00602678571428579*G0_1_0_0_1_1 - 0.00200892857142856*G0_1_0_0_1_2 - 0.00200892857142856*G0_1_0_0_2_1 + 0.0140625000000001*G0_1_0_1_0_0 + 0.0060267857142858*G0_1_0_1_0_1 - 0.00200892857142856*G0_1_0_1_0_2 + 0.0060267857142858*G0_1_0_1_1_0 - 0.108482142857142*G0_1_0_1_1_1 + 0.00602678571428573*G0_1_0_1_1_2 - 0.00200892857142856*G0_1_0_1_2_0 + 0.00602678571428573*G0_1_0_1_2_1 + 0.00200892857142857*G0_1_0_1_2_2 - 0.00200892857142856*G0_1_0_2_0_1 - 0.00200892857142856*G0_1_0_2_1_0 + 0.00602678571428572*G0_1_0_2_1_1 + 0.00200892857142857*G0_1_0_2_1_2 + 0.00200892857142857*G0_1_0_2_2_1 + 0.018080357142857*G0_1_0_2_2_2;
    A[0] = A[11] + 0.200892857142857*G0_0_0_0_0_0 + 0.0142857142857143*G0_0_0_0_0_1 + 0.0151785714285714*G0_0_0_0_0_2 + 0.0142857142857143*G0_0_0_0_1_0 - 0.0142857142857143*G0_0_0_0_1_1 + 0.0151785714285714*G0_0_0_0_2_0 + 0.000892857142857139*G0_0_0_0_2_2 + 0.0142857142857143*G0_0_0_1_0_0 - 0.0142857142857143*G0_0_0_1_0_1 - 0.0142857142857143*G0_0_0_1_1_0 - 0.200892857142857*G0_0_0_1_1_1 - 0.0151785714285714*G0_0_0_1_1_2 - 0.0151785714285714*G0_0_0_1_2_1 - 0.00089285714285714*G0_0_0_1_2_2 + 0.0151785714285714*G0_0_0_2_0_0 + 0.000892857142857139*G0_0_0_2_0_2 - 0.0151785714285714*G0_0_0_2_1_1 - 0.00089285714285714*G0_0_0_2_1_2 + 0.000892857142857139*G0_0_0_2_2_0 - 0.000892857142857141*G0_0_0_2_2_1 + 0.213392857142857*G0_0_1_0_0_0 + 0.0193452380952381*G0_0_1_0_0_1 + 0.0193452380952381*G0_0_1_0_0_2 + 0.0193452380952381*G0_0_1_0_1_0 + 0.0050595238095238*G0_0_1_0_1_1 + 0.0025297619047619*G0_0_1_0_1_2 + 0.0193452380952381*G0_0_1_0_2_0 + 0.0025297619047619*G0_0_1_0_2_1 + 0.0050595238095238*G0_0_1_0_2_2 + 0.0193452380952381*G0_0_1_1_0_0 + 0.0050595238095238*G0_0_1_1_0_1 + 0.0025297619047619*G0_0_1_1_0_2 + 0.0050595238095238*G0_0_1_1_1_0 + 0.0125*G0_0_1_1_1_1 + 0.00416666666666666*G0_0_1_1_1_2 + 0.0025297619047619*G0_0_1_1_2_0 + 0.00416666666666666*G0_0_1_1_2_1 + 0.00416666666666666*G0_0_1_1_2_2 + 0.0193452380952381*G0_0_1_2_0_0 + 0.0025297619047619*G0_0_1_2_0_1 + 0.0050595238095238*G0_0_1_2_0_2 + 0.0025297619047619*G0_0_1_2_1_0 + 0.00416666666666666*G0_0_1_2_1_1 + 0.00416666666666666*G0_0_1_2_1_2 + 0.0050595238095238*G0_0_1_2_2_0 + 0.00416666666666666*G0_0_1_2_2_1 + 0.0125*G0_0_1_2_2_2 + 0.213392857142857*G0_1_0_0_0_0 + 0.0193452380952381*G0_1_0_0_0_1 + 0.0193452380952381*G0_1_0_0_0_2 + 0.0193452380952381*G0_1_0_0_1_0 + 0.0050595238095238*G0_1_0_0_1_1 + 0.0025297619047619*G0_1_0_0_1_2 + 0.0193452380952381*G0_1_0_0_2_0 + 0.0025297619047619*G0_1_0_0_2_1 + 0.0050595238095238*G0_1_0_0_2_2 + 0.0193452380952381*G0_1_0_1_0_0 + 0.0050595238095238*G0_1_0_1_0_1 + 0.0025297619047619*G0_1_0_1_0_2 + 0.0050595238095238*G0_1_0_1_1_0 + 0.0125*G0_1_0_1_1_1 + 0.00416666666666666*G0_1_0_1_1_2 + 0.0025297619047619*G0_1_0_1_2_0 + 0.00416666666666666*G0_1_0_1_2_1 + 0.00416666666666666*G0_1_0_1_2_2 + 0.0193452380952381*G0_1_0_2_0_0 + 0.0025297619047619*G0_1_0_2_0_1 + 0.0050595238095238*G0_1_0_2_0_2 + 0.0025297619047619*G0_1_0_2_1_0 + 0.00416666666666666*G0_1_0_2_1_1 + 0.00416666666666666*G0_1_0_2_1_2 + 0.0050595238095238*G0_1_0_2_2_0 + 0.00416666666666666*G0_1_0_2_2_1 + 0.0125*G0_1_0_2_2_2 + 0.213392857142857*G0_1_1_0_0_0 + 0.0193452380952381*G0_1_1_0_0_1 + 0.0193452380952381*G0_1_1_0_0_2 + 0.0193452380952381*G0_1_1_0_1_0 + 0.0050595238095238*G0_1_1_0_1_1 + 0.0025297619047619*G0_1_1_0_1_2 + 0.0193452380952381*G0_1_1_0_2_0 + 0.0025297619047619*G0_1_1_0_2_1 + 0.0050595238095238*G0_1_1_0_2_2 + 0.0193452380952381*G0_1_1_1_0_0 + 0.0050595238095238*G0_1_1_1_0_1 + 0.0025297619047619*G0_1_1_1_0_2 + 0.0050595238095238*G0_1_1_1_1_0 + 0.0125*G0_1_1_1_1_1 + 0.00416666666666666*G0_1_1_1_1_2 + 0.0025297619047619*G0_1_1_1_2_0 + 0.00416666666666666*G0_1_1_1_2_1 + 0.00416666666666666*G0_1_1_1_2_2 + 0.0193452380952381*G0_1_1_2_0_0 + 0.0025297619047619*G0_1_1_2_0_1 + 0.0050595238095238*G0_1_1_2_0_2 + 0.0025297619047619*G0_1_1_2_1_0 + 0.00416666666666666*G0_1_1_2_1_1 + 0.00416666666666666*G0_1_1_2_1_2 + 0.0050595238095238*G0_1_1_2_2_0 + 0.00416666666666666*G0_1_1_2_2_1 + 0.0125*G0_1_1_2_2_2;
    A[79] = A[97] + 0.144642857142857*G0_0_1_0_0_0 - 0.0361607142857143*G0_0_1_0_0_1 + 0.0361607142857142*G0_0_1_0_0_2 - 0.0361607142857143*G0_0_1_0_1_0 - 0.0723214285714286*G0_0_1_0_1_1 + 0.0361607142857142*G0_0_1_0_2_0 + 0.0241071428571428*G0_0_1_0_2_2 - 0.0361607142857143*G0_0_1_1_0_0 - 0.0723214285714286*G0_0_1_1_0_1 - 0.0723214285714286*G0_0_1_1_1_0 - 0.0723214285714286*G0_0_1_1_1_1 + 0.0120535714285714*G0_0_1_1_2_2 + 0.0361607142857142*G0_0_1_2_0_0 + 0.0241071428571428*G0_0_1_2_0_2 + 0.0120535714285714*G0_0_1_2_1_2 + 0.0241071428571428*G0_0_1_2_2_0 + 0.0120535714285714*G0_0_1_2_2_1 + 0.036160714285714*G0_0_1_2_2_2 - 0.144642857142857*G0_1_0_0_0_0 + 0.0361607142857143*G0_1_0_0_0_1 - 0.0361607142857142*G0_1_0_0_0_2 + 0.0361607142857143*G0_1_0_0_1_0 + 0.0723214285714286*G0_1_0_0_1_1 - 0.0361607142857142*G0_1_0_0_2_0 - 0.0241071428571428*G0_1_0_0_2_2 + 0.0361607142857143*G0_1_0_1_0_0 + 0.0723214285714286*G0_1_0_1_0_1 + 0.0723214285714286*G0_1_0_1_1_0 + 0.0723214285714286*G0_1_0_1_1_1 - 0.0120535714285714*G0_1_0_1_2_2 - 0.0361607142857142*G0_1_0_2_0_0 - 0.0241071428571428*G0_1_0_2_0_2 - 0.0120535714285714*G0_1_0_2_1_2 - 0.0241071428571428*G0_1_0_2_2_0 - 0.0120535714285714*G0_1_0_2_2_1 - 0.0361607142857141*G0_1_0_2_2_2;
    A[38] = -A[33] + 0.0662946428571429*G0_0_0_0_0_0 + 0.0301339285714285*G0_0_0_0_0_1 + 0.0441964285714285*G0_0_0_0_0_2 + 0.0301339285714285*G0_0_0_0_1_0 + 0.0441964285714285*G0_0_0_0_1_1 + 0.0301339285714285*G0_0_0_0_1_2 + 0.0441964285714285*G0_0_0_0_2_0 + 0.0301339285714285*G0_0_0_0_2_1 + 0.0662946428571428*G0_0_0_0_2_2 + 0.0301339285714285*G0_0_0_1_0_0 + 0.0441964285714285*G0_0_0_1_0_1 + 0.0301339285714285*G0_0_0_1_0_2 + 0.0441964285714285*G0_0_0_1_1_0 + 0.0843749999999999*G0_0_0_1_1_1 + 0.088392857142857*G0_0_0_1_1_2 + 0.0301339285714285*G0_0_0_1_2_0 + 0.088392857142857*G0_0_0_1_2_1 + 0.0904017857142855*G0_0_0_1_2_2 + 0.0441964285714285*G0_0_0_2_0_0 + 0.0301339285714285*G0_0_0_2_0_1 + 0.0662946428571428*G0_0_0_2_0_2 + 0.0301339285714285*G0_0_0_2_1_0 + 0.088392857142857*G0_0_0_2_1_1 + 0.0904017857142855*G0_0_0_2_1_2 + 0.0662946428571428*G0_0_0_2_2_0 + 0.0904017857142855*G0_0_0_2_2_1 + 0.265178571428571*G0_0_0_2_2_2 + 0.0180803571428571*G0_1_0_0_0_0 + 0.0160714285714286*G0_1_0_0_0_1 + 0.00602678571428572*G0_1_0_0_0_2 + 0.0160714285714286*G0_1_0_0_1_0 + 0.0120535714285715*G0_1_0_0_1_1 + 0.0080357142857143*G0_1_0_0_1_2 + 0.00602678571428572*G0_1_0_0_2_0 + 0.0080357142857143*G0_1_0_0_2_1 + 0.00602678571428569*G0_1_0_0_2_2 + 0.0160714285714286*G0_1_0_1_0_0 + 0.0120535714285715*G0_1_0_1_0_1 + 0.00803571428571429*G0_1_0_1_0_2 + 0.0120535714285715*G0_1_0_1_1_0 - 0.289285714285714*G0_1_0_1_1_1 + 0.0120535714285715*G0_1_0_1_1_2 + 0.0080357142857143*G0_1_0_1_2_0 + 0.0120535714285715*G0_1_0_1_2_1 + 0.0160714285714285*G0_1_0_1_2_2 + 0.00602678571428572*G0_1_0_2_0_0 + 0.0080357142857143*G0_1_0_2_0_1 + 0.00602678571428569*G0_1_0_2_0_2 + 0.0080357142857143*G0_1_0_2_1_0 + 0.0120535714285715*G0_1_0_2_1_1 + 0.0160714285714285*G0_1_0_2_1_2 + 0.00602678571428569*G0_1_0_2_2_0 + 0.0160714285714285*G0_1_0_2_2_1 + 0.018080357142857*G0_1_0_2_2_2;
    A[20] = -A[21] - 0.0281249999999999*G0_1_0_0_0_0 - 0.00312499999999999*G0_1_0_0_0_1 + 0.000446428571428575*G0_1_0_0_0_2 - 0.00312499999999999*G0_1_0_0_1_0 + 0.00312499999999999*G0_1_0_0_1_1 + 0.000446428571428575*G0_1_0_0_2_0 + 0.00357142857142857*G0_1_0_0_2_2 - 0.00312499999999999*G0_1_0_1_0_0 + 0.00312499999999999*G0_1_0_1_0_1 + 0.00312499999999999*G0_1_0_1_1_0 + 0.0281249999999999*G0_1_0_1_1_1 - 0.000446428571428581*G0_1_0_1_1_2 - 0.000446428571428582*G0_1_0_1_2_1 - 0.00357142857142858*G0_1_0_1_2_2 + 0.000446428571428575*G0_1_0_2_0_0 + 0.00357142857142857*G0_1_0_2_0_2 - 0.000446428571428581*G0_1_0_2_1_1 - 0.00357142857142858*G0_1_0_2_1_2 + 0.00357142857142857*G0_1_0_2_2_0 - 0.00357142857142858*G0_1_0_2_2_1 - 0.0285714285714285*G0_1_1_0_0_0 - 0.00372023809523809*G0_1_1_0_0_1 - 0.000148809523809521*G0_1_1_0_0_2 - 0.00372023809523809*G0_1_1_0_1_0 - 0.000595238095238096*G0_1_1_0_1_1 - 0.000520833333333334*G0_1_1_0_1_2 - 0.000148809523809521*G0_1_1_0_2_0 - 0.000520833333333333*G0_1_1_0_2_1 - 0.000148809523809536*G0_1_1_0_2_2 - 0.00372023809523809*G0_1_1_1_0_0 - 0.000595238095238096*G0_1_1_1_0_1 - 0.000520833333333334*G0_1_1_1_0_2 - 0.000595238095238096*G0_1_1_1_1_0 - 0.000446428571428582*G0_1_1_1_1_1 - 0.000595238095238101*G0_1_1_1_1_2 - 0.000520833333333334*G0_1_1_1_2_0 - 0.000595238095238101*G0_1_1_1_2_1 - 0.00372023809523811*G0_1_1_1_2_2 - 0.000148809523809522*G0_1_1_2_0_0 - 0.000520833333333333*G0_1_1_2_0_1 - 0.000148809523809536*G0_1_1_2_0_2 - 0.000520833333333334*G0_1_1_2_1_0 - 0.000595238095238102*G0_1_1_2_1_1 - 0.00372023809523811*G0_1_1_2_1_2 - 0.000148809523809536*G0_1_1_2_2_0 - 0.00372023809523811*G0_1_1_2_2_1 - 0.0285714285714287*G0_1_1_2_2_2;
    A[18] = -A[13] - 0.0575892857142857*G0_0_0_0_0_0 - 0.0116071428571428*G0_0_0_0_0_1 - 0.0191964285714285*G0_0_0_0_0_2 - 0.0116071428571428*G0_0_0_0_1_0 - 0.0160714285714285*G0_0_0_0_1_1 - 0.00580357142857141*G0_0_0_0_1_2 - 0.0191964285714285*G0_0_0_0_2_0 - 0.00580357142857141*G0_0_0_0_2_1 - 0.0191964285714285*G0_0_0_0_2_2 - 0.0116071428571428*G0_0_0_1_0_0 - 0.0160714285714285*G0_0_0_1_0_1 - 0.00580357142857141*G0_0_0_1_0_2 - 0.0160714285714285*G0_0_0_1_1_0 - 0.20625*G0_0_0_1_1_1 - 0.0160714285714285*G0_0_0_1_1_2 - 0.00580357142857141*G0_0_0_1_2_0 - 0.0160714285714285*G0_0_0_1_2_1 - 0.0116071428571428*G0_0_0_1_2_2 - 0.0191964285714285*G0_0_0_2_0_0 - 0.00580357142857141*G0_0_0_2_0_1 - 0.0191964285714285*G0_0_0_2_0_2 - 0.00580357142857141*G0_0_0_2_1_0 - 0.0160714285714285*G0_0_0_2_1_1 - 0.0116071428571428*G0_0_0_2_1_2 - 0.0191964285714285*G0_0_0_2_2_0 - 0.0116071428571428*G0_0_0_2_2_1 - 0.0575892857142857*G0_0_0_2_2_2;
    A[88] = -A[83] + 0.247098214285714*G0_0_0_0_0_0 + 0.0743303571428571*G0_0_0_0_0_1 + 0.0602678571428571*G0_0_0_0_0_2 + 0.0743303571428571*G0_0_0_0_1_0 + 0.0763392857142857*G0_0_0_0_1_1 + 0.0220982142857142*G0_0_0_0_1_2 + 0.0602678571428571*G0_0_0_0_2_0 + 0.0220982142857142*G0_0_0_0_2_1 + 0.0381696428571428*G0_0_0_0_2_2 + 0.0743303571428571*G0_0_0_1_0_0 + 0.0763392857142857*G0_0_0_1_0_1 + 0.0220982142857142*G0_0_0_1_0_2 + 0.0763392857142857*G0_0_0_1_1_0 + 0.373660714285714*G0_0_0_1_1_1 + 0.032142857142857*G0_0_0_1_1_2 + 0.0220982142857142*G0_0_0_1_2_0 + 0.032142857142857*G0_0_0_1_2_1 + 0.0140625*G0_0_0_1_2_2 + 0.0602678571428571*G0_0_0_2_0_0 + 0.0220982142857142*G0_0_0_2_0_1 + 0.0381696428571428*G0_0_0_2_0_2 + 0.0220982142857142*G0_0_0_2_1_0 + 0.032142857142857*G0_0_0_2_1_1 + 0.0140625*G0_0_0_2_1_2 + 0.0381696428571428*G0_0_0_2_2_0 + 0.0140625*G0_0_0_2_2_1 + 0.0482142857142857*G0_0_0_2_2_2 - 0.0180803571428571*G0_1_0_0_0_0 - 0.0160714285714286*G0_1_0_0_0_1 - 0.00602678571428572*G0_1_0_0_0_2 - 0.0160714285714286*G0_1_0_0_1_0 - 0.0120535714285715*G0_1_0_0_1_1 - 0.00803571428571429*G0_1_0_0_1_2 - 0.00602678571428572*G0_1_0_0_2_0 - 0.00803571428571429*G0_1_0_0_2_1 - 0.0060267857142857*G0_1_0_0_2_2 - 0.0160714285714286*G0_1_0_1_0_0 - 0.0120535714285715*G0_1_0_1_0_1 - 0.00803571428571429*G0_1_0_1_0_2 - 0.0120535714285715*G0_1_0_1_1_0 + 0.289285714285714*G0_1_0_1_1_1 - 0.0120535714285714*G0_1_0_1_1_2 - 0.00803571428571429*G0_1_0_1_2_0 - 0.0120535714285715*G0_1_0_1_2_1 - 0.0160714285714285*G0_1_0_1_2_2 - 0.00602678571428572*G0_1_0_2_0_0 - 0.00803571428571429*G0_1_0_2_0_1 - 0.0060267857142857*G0_1_0_2_0_2 - 0.00803571428571429*G0_1_0_2_1_0 - 0.0120535714285714*G0_1_0_2_1_1 - 0.0160714285714285*G0_1_0_2_1_2 - 0.0060267857142857*G0_1_0_2_2_0 - 0.0160714285714285*G0_1_0_2_2_1 - 0.018080357142857*G0_1_0_2_2_2;
    A[77] = A[88] + 0.325446428571428*G0_0_0_0_0_0 + 0.0120535714285714*G0_0_0_0_0_1 + 0.0241071428571428*G0_0_0_0_0_2 + 0.0120535714285714*G0_0_0_0_1_0 - 0.0120535714285714*G0_0_0_0_1_1 + 0.0241071428571428*G0_0_0_0_2_0 + 0.0120535714285714*G0_0_0_1_0_0 - 0.0120535714285714*G0_0_0_1_0_1 - 0.0120535714285714*G0_0_0_1_1_0 - 0.325446428571428*G0_0_0_1_1_1 - 0.0241071428571428*G0_0_0_1_1_2 - 0.0241071428571428*G0_0_0_1_2_1 + 0.0241071428571428*G0_0_0_2_0_0 - 0.0241071428571428*G0_0_0_2_1_1 - 0.0241071428571427*G0_0_1_0_0_0 + 0.0542410714285715*G0_0_1_0_0_1 + 0.00602678571428572*G0_0_1_0_0_2 + 0.0542410714285715*G0_0_1_0_1_0 + 0.0421875*G0_0_1_0_1_1 + 0.0120535714285714*G0_0_1_0_1_2 + 0.00602678571428572*G0_0_1_0_2_0 + 0.0120535714285714*G0_0_1_0_2_1 + 0.0060267857142857*G0_0_1_0_2_2 + 0.0542410714285715*G0_0_1_1_0_0 + 0.0421875*G0_0_1_1_0_1 + 0.0120535714285714*G0_0_1_1_0_2 + 0.0421875*G0_0_1_1_1_0 - 0.349553571428571*G0_0_1_1_1_1 - 0.0180803571428571*G0_0_1_1_1_2 + 0.0120535714285714*G0_0_1_1_2_0 - 0.0180803571428571*G0_0_1_1_2_1 + 0.0060267857142857*G0_0_1_1_2_2 + 0.00602678571428572*G0_0_1_2_0_0 + 0.0120535714285714*G0_0_1_2_0_1 + 0.0060267857142857*G0_0_1_2_0_2 + 0.0120535714285714*G0_0_1_2_1_0 - 0.0180803571428571*G0_0_1_2_1_1 + 0.0060267857142857*G0_0_1_2_1_2 + 0.0060267857142857*G0_0_1_2_2_0 + 0.0060267857142857*G0_0_1_2_2_1 + 0.0120535714285714*G0_0_1_2_2_2 - 0.0241071428571427*G0_1_0_0_0_0 + 0.0542410714285715*G0_1_0_0_0_1 + 0.00602678571428572*G0_1_0_0_0_2 + 0.0542410714285715*G0_1_0_0_1_0 + 0.0421875*G0_1_0_0_1_1 + 0.0120535714285714*G0_1_0_0_1_2 + 0.00602678571428572*G0_1_0_0_2_0 + 0.0120535714285714*G0_1_0_0_2_1 + 0.0060267857142857*G0_1_0_0_2_2 + 0.0542410714285715*G0_1_0_1_0_0 + 0.0421875*G0_1_0_1_0_1 + 0.0120535714285714*G0_1_0_1_0_2 + 0.0421875*G0_1_0_1_1_0 - 0.349553571428571*G0_1_0_1_1_1 - 0.0180803571428571*G0_1_0_1_1_2 + 0.0120535714285714*G0_1_0_1_2_0 - 0.0180803571428571*G0_1_0_1_2_1 + 0.0060267857142857*G0_1_0_1_2_2 + 0.00602678571428572*G0_1_0_2_0_0 + 0.0120535714285714*G0_1_0_2_0_1 + 0.0060267857142857*G0_1_0_2_0_2 + 0.0120535714285714*G0_1_0_2_1_0 - 0.0180803571428571*G0_1_0_2_1_1 + 0.0060267857142857*G0_1_0_2_1_2 + 0.0060267857142857*G0_1_0_2_2_0 + 0.0060267857142857*G0_1_0_2_2_1 + 0.0120535714285714*G0_1_0_2_2_2 + 0.229017857142857*G0_1_1_0_0_0 + 0.0964285714285714*G0_1_1_0_0_1 + 0.0361607142857142*G0_1_1_0_0_2 + 0.0964285714285714*G0_1_1_0_1_0 + 0.0120535714285714*G0_1_1_0_1_2 + 0.0361607142857142*G0_1_1_0_2_0 + 0.0120535714285714*G0_1_1_0_2_1 + 0.0120535714285714*G0_1_1_0_2_2 + 0.0964285714285714*G0_1_1_1_0_0 + 0.0120535714285714*G0_1_1_1_0_2 - 0.602678571428571*G0_1_1_1_1_1 - 0.0482142857142856*G0_1_1_1_1_2 + 0.0120535714285714*G0_1_1_1_2_0 - 0.0482142857142856*G0_1_1_1_2_1 + 0.0361607142857142*G0_1_1_2_0_0 + 0.0120535714285714*G0_1_1_2_0_1 + 0.0120535714285714*G0_1_1_2_0_2 + 0.0120535714285714*G0_1_1_2_1_0 - 0.0482142857142856*G0_1_1_2_1_1 + 0.0120535714285714*G0_1_1_2_2_0 + 0.0120535714285714*G0_1_1_2_2_2;
    A[43] = -A[63] - 0.0180803571428571*G0_1_0_0_0_0 + 0.00602678571428571*G0_1_0_0_0_1 - 0.00803571428571428*G0_1_0_0_0_2 + 0.00602678571428571*G0_1_0_0_1_0 + 0.0180803571428571*G0_1_0_0_1_1 + 0.0200892857142857*G0_1_0_0_1_2 - 0.00803571428571428*G0_1_0_0_2_0 + 0.0200892857142857*G0_1_0_0_2_1 + 0.0120535714285714*G0_1_0_0_2_2 + 0.00602678571428571*G0_1_0_1_0_0 + 0.0180803571428571*G0_1_0_1_0_1 + 0.0200892857142857*G0_1_0_1_0_2 + 0.0180803571428571*G0_1_0_1_1_0 + 0.0904017857142858*G0_1_0_1_1_1 + 0.0883928571428571*G0_1_0_1_1_2 + 0.0200892857142857*G0_1_0_1_2_0 + 0.0883928571428571*G0_1_0_1_2_1 + 0.0843749999999999*G0_1_0_1_2_2 - 0.00803571428571428*G0_1_0_2_0_0 + 0.0200892857142857*G0_1_0_2_0_1 + 0.0120535714285714*G0_1_0_2_0_2 + 0.0200892857142857*G0_1_0_2_1_0 + 0.0883928571428571*G0_1_0_2_1_1 + 0.0843749999999999*G0_1_0_2_1_2 + 0.0120535714285714*G0_1_0_2_2_0 + 0.0843749999999999*G0_1_0_2_2_1 + 0.216964285714286*G0_1_0_2_2_2 - 0.00602678571428579*G0_1_1_0_0_0 - 0.0160714285714286*G0_1_1_0_0_1 - 0.00602678571428569*G0_1_1_0_0_2 - 0.0160714285714286*G0_1_1_0_1_0 - 0.0421874999999999*G0_1_1_0_1_1 - 0.00602678571428569*G0_1_1_0_2_0 - 0.00401785714285712*G0_1_1_0_2_2 - 0.0160714285714286*G0_1_1_1_0_0 - 0.0421874999999999*G0_1_1_1_0_1 - 0.0421874999999999*G0_1_1_1_1_0 - 0.241071428571428*G0_1_1_1_1_1 + 0.0180803571428572*G0_1_1_1_1_2 + 0.0180803571428572*G0_1_1_1_2_1 + 0.0160714285714286*G0_1_1_1_2_2 - 0.00602678571428569*G0_1_1_2_0_0 - 0.00401785714285712*G0_1_1_2_0_2 + 0.0180803571428572*G0_1_1_2_1_1 + 0.0160714285714286*G0_1_1_2_1_2 - 0.00401785714285712*G0_1_1_2_2_0 + 0.0160714285714286*G0_1_1_2_2_1 + 0.0120535714285713*G0_1_1_2_2_2;
    A[95] = A[59] + 0.144642857142857*G0_0_1_0_0_0 + 0.0361607142857143*G0_0_1_0_0_1 - 0.0361607142857142*G0_0_1_0_0_2 + 0.0361607142857143*G0_0_1_0_1_0 + 0.0241071428571429*G0_0_1_0_1_1 - 0.0361607142857142*G0_0_1_0_2_0 - 0.0723214285714284*G0_0_1_0_2_2 + 0.0361607142857143*G0_0_1_1_0_0 + 0.0241071428571429*G0_0_1_1_0_1 + 0.0241071428571429*G0_0_1_1_1_0 + 0.0361607142857143*G0_0_1_1_1_1 + 0.0120535714285714*G0_0_1_1_1_2 + 0.0120535714285714*G0_0_1_1_2_1 - 0.0361607142857142*G0_0_1_2_0_0 - 0.0723214285714284*G0_0_1_2_0_2 + 0.0120535714285714*G0_0_1_2_1_1 - 0.0723214285714284*G0_0_1_2_2_0 - 0.0723214285714283*G0_0_1_2_2_2 - 0.144642857142857*G0_1_0_0_0_0 - 0.0361607142857143*G0_1_0_0_0_1 + 0.0361607142857142*G0_1_0_0_0_2 - 0.0361607142857143*G0_1_0_0_1_0 - 0.0241071428571429*G0_1_0_0_1_1 + 0.0361607142857142*G0_1_0_0_2_0 + 0.0723214285714284*G0_1_0_0_2_2 - 0.0361607142857143*G0_1_0_1_0_0 - 0.0241071428571429*G0_1_0_1_0_1 - 0.0241071428571429*G0_1_0_1_1_0 - 0.0361607142857143*G0_1_0_1_1_1 - 0.0120535714285714*G0_1_0_1_1_2 - 0.0120535714285714*G0_1_0_1_2_1 + 0.0361607142857142*G0_1_0_2_0_0 + 0.0723214285714284*G0_1_0_2_0_2 - 0.0120535714285714*G0_1_0_2_1_1 + 0.0723214285714284*G0_1_0_2_2_0 + 0.0723214285714282*G0_1_0_2_2_2;
    A[52] = -A[32] + 0.0120535714285714*G0_0_1_0_0_0 + 0.00133928571428571*G0_0_1_0_0_1 + 0.00535714285714286*G0_0_1_0_0_2 + 0.00133928571428571*G0_0_1_0_1_0 - 0.00133928571428572*G0_0_1_0_1_1 + 0.00535714285714286*G0_0_1_0_2_0 - 0.00803571428571426*G0_0_1_0_2_2 + 0.00133928571428571*G0_0_1_1_0_0 - 0.00133928571428572*G0_0_1_1_0_1 - 0.00133928571428572*G0_0_1_1_1_0 - 0.0120535714285714*G0_0_1_1_1_1 - 0.00535714285714284*G0_0_1_1_1_2 - 0.00535714285714284*G0_0_1_1_2_1 + 0.0080357142857143*G0_0_1_1_2_2 + 0.00535714285714286*G0_0_1_2_0_0 - 0.00803571428571426*G0_0_1_2_0_2 - 0.00535714285714285*G0_0_1_2_1_1 + 0.0080357142857143*G0_0_1_2_1_2 - 0.00803571428571426*G0_0_1_2_2_0 + 0.0080357142857143*G0_0_1_2_2_1 + 0.0629464285714284*G0_1_1_0_0_0 + 0.0142857142857143*G0_1_1_0_0_1 + 0.00580357142857142*G0_1_1_0_0_2 + 0.0142857142857143*G0_1_1_0_1_0 + 0.0129464285714285*G0_1_1_0_1_1 + 0.00223214285714285*G0_1_1_0_1_2 + 0.00580357142857142*G0_1_1_0_2_0 + 0.00223214285714285*G0_1_1_0_2_1 + 0.00267857142857147*G0_1_1_0_2_2 + 0.0142857142857143*G0_1_1_1_0_0 + 0.0129464285714285*G0_1_1_1_0_1 + 0.00223214285714285*G0_1_1_1_0_2 + 0.0129464285714285*G0_1_1_1_1_0 + 0.050892857142857*G0_1_1_1_1_1 + 0.00044642857142857*G0_1_1_1_1_2 + 0.00223214285714285*G0_1_1_1_2_0 + 0.00044642857142857*G0_1_1_1_2_1 + 0.0107142857142858*G0_1_1_1_2_2 + 0.00580357142857142*G0_1_1_2_0_0 + 0.00223214285714285*G0_1_1_2_0_1 + 0.00267857142857147*G0_1_1_2_0_2 + 0.00223214285714285*G0_1_1_2_1_0 + 0.000446428571428571*G0_1_1_2_1_1 + 0.0107142857142858*G0_1_1_2_1_2 + 0.00267857142857147*G0_1_1_2_2_0 + 0.0107142857142858*G0_1_1_2_2_1 + 0.107142857142858*G0_1_1_2_2_2;
  }

  /// Tabulate the tensor for the contribution from a local cell
  /// using the specified reference cell quadrature points/weights
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c,
                               unsigned int num_quadrature_points,
                               const double * const * quadrature_points,
                               const double* quadrature_weights) const
  {
    throw std::runtime_error("Quadrature version of tabulate_tensor not available when using the FFC tensor representation.");
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class laplacian_f3_p1_q3_tensor_form_0: public ufc::form
{
public:

  /// Constructor
  laplacian_f3_p1_q3_tensor_form_0() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~laplacian_f3_p1_q3_tensor_form_0()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "Form([Integral(Product(IndexSum(Product(Indexed(ComponentTensor(SpatialDerivative(Argument(FiniteElement('Lagrange', Cell('triangle', Space(2)), 3, None), 0), MultiIndex((Index(0),), {Index(0): 2})), MultiIndex((Index(0),), {Index(0): 2})), MultiIndex((Index(1),), {Index(1): 2})), Indexed(ComponentTensor(SpatialDerivative(Argument(FiniteElement('Lagrange', Cell('triangle', Space(2)), 3, None), 1), MultiIndex((Index(2),), {Index(2): 2})), MultiIndex((Index(2),), {Index(2): 2})), MultiIndex((Index(1),), {Index(1): 2}))), MultiIndex((Index(1),), {Index(1): 2})), Product(Coefficient(FiniteElement('Lagrange', Cell('triangle', Space(2)), 1, None), 2), Product(Coefficient(FiniteElement('Lagrange', Cell('triangle', Space(2)), 1, None), 0), Coefficient(FiniteElement('Lagrange', Cell('triangle', Space(2)), 1, None), 1)))), Measure('cell', 0, None))])";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 2;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 3;
  }

  /// Return the number of cell domains
  virtual unsigned int num_cell_domains() const
  {
    return 1;
  }

  /// Return the number of exterior facet domains
  virtual unsigned int num_exterior_facet_domains() const
  {
    return 0;
  }

  /// Return the number of interior facet domains
  virtual unsigned int num_interior_facet_domains() const
  {
    return 0;
  }

  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new laplacian_f3_p1_q3_tensor_finite_element_1();
        break;
      }
    case 1:
      {
        return new laplacian_f3_p1_q3_tensor_finite_element_1();
        break;
      }
    case 2:
      {
        return new laplacian_f3_p1_q3_tensor_finite_element_0();
        break;
      }
    case 3:
      {
        return new laplacian_f3_p1_q3_tensor_finite_element_0();
        break;
      }
    case 4:
      {
        return new laplacian_f3_p1_q3_tensor_finite_element_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new dofmap for argument function i
  virtual ufc::dofmap* create_dofmap(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new laplacian_f3_p1_q3_tensor_dofmap_1();
        break;
      }
    case 1:
      {
        return new laplacian_f3_p1_q3_tensor_dofmap_1();
        break;
      }
    case 2:
      {
        return new laplacian_f3_p1_q3_tensor_dofmap_0();
        break;
      }
    case 3:
      {
        return new laplacian_f3_p1_q3_tensor_dofmap_0();
        break;
      }
    case 4:
      {
        return new laplacian_f3_p1_q3_tensor_dofmap_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new laplacian_f3_p1_q3_tensor_cell_integral_0_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

#endif
